@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -73,14 +72,10 @@ class BondDissociationEnergies(MSONable):
             required_keys.append("pcm_dielectric")
         for key in required_keys:
             if key not in self.molecule_entry:
-                raise RuntimeError(
-                    key + " must be present in molecule entry! Exiting..."
-                )
+                raise RuntimeError(key + " must be present in molecule entry! Exiting...")
             for entry in self.filtered_entries:
                 if key not in entry:
-                    raise RuntimeError(
-                        key + " must be present in all fragment entries! Exiting..."
-                    )
+                    raise RuntimeError(key + " must be present in all fragment entries! Exiting...")
 
         # Define expected charges
         if not allow_additional_charge_separation:
@@ -166,24 +161,18 @@ class BondDissociationEnergies(MSONable):
                     for frag in opened_entries[0]:  # 0 -> no structural change
                         # Since a ring opening still yields a single molecule, it should have the same charge as the
                         # principle:
-                        if (
-                            frag["initial_molecule"]["charge"]
-                            == self.molecule_entry["final_molecule"]["charge"]
-                        ):
+                        if frag["initial_molecule"]["charge"] == self.molecule_entry["final_molecule"]["charge"]:
                             good_entries.append(frag)
                     # If we didn't find any good entries, let's also look at those that exhibit structural changes:
                     if len(good_entries) == 0:
                         for frag in opened_entries[1]:  # 1 -> YES structural change
-                            if (
-                                frag["initial_molecule"]["charge"]
-                                == self.molecule_entry["final_molecule"]["charge"]
-                            ):
+                            if frag["initial_molecule"]["charge"] == self.molecule_entry["final_molecule"]["charge"]:
                                 good_entries.append(frag)
                     # If we still have no good entries, something must have gone wrong with the calculations:
                     if len(good_entries) == 0:
                         bb = BabelMolAdaptor.from_molecule_graph(RO_frag)
                         pbmol = bb.pybel_mol
-                        smiles = pbmol.write(str("smi")).split()[0]
+                        smiles = pbmol.write("smi").split()[0]
                         specie = nx.get_node_attributes(self.mol_graph.graph, "specie")
                         print(
                             "Missing ring opening fragment resulting from the breakage of "
@@ -198,23 +187,15 @@ class BondDissociationEnergies(MSONable):
                             + smiles
                         )
                     elif len(good_entries) == 1:
-                        # If we have only one good entry, format it and addd it to the list that will eventually return:
-                        self.bond_dissociation_energies += [
-                            self.build_new_entry(good_entries, bonds)
-                        ]
+                        # If we have only one good entry, format it and add it to the list that will eventually return:
+                        self.bond_dissociation_energies += [self.build_new_entry(good_entries, bonds)]
                     else:
                         # We shouldn't ever encounter more than one good entry.
-                        raise RuntimeError(
-                            "There should only be one valid ring opening fragment! Exiting..."
-                        )
+                        raise RuntimeError("There should only be one valid ring opening fragment! Exiting...")
             elif len(bonds) == 2:
-                raise RuntimeError(
-                    "Should only be trying to break two bonds if multibreak is true! Exiting..."
-                )
+                raise RuntimeError("Should only be trying to break two bonds if multibreak is true! Exiting...")
             else:
-                print(
-                    "No reason to try and break more than two bonds at once! Exiting..."
-                )
+                print("No reason to try and break more than two bonds at once! Exiting...")
                 raise ValueError
             frag_success = False
         if frag_success:
@@ -249,39 +230,26 @@ class BondDissociationEnergies(MSONable):
                 if len(frag1_charges_found) < len(self.expected_charges):
                     bb = BabelMolAdaptor(frags[0].molecule)
                     pbmol = bb.pybel_mol
-                    smiles = pbmol.write(str("smi")).split()[0]
+                    smiles = pbmol.write("smi").split()[0]
                     for charge in self.expected_charges:
                         if charge not in frag1_charges_found:
-                            print(
-                                "Missing charge "
-                                + str(charge)
-                                + " for fragment "
-                                + smiles
-                            )
+                            print("Missing charge " + str(charge) + " for fragment " + smiles)
                 if len(frag2_charges_found) < len(self.expected_charges):
                     bb = BabelMolAdaptor(frags[1].molecule)
                     pbmol = bb.pybel_mol
-                    smiles = pbmol.write(str("smi")).split()[0]
+                    smiles = pbmol.write("smi").split()[0]
                     for charge in self.expected_charges:
                         if charge not in frag2_charges_found:
-                            print(
-                                "Missing charge "
-                                + str(charge)
-                                + " for fragment "
-                                + smiles
-                            )
+                            print("Missing charge " + str(charge) + " for fragment " + smiles)
                 # Now we attempt to pair fragments with the right total charge, starting with only fragments with no
                 # structural change:
                 for frag1 in frag1_entries[0]:  # 0 -> no structural change
                     for frag2 in frag2_entries[0]:  # 0 -> no structural change
                         if (
-                            frag1["initial_molecule"]["charge"]
-                            + frag2["initial_molecule"]["charge"]
+                            frag1["initial_molecule"]["charge"] + frag2["initial_molecule"]["charge"]
                             == self.molecule_entry["final_molecule"]["charge"]
                         ):
-                            self.bond_dissociation_energies += [
-                                self.build_new_entry([frag1, frag2], bonds)
-                            ]
+                            self.bond_dissociation_energies += [self.build_new_entry([frag1, frag2], bonds)]
                             num_entries_for_this_frag_pair += 1
                 # If we haven't found the number of fragment pairs that we expect, we expand our search to include
                 # fragments that do exhibit structural change:
@@ -289,24 +257,18 @@ class BondDissociationEnergies(MSONable):
                     for frag1 in frag1_entries[0]:  # 0 -> no structural change
                         for frag2 in frag2_entries[1]:  # 1 -> YES structural change
                             if (
-                                frag1["initial_molecule"]["charge"]
-                                + frag2["initial_molecule"]["charge"]
+                                frag1["initial_molecule"]["charge"] + frag2["initial_molecule"]["charge"]
                                 == self.molecule_entry["final_molecule"]["charge"]
                             ):
-                                self.bond_dissociation_energies += [
-                                    self.build_new_entry([frag1, frag2], bonds)
-                                ]
+                                self.bond_dissociation_energies += [self.build_new_entry([frag1, frag2], bonds)]
                                 num_entries_for_this_frag_pair += 1
                     for frag1 in frag1_entries[1]:  # 1 -> YES structural change
                         for frag2 in frag2_entries[0]:  # 0 -> no structural change
                             if (
-                                frag1["initial_molecule"]["charge"]
-                                + frag2["initial_molecule"]["charge"]
+                                frag1["initial_molecule"]["charge"] + frag2["initial_molecule"]["charge"]
                                 == self.molecule_entry["final_molecule"]["charge"]
                             ):
-                                self.bond_dissociation_energies += [
-                                    self.build_new_entry([frag1, frag2], bonds)
-                                ]
+                                self.bond_dissociation_energies += [self.build_new_entry([frag1, frag2], bonds)]
                                 num_entries_for_this_frag_pair += 1
 
     def search_fragment_entries(self, frag):
@@ -323,9 +285,7 @@ class BondDissociationEnergies(MSONable):
         initial_entries = []
         final_entries = []
         for entry in self.filtered_entries:
-            if frag.isomorphic_to(entry["initial_molgraph"]) and frag.isomorphic_to(
-                entry["final_molgraph"]
-            ):
+            if frag.isomorphic_to(entry["initial_molgraph"]) and frag.isomorphic_to(entry["final_molgraph"]):
                 entries += [entry]
             elif frag.isomorphic_to(entry["initial_molgraph"]):
                 initial_entries += [entry]
@@ -366,15 +326,13 @@ class BondDissociationEnergies(MSONable):
             entry["final_molgraph"] = MoleculeGraph.with_local_env_strategy(
                 Molecule.from_dict(entry["final_molecule"]), OpenBabelNN()
             )
-            # Classify any potential structural change that occured during optimization:
+            # Classify any potential structural change that occurred during optimization:
             if entry["initial_molgraph"].isomorphic_to(entry["final_molgraph"]):
                 entry["structure_change"] = "no_change"
             else:
                 initial_graph = entry["initial_molgraph"].graph
                 final_graph = entry["final_molgraph"].graph
-                if nx.is_connected(
-                    initial_graph.to_undirected()
-                ) and not nx.is_connected(final_graph.to_undirected()):
+                if nx.is_connected(initial_graph.to_undirected()) and not nx.is_connected(final_graph.to_undirected()):
                     entry["structure_change"] = "unconnected_fragments"
                 elif final_graph.number_of_edges() < initial_graph.number_of_edges():
                     entry["structure_change"] = "fewer_bonds"
@@ -387,14 +345,9 @@ class BondDissociationEnergies(MSONable):
             for ii, filtered_entry in enumerate(self.filtered_entries):
                 if filtered_entry["formula_pretty"] == entry["formula_pretty"]:
                     if (
-                        filtered_entry["initial_molgraph"].isomorphic_to(
-                            entry["initial_molgraph"]
-                        )
-                        and filtered_entry["final_molgraph"].isomorphic_to(
-                            entry["final_molgraph"]
-                        )
-                        and filtered_entry["initial_molecule"]["charge"]
-                        == entry["initial_molecule"]["charge"]
+                        filtered_entry["initial_molgraph"].isomorphic_to(entry["initial_molgraph"])
+                        and filtered_entry["final_molgraph"].isomorphic_to(entry["final_molgraph"])
+                        and filtered_entry["initial_molecule"]["charge"] == entry["initial_molecule"]["charge"]
                     ):
                         found_similar_entry = True
                         # If two entries are found that pass the above similarity check, take the one with the lower
@@ -418,8 +371,7 @@ class BondDissociationEnergies(MSONable):
         specie = nx.get_node_attributes(self.mol_graph.graph, "specie")
         if len(frags) == 2:
             new_entry = [
-                self.molecule_entry["final_energy"]
-                - (frags[0]["final_energy"] + frags[1]["final_energy"]),
+                self.molecule_entry["final_energy"] - (frags[0]["final_energy"] + frags[1]["final_energy"]),
                 bonds,
                 specie[bonds[0][0]],
                 specie[bonds[0][1]],
