@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -107,7 +106,7 @@ class FreysoldtCorrection(DefectCorrection):
                     initial_defect_structure (Structure) structure corresponding to
                         initial defect supercell structure (uses Lattice for charge correction)
 
-                    defect_frac_sc_coords (3 x 1 array) Fractional co-ordinates of
+                    defect_frac_sc_coords (3 x 1 array) Fractional coordinates of
                         defect location in supercell structure
         Returns:
             FreysoldtCorrection values as a dictionary
@@ -174,7 +173,7 @@ class FreysoldtCorrection(DefectCorrection):
         Return:
             Electrostatic Point Charge contribution to Freysoldt Correction (float)
         """
-        logger.info("Running Freysoldt 2011 PC calculation (should be " "equivalent to sxdefectalign)")
+        logger.info("Running Freysoldt 2011 PC calculation (should be equivalent to sxdefectalign)")
         logger.debug("defect lattice constants are (in angstroms)" + str(lattice.abc))
 
         [a1, a2, a3] = ang_to_bohr * np.array(lattice.get_cartesian_coords(1))
@@ -606,12 +605,12 @@ class KumagaiCorrection(DefectCorrection):
                 "dist_to_defect": dist_to_defect,
             }
 
-            logger.debug("For atom {}\n\tbulk/defect DFT potential difference = " "{}".format(defect_struct_index, Vqb))
-            logger.debug("\tanisotropic model charge: {}".format(Vpc))
-            logger.debug("\t\treciprocal part: {}".format(recip_sum * kumagai_to_V * q))
-            logger.debug("\t\treal part: {}".format(real_sum * kumagai_to_V * q))
-            logger.debug("\t\tself interaction part: {}".format(potential_shift * kumagai_to_V * q))
-            logger.debug("\trelative_vector to defect: {}".format(vec_defect_to_site))
+            logger.debug(f"For atom {defect_struct_index}\n\tbulk/defect DFT potential difference = {Vqb}")
+            logger.debug(f"\tanisotropic model charge: {Vpc}")
+            logger.debug(f"\t\treciprocal part: {recip_sum * kumagai_to_V * q}")
+            logger.debug(f"\t\treal part: {real_sum * kumagai_to_V * q}")
+            logger.debug(f"\t\tself interaction part: {potential_shift * kumagai_to_V * q}")
+            logger.debug(f"\trelative_vector to defect: {vec_defect_to_site}")
 
             if dist_to_defect > sampling_radius:
                 logger.debug(
@@ -629,7 +628,7 @@ class KumagaiCorrection(DefectCorrection):
         if len(for_correction):
             pot_alignment = np.mean(for_correction)
         else:
-            logger.info("No atoms sampled for_correction radius!" " Assigning potential alignment value of 0.")
+            logger.info("No atoms sampled for_correction radius! Assigning potential alignment value of 0.")
             pot_alignment = 0.0
 
         self.metadata["potalign"] = pot_alignment
@@ -828,6 +827,11 @@ class BandFillingCorrection(DefectCorrection):
                         VBM of bulk calculation (or band structure calculation of bulk);
                         calculated on same level of theory as the defect
                         (ex. GGA defects -> requires GGA vbm)
+
+                    run_metadata["defect_incar"] (dict)
+                        Dictionary of INCAR settings for the defect calculation,
+                        required to check if the calculation included spin-orbit coupling
+                        (to determine the spin factor for occupancies of the electron bands)
         Returns:
             Bandfilling Correction value as a dictionary
 
@@ -837,14 +841,15 @@ class BandFillingCorrection(DefectCorrection):
         potalign = entry.parameters["potalign"]
         vbm = entry.parameters["vbm"]
         cbm = entry.parameters["cbm"]
+        soc_calc = entry.parameters["run_metadata"]["defect_incar"].get("LSORBIT")
 
-        bf_corr = self.perform_bandfill_corr(eigenvalues, kpoint_weights, potalign, vbm, cbm)
+        bf_corr = self.perform_bandfill_corr(eigenvalues, kpoint_weights, potalign, vbm, cbm, soc_calc)
 
         entry.parameters["bandfilling_meta"] = dict(self.metadata)
 
         return {"bandfilling_correction": bf_corr}
 
-    def perform_bandfill_corr(self, eigenvalues, kpoint_weights, potalign, vbm, cbm):
+    def perform_bandfill_corr(self, eigenvalues, kpoint_weights, potalign, vbm, cbm, soc_calc=False):
         """
         This calculates the band filling correction based on excess of electrons/holes in CB/VB...
 
@@ -861,15 +866,15 @@ class BandFillingCorrection(DefectCorrection):
         core_occupation_value = list(eigenvalues.values())[0][0][0][1]  # get occupation of a core eigenvalue
         if len(eigenvalues.keys()) == 1:
             # needed because occupation of non-spin calcs is sometimes still 1... should be 2
-            spinfctr = 2.0 if core_occupation_value == 1.0 else 1.0
+            spinfctr = 2.0 if core_occupation_value == 1.0 and not soc_calc else 1.0
         elif len(eigenvalues.keys()) == 2:
             spinfctr = 1.0
         else:
             raise ValueError("Eigenvalue keys greater than 2")
 
         # for tracking mid gap states...
-        shifted_cbm = potalign + cbm  # shift cbm with potential alignment
-        shifted_vbm = potalign + vbm  # shift vbm with potential alignment
+        shifted_cbm = cbm - potalign  # shift cbm with potential alignment
+        shifted_vbm = vbm - potalign  # shift vbm with potential alignment
 
         for spinset in eigenvalues.values():
             for kptset, weight in zip(spinset, kpoint_weights):
