@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -43,7 +42,7 @@ class DefectCompatibility(MSONable):
                     "initial_defect_structure", "defect_frac_sc_coords"]
         kumagai: [ "dielectric", "bulk_atomic_site_averages", "defect_atomic_site_averages",
                    "site_matching_indices", "initial_defect_structure", "defect_frac_sc_coords"]
-        bandfilling: ["eigenvalues", "kpoint_weights", "potalign", "vbm", "cbm"]
+        bandfilling: ["eigenvalues", "kpoint_weights", "potalign", "vbm", "cbm", "run_metadata"]
         bandshifting: ["hybrid_cbm", "hybrid_vbm", "vbm", "cbm"]
         defect relaxation/structure analysis: ["final_defect_structure", "initial_defect_structure",
                                               "sampling_radius", "defect_frac_sc_coords"]
@@ -254,7 +253,7 @@ class DefectCompatibility(MSONable):
             try:
                 defect_entry = self.perform_kumagai(defect_entry)
             except Exception:
-                logger.info("Kumagai correction error occured! Wont perform correction.")
+                logger.info("Kumagai correction error occurred! Wont perform correction.")
 
         # add potalign based on preferred correction setting if it does not already exist in defect entry
         if self.preferred_cc == "freysoldt":
@@ -287,6 +286,7 @@ class DefectCompatibility(MSONable):
             "potalign",
             "vbm",
             "cbm",
+            "run_metadata",
         ]
         run_bandfilling = len(set(defect_entry.parameters.keys()).intersection(required_bandfilling_params)) == len(
             required_bandfilling_params
@@ -604,7 +604,7 @@ class DefectCompatibility(MSONable):
             distdata.append([distance_to_defect, distmatrix[ind, ind], int(ind)])
 
         if defindex is None and not isinstance(defect_entry.defect, Vacancy):
-            raise ValueError("fractional coordinate for defect could not be " "identified in initial_defect_structure")
+            raise ValueError("fractional coordinate for defect could not be identified in initial_defect_structure")
 
         distdata.sort()
         tot_relax_outside_rad = 0.0
