@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -42,6 +41,14 @@ __email__ = "wrichard@mit.edu"
 __status__ = "Beta"
 __date__ = "5/2/13"
 
+warnings.warn(
+    "All code in pymatgen.analysis.diffusion_analyzer has been moved to the separate add-on package"
+    "pymatgen-diffusion, which also includes a lot more functionality for analyzing diffusion"
+    "calculations. This module here is retained for backwards compatibility. It will be removed from"
+    "2022.1.1.",
+    FutureWarning,
+)
+
 
 class DiffusionAnalyzer(MSONable):
     """
@@ -432,7 +439,7 @@ class DiffusionAnalyzer(MSONable):
         from pymatgen.util.plotting import pretty_plot
 
         if self.lattices is not None and len(self.lattices) > 1:
-            warnings.warn("Note the method doesn't apply to NPT-AIMD " "simulation analysis!")
+            warnings.warn("Note the method doesn't apply to NPT-AIMD simulation analysis!")
 
         plt = pretty_plot(12, 8, plt=plt)
         step = (self.corrected_displacements.shape[1] - 1) // (granularity - 1)
@@ -463,7 +470,7 @@ class DiffusionAnalyzer(MSONable):
         plt.plot(plot_dt, rms[:, 0], label="RMS")
         plt.plot(plot_dt, rms[:, 1], label="max")
         plt.legend(loc="best")
-        plt.xlabel("Timestep ({})".format(unit))
+        plt.xlabel(f"Timestep ({unit})")
         plt.ylabel("normalized distance")
         plt.tight_layout()
         return plt
@@ -499,7 +506,7 @@ class DiffusionAnalyzer(MSONable):
         elif mode == "sites":
             for i, site in enumerate(self.structure):
                 sd = self.sq_disp_ions[i, :]
-                plt.plot(plot_dt, sd, label="%s - %d" % (site.specie.__str__(), i))
+                plt.plot(plot_dt, sd, label=f"{site.specie.__str__()} - {i}")
             plt.legend(loc=2, prop={"size": 20})
         elif mode == "mscd":
             plt.plot(plot_dt, self.mscd, "r")
@@ -512,7 +519,7 @@ class DiffusionAnalyzer(MSONable):
             plt.plot(plot_dt, self.msd_components[:, 2], "b")
             plt.legend(["Overall", "a", "b", "c"], loc=2, prop={"size": 20})
 
-        plt.xlabel("Timestep ({})".format(unit))
+        plt.xlabel(f"Timestep ({unit})")
         if mode == "mscd":
             plt.ylabel("MSCD ($\\AA^2$)")
         else:
@@ -551,7 +558,7 @@ class DiffusionAnalyzer(MSONable):
             f.write(delimiter.join(["t", "MSD", "MSD_a", "MSD_b", "MSD_c", "MSCD"]))
             f.write("\n")
             for dt, msd, msdc, mscd in zip(self.dt, self.msd, self.msd_components, self.mscd):
-                f.write(delimiter.join(["%s" % v for v in [dt, msd] + list(msdc) + [mscd]]))
+                f.write(delimiter.join([f"{v}" for v in [dt, msd] + list(msdc) + [mscd]]))
                 f.write("\n")
 
     @classmethod
@@ -655,7 +662,7 @@ class DiffusionAnalyzer(MSONable):
                 # check that the runs are continuous
                 fdist = pbc_diff(vr.initial_structure.frac_coords, final_structure.frac_coords)
                 if np.any(fdist > 0.001):
-                    raise ValueError("initial and final structures do not " "match.")
+                    raise ValueError("initial and final structures do not match.")
                 final_structure = vr.final_structure
 
                 assert (vr.ionic_step_skip or 1) == step_skip
@@ -716,7 +723,7 @@ class DiffusionAnalyzer(MSONable):
                 Examples include smoothed, min_obs, avg_nsteps.
         """
         if ncores is not None and len(filepaths) > 1:
-            p = multiprocessing.Pool(ncores)
+            p = multiprocessing.Pool(ncores)  # pylint: disable=R1732
             vaspruns = p.imap(_get_vasprun, [(fp, step_skip) for fp in filepaths])
             analyzer = cls.from_vaspruns(
                 vaspruns, specie=specie, initial_disp=initial_disp, initial_structure=initial_structure, **kwargs
@@ -920,7 +927,7 @@ def get_arrhenius_plot(temps, diffusivities, diffusivity_errors=None, **kwargs):
     plt.text(
         0.6,
         0.85,
-        "E$_a$ = {:.0f} meV".format(Ea * 1000),
+        f"E$_a$ = {Ea * 1000:.0f} meV",
         fontsize=30,
         transform=plt.axes().transAxes,
     )
