@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -11,11 +10,9 @@ stress-strain data
 
 import itertools
 import warnings
-from collections import OrderedDict
 
 import numpy as np
 import sympy as sp
-from monty.dev import deprecated
 from scipy.integrate import quad
 from scipy.optimize import root
 from scipy.special import factorial
@@ -33,7 +30,7 @@ from pymatgen.core.units import Unit
 
 __author__ = "Joseph Montoya"
 __copyright__ = "Copyright 2012, The Materials Project"
-__credits__ = "Maarten de Jong, Ian Winter, Shyam Dwaraknath, " "Mark Asta, Anubhav Jain"
+__credits__ = "Maarten de Jong, Ian Winter, Shyam Dwaraknath, Mark Asta, Anubhav Jain"
 __version__ = "1.0"
 __maintainer__ = "Joseph Montoya"
 __email__ = "montoyjh@lbl.gov"
@@ -61,7 +58,7 @@ class NthOrderElasticTensor(Tensor):
         if obj.rank % 2 != 0:
             raise ValueError("ElasticTensor must have even rank")
         if not obj.is_voigt_symmetric(tol):
-            warnings.warn("Input elastic tensor does not satisfy " "standard voigt symmetries")
+            warnings.warn("Input elastic tensor does not satisfy standard voigt symmetries")
         return obj.view(cls)
 
     @property
@@ -129,7 +126,7 @@ def raise_error_if_unphysical(f):
 
         """
         if self.k_vrh < 0 or self.g_vrh < 0:
-            raise ValueError("Bulk or shear modulus is negative, property " "cannot be determined")
+            raise ValueError("Bulk or shear modulus is negative, property cannot be determined")
         return f(self, *args, **kwargs)
 
     return wrapper
@@ -270,7 +267,7 @@ class ElasticTensor(NthOrderElasticTensor):
         weight = float(structure.composition.weight)
         mass_density = 1.6605e3 * nsites * weight / (natoms * volume)
         if self.g_vrh < 0:
-            raise ValueError("k_vrh or g_vrh is negative, " "sound velocity is undefined")
+            raise ValueError("k_vrh or g_vrh is negative, sound velocity is undefined")
         return (1e9 * self.g_vrh / mass_density) ** 0.5
 
     @raise_error_if_unphysical
@@ -291,7 +288,7 @@ class ElasticTensor(NthOrderElasticTensor):
         weight = float(structure.composition.weight)
         mass_density = 1.6605e3 * nsites * weight / (natoms * volume)
         if self.g_vrh < 0:
-            raise ValueError("k_vrh or g_vrh is negative, " "sound velocity is undefined")
+            raise ValueError("k_vrh or g_vrh is negative, sound velocity is undefined")
         return (1e9 * (self.k_vrh + 4.0 / 3.0 * self.g_vrh) / mass_density) ** 0.5
 
     @raise_error_if_unphysical
@@ -309,7 +306,7 @@ class ElasticTensor(NthOrderElasticTensor):
         volume = structure.volume
         natoms = structure.composition.num_atoms
         num_density = 1e30 * nsites / volume
-        tot_mass = sum([e.atomic_mass for e in structure.species])
+        tot_mass = sum(e.atomic_mass for e in structure.species)
         avg_mass = 1.6605e-27 * tot_mass / natoms
         return (
             0.38483
@@ -366,7 +363,7 @@ class ElasticTensor(NthOrderElasticTensor):
         """
         nsites = structure.num_sites
         volume = structure.volume
-        tot_mass = sum([e.atomic_mass for e in structure.species])
+        tot_mass = sum(e.atomic_mass for e in structure.species)
         natoms = structure.composition.num_atoms
         weight = float(structure.composition.weight)
         avg_mass = 1.6605e-27 * tot_mass / natoms
@@ -407,17 +404,6 @@ class ElasticTensor(NthOrderElasticTensor):
         td = 1.05457e-34 / 1.38065e-23 * vm * (6 * np.pi ** 2 / v0) ** (1.0 / 3.0)
         return td
 
-    @deprecated(
-        "debye_temperature_from_sound_velocities is now the default"
-        "debye_temperature function, this one will be removed."
-    )
-    @raise_error_if_unphysical
-    def debye_temperature_from_sound_velocities(self, structure):
-        """
-        Estimates Debye temperature from sound velocities
-        """
-        return self.debye_temperature(structure)
-
     @property
     def universal_anisotropy(self):
         """
@@ -529,9 +515,9 @@ class ElasticTensor(NthOrderElasticTensor):
         strain_states = [tuple(ss) for ss in np.eye(6)]
         ss_dict = get_strain_state_dict(strains, stresses, eq_stress=eq_stress)
         if not set(strain_states) <= set(ss_dict.keys()):
-            raise ValueError("Missing independent strain states: " "{}".format(set(strain_states) - set(ss_dict)))
+            raise ValueError(f"Missing independent strain states: {set(strain_states) - set(ss_dict)}")
         if len(set(ss_dict.keys()) - set(strain_states)) > 0:
-            warnings.warn("Extra strain states in strain-stress pairs " "are neglected in independent strain fitting")
+            warnings.warn("Extra strain states in strain-stress pairs are neglected in independent strain fitting")
         c_ij = np.zeros((6, 6))
         for i in range(6):
             istrains = ss_dict[strain_states[i]]["strains"]
@@ -607,13 +593,13 @@ class ElasticTensorExpansion(TensorCollection):
         Calculate's a given elastic tensor's contribution to the
         stress using Einstein summation
         """
-        return sum([c.calculate_stress(strain) for c in self])
+        return sum(c.calculate_stress(strain) for c in self)
 
     def energy_density(self, strain, convert_GPa_to_eV=True):
         """
         Calculates the elastic energy density due to a strain
         """
-        return sum([c.energy_density(strain, convert_GPa_to_eV) for c in self])
+        return sum(c.energy_density(strain, convert_GPa_to_eV) for c in self)
 
     def get_ggt(self, n, u):
         """
@@ -651,7 +637,7 @@ class ElasticTensorExpansion(TensorCollection):
                 to quadpy.sphere.Lebedev(19) as read from file
         """
         if temperature and not structure:
-            raise ValueError("If using temperature input, you must also " "include structure")
+            raise ValueError("If using temperature input, you must also include structure")
 
         quad = quad if quad else DEFAULT_QUAD
         points = quad["points"]
@@ -767,7 +753,7 @@ class ElasticTensorExpansion(TensorCollection):
         """
         # TODO: this might have a general form
         if not self.order <= 4:
-            raise ValueError("Compliance tensor expansion only " "supported for fourth-order and lower")
+            raise ValueError("Compliance tensor expansion only supported for fourth-order and lower")
         ce_exp = [ElasticTensor(self[0]).compliance_tensor]
         einstring = "ijpq,pqrsuv,rskl,uvmn->ijklmn"
         ce_exp.append(np.einsum(einstring, -ce_exp[-1], self[1], ce_exp[-1], ce_exp[-1]))
@@ -983,15 +969,14 @@ def get_strain_state_dict(strains, stresses, eq_stress=None, tol=1e-10, add_eq=T
         sort (bool): flag for whether to sort strain states
 
     Returns:
-        OrderedDict with strain state keys and dictionaries
-        with stress-strain data corresponding to strain state
+        dict: strain state keys and dictionaries with stress-strain data corresponding to strain state
     """
     # Recast stress/strains
     vstrains = np.array([Strain(s).zeroed(tol).voigt for s in strains])  # pylint: disable=E1101
     vstresses = np.array([Stress(s).zeroed(tol).voigt for s in stresses])  # pylint: disable=E1101
     # Collect independent strain states:
     independent = {tuple(np.nonzero(vstrain)[0].tolist()) for vstrain in vstrains}
-    strain_state_dict = OrderedDict()
+    strain_state_dict = {}
     if add_eq:
         if eq_stress is not None:
             veq_stress = Stress(eq_stress).voigt
@@ -1056,7 +1041,7 @@ def generate_pseudo(strain_states, order=3):
             exps /= np.math.factorial(degree - 1)
             sarr[n] = [sp.diff(exp, s, degree - 1) for exp in exps]
         svec = sarr.ravel()
-        present_syms = set.union(*[exp.atoms(sp.Symbol) for exp in svec])
+        present_syms = set.union(*(exp.atoms(sp.Symbol) for exp in svec))
         absent_syms += [set(cvec) - present_syms]
         m = np.zeros((6 * nstates, len(cvec)))
         for n, c in enumerate(cvec):
