@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -7,7 +6,7 @@ This module defines classes to represent all xas and stitching methods
 """
 import math
 import warnings
-from typing import List
+from typing import List, Literal
 
 import numpy as np
 from scipy.interpolate import interp1d
@@ -91,10 +90,10 @@ class XAS(Spectrum):
         self.absorbing_index = absorbing_index
         # check for empty spectra and negative intensities
         if sum(1 for i in self.y if i <= 0) / len(self.y) > 0.05:
-            raise ValueError("Please double check the intensities. " "Most of them are non-positive values. ")
+            raise ValueError("Please double check the intensities. Most of them are non-positive values. ")
 
     def __str__(self):
-        return "%s %s Edge %s for %s: %s" % (
+        return "{} {} Edge {} for {}: {}".format(
             self.absorbing_element,
             self.edge,
             self.spectrum_type,
@@ -102,7 +101,7 @@ class XAS(Spectrum):
             super().__str__(),
         )
 
-    def stitch(self, other: "XAS", num_samples: int = 500, mode: str = "XAFS") -> "XAS":
+    def stitch(self, other: "XAS", num_samples: int = 500, mode: Literal["XAFS", "L23"] = "XAFS") -> "XAS":
         """
         Stitch XAS objects to get the full XAFS spectrum or L23 edge XANES
         spectrum depending on the mode.
@@ -120,8 +119,8 @@ class XAS(Spectrum):
         Args:
             other: Another XAS object.
             num_samples(int): Number of samples for interpolation.
-            mode(str): Either XAFS mode for stitching XANES and EXAFS
-                        or L23 mode for stitching L2 and L3.
+            mode("XAFS" | "L23"): Either XAFS mode for stitching XANES and EXAFS
+                or L23 mode for stitching L2 and L3.
 
         Returns:
             XAS object: The stitched spectrum.
@@ -136,9 +135,9 @@ class XAS(Spectrum):
 
         if mode == "XAFS":
             if not self.edge == other.edge:
-                raise ValueError("Only spectrum with the same absorption " "edge can be stitched in XAFS mode.")
+                raise ValueError("Only spectrum with the same absorption edge can be stitched in XAFS mode.")
             if self.spectrum_type == other.spectrum_type:
-                raise ValueError("Need one XANES and one EXAFS spectrum to " "stitch in XAFS mode")
+                raise ValueError("Need one XANES and one EXAFS spectrum to stitch in XAFS mode")
 
             xanes = self if self.spectrum_type == "XANES" else other
             exafs = self if self.spectrum_type == "EXAFS" else other
@@ -199,13 +198,13 @@ class XAS(Spectrum):
 
         if mode == "L23":
             if self.spectrum_type != "XANES" or other.spectrum_type != "XANES":
-                raise ValueError("Only XANES spectrum can be stitched in " "L23 mode.")
+                raise ValueError("Only XANES spectrum can be stitched in L23 mode.")
             if self.edge not in ["L2", "L3"] or other.edge not in ["L2", "L3"] or self.edge == other.edge:
-                raise ValueError("Need one L2 and one L3 edge spectrum to stitch" "in L23 mode.")
+                raise ValueError("Need one L2 and one L3 edge spectrum to stitch in L23 mode.")
             l2_xanes = self if self.edge == "L2" else other
             l3_xanes = self if self.edge == "L3" else other
             if l2_xanes.absorbing_element.number > 30:
-                raise ValueError("Does not support L2,3-edge XANES for {} element".format(l2_xanes.absorbing_element))
+                raise ValueError(f"Does not support L2,3-edge XANES for {l2_xanes.absorbing_element} element")
 
             l2_f = interp1d(
                 l2_xanes.x,
@@ -221,7 +220,7 @@ class XAS(Spectrum):
             idx = energy.index(min(energy, key=lambda x: (abs(x - l2_xanes.x[0]))))
             if abs(mu[idx] - mu[idx - 1]) / (mu[idx - 1]) > 0.1:
                 warnings.warn(
-                    "There might exist a jump at the L2 and " "L3-edge junction.",
+                    "There might exist a jump at the L2 and L3-edge junction.",
                     UserWarning,
                 )
 
@@ -248,10 +247,10 @@ def site_weighted_spectrum(xas_list: List["XAS"], num_samples: int = 500) -> "XA
         raise ValueError("The input structures mismatch")
     if not len({i.absorbing_element for i in xas_list}) == len({i.edge for i in xas_list}) == 1:
         raise ValueError(
-            "Can only perform site-weighting for spectra with " "same absorbing element and same absorbing edge."
+            "Can only perform site-weighting for spectra with same absorbing element and same absorbing edge."
         )
     if len({i.absorbing_index for i in xas_list}) == 1 or None in {i.absorbing_index for i in xas_list}:
-        raise ValueError("Need at least two site-wise spectra to perform " "site-weighting")
+        raise ValueError("Need at least two site-wise spectra to perform site-weighting")
 
     sa = SpacegroupAnalyzer(groups[0][0])
     ss = sa.get_symmetrized_structure()
