@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -14,9 +13,8 @@ __copyright__ = "Copyright 2012, The Materials Project"
 
 import itertools
 from dataclasses import dataclass
-from typing import Iterable, Dict
+from typing import Dict, Iterable, List, Tuple, Union
 
-from monty.dev import deprecated
 from scipy.constants import N_A
 
 from pymatgen.analysis.phase_diagram import PDEntry, PhaseDiagram
@@ -24,7 +22,7 @@ from pymatgen.apps.battery.battery_abc import AbstractElectrode, AbstractVoltage
 from pymatgen.core.composition import Composition
 from pymatgen.core.periodic_table import Element
 from pymatgen.core.units import Charge, Time
-from pymatgen.entries.computed_entries import ComputedEntry
+from pymatgen.entries.computed_entries import ComputedEntry, ComputedStructureEntry
 
 
 @dataclass
@@ -35,25 +33,32 @@ class InsertionElectrode(AbstractElectrode):
     insertion battery electrode.
     """
 
-    _stable_entries: Iterable[ComputedEntry]
-    _unstable_entries: Iterable[ComputedEntry]
+    stable_entries: Iterable[ComputedEntry]
+    unstable_entries: Iterable[ComputedEntry]
 
     @classmethod
-    def from_entries(cls, entries, working_ion_entry, strip_structures=False):
+    def from_entries(
+        cls,
+        entries: Iterable[Union[ComputedEntry, ComputedStructureEntry]],
+        working_ion_entry: Union[ComputedEntry, ComputedStructureEntry, PDEntry],
+        strip_structures: bool = False,
+    ):
         """
         Create a new InsertionElectrode.
 
         Args:
-            entries: A list of ComputedStructureEntries (or subclasses)
-                representing the different topotactic states of the battery,
-                e.g. TiO2 and LiTiO2.
+            entries: A list of ComputedEntries, ComputedStructureEntries, or
+                subclasses representing the different topotactic states
+                of the battery, e.g. TiO2 and LiTiO2.
             working_ion_entry: A single ComputedEntry or PDEntry
                 representing the element that carries charge across the
                 battery, e.g. Li.
             strip_structures: Since the electrode document only uses volume we can make the
                 electrode object significantly leaner by dropping the structure data.
-                If this parameter is set to True, the ComputedStructureEntry will be replaced
-                with ComputedEntry and the volume will be stored in ComputedEntry.data['volume']
+                If this parameter is set to True, the ComputedStructureEntry will be
+                replaced with a ComputedEntry and the volume will be stored in
+                ComputedEntry.data['volume']. If entries provided are ComputedEntries,
+                must set strip_structures=False.
         """
 
         if strip_structures:
@@ -74,27 +79,30 @@ class InsertionElectrode(AbstractElectrode):
         for entry in entries:
             elements.update(entry.composition.elements)
 
-        # Set an artificial energy for each element for convex hull generation
-        element_energy = max([entry.energy_per_atom for entry in entries]) + 10
+        # Set an artificial high energy for each element for convex hull generation
+        element_energy = max(entry.energy_per_atom for entry in entries) + 10
 
-        pdentries = []
+        pdentries: List[Union[ComputedEntry, ComputedStructureEntry, PDEntry]] = []
         pdentries.extend(entries)
         pdentries.extend([PDEntry(Composition({el: 1}), element_energy) for el in elements])
 
-        # Make phase diagram to determine which entries are stable vs. unstable
+        # Make phase diagram to determine which entries are stable vs. unstable.
+        # For each working ion concentration, we want one stable entry
+        # to use in forming voltage pairs. PhaseDiagram allows for easy comparison
+        # of entry energies.
         pd = PhaseDiagram(pdentries)
 
         def lifrac(e):
             return e.composition.get_atomic_fraction(_working_ion)
 
         # stable entries ordered by amount of Li asc
-        _stable_entries = tuple(sorted([e for e in pd.stable_entries if e in entries], key=lifrac))
+        _stable_entries = tuple(sorted((e for e in pd.stable_entries if e in entries), key=lifrac))
 
         # unstable entries ordered by amount of Li asc
-        _unstable_entries = tuple(sorted([e for e in pd.unstable_entries if e in entries], key=lifrac))
+        _unstable_entries = tuple(sorted((e for e in pd.unstable_entries if e in entries), key=lifrac))
 
         # create voltage pairs
-        _vpairs = tuple(
+        _vpairs: Tuple[AbstractVoltagePair] = tuple(  # type: ignore
             InsertionVoltagePair.from_entries(
                 _stable_entries[i],
                 _stable_entries[i + 1],
@@ -103,12 +111,12 @@ class InsertionElectrode(AbstractElectrode):
             for i in range(len(_stable_entries) - 1)
         )
         framework = _vpairs[0].framework
-        return cls(
+        return cls(  # pylint: disable=E1123
             voltage_pairs=_vpairs,
             working_ion_entry=_working_ion_entry,
-            _stable_entries=_stable_entries,
-            _unstable_entries=_unstable_entries,
-            _framework_formula=framework.reduced_formula,
+            stable_entries=_stable_entries,
+            unstable_entries=_unstable_entries,
+            framework_formula=framework.reduced_formula,
         )
 
     def get_stable_entries(self, charge_to_discharge=True):
@@ -123,7 +131,7 @@ class InsertionElectrode(AbstractElectrode):
             A list of stable entries in the electrode, ordered by amount of the
             working ion.
         """
-        list_copy = list(self._stable_entries)
+        list_copy = list(self.stable_entries)
         return list_copy if charge_to_discharge else list_copy.reverse()
 
     def get_unstable_entries(self, charge_to_discharge=True):
@@ -138,7 +146,7 @@ class InsertionElectrode(AbstractElectrode):
             A list of unstable entries in the electrode, ordered by amount of
             the working ion.
         """
-        list_copy = list(self._unstable_entries)
+        list_copy = list(self.unstable_entries)
         return list_copy if charge_to_discharge else list_copy.reverse()
 
     def get_all_entries(self, charge_to_discharge=True):
@@ -168,14 +176,14 @@ class InsertionElectrode(AbstractElectrode):
         """
         The most charged entry along the topotactic path.
         """
-        return self._stable_entries[0]
+        return self.stable_entries[0]
 
     @property
     def fully_discharged_entry(self):
         """
         The most discharged entry along the topotactic path.
         """
-        return self._stable_entries[-1]
+        return self.stable_entries[-1]
 
     def get_max_instability(self, min_voltage=None, max_voltage=None):
         """
@@ -191,9 +199,9 @@ class InsertionElectrode(AbstractElectrode):
         """
         data = []
         for pair in self._select_in_voltage_range(min_voltage, max_voltage):
-            if pair.decomp_e_charge is not None:
+            if getattr(pair, "decomp_e_charge", None) is not None:
                 data.append(pair.decomp_e_charge)
-            if pair.decomp_e_discharge is not None:
+            if getattr(pair, "decomp_e_discharge", None) is not None:
                 data.append(pair.decomp_e_discharge)
         return max(data) if len(data) > 0 else None
 
@@ -211,9 +219,9 @@ class InsertionElectrode(AbstractElectrode):
         """
         data = []
         for pair in self._select_in_voltage_range(min_voltage, max_voltage):
-            if pair.decomp_e_charge is not None:
+            if getattr(pair, "decomp_e_charge", None) is not None:
                 data.append(pair.decomp_e_charge)
-            if pair.decomp_e_discharge is not None:
+            if getattr(pair, "decomp_e_discharge", None) is not None:
                 data.append(pair.decomp_e_discharge)
         return min(data) if len(data) > 0 else None
 
@@ -358,74 +366,6 @@ class InsertionElectrode(AbstractElectrode):
 
         return d
 
-    @deprecated(
-        replacement=get_summary_dict,
-        message="Name and logic changed, as_dict_summary will be removed in a future release.",
-    )
-    def as_dict_summary(self, print_subelectrodes=True):
-        """
-        Generate a summary dict.
-
-        Args:
-            print_subelectrodes: Also print data on all the possible
-                subelectrodes.
-
-        Returns:
-            A summary of this electrode"s properties in dict format.
-        """
-        chg_comp = self.fully_charged_entry.composition
-        dischg_comp = self.fully_discharged_entry.composition
-
-        ion = self.working_ion
-        d = {
-            "average_voltage": self.get_average_voltage(),
-            "max_voltage": self.max_voltage,
-            "min_voltage": self.min_voltage,
-            "max_delta_volume": self.max_delta_volume,
-            "max_voltage_step": self.max_voltage_step,
-            "capacity_grav": self.get_capacity_grav(),
-            "capacity_vol": self.get_capacity_vol(),
-            "energy_grav": self.get_specific_energy(),
-            "energy_vol": self.get_energy_density(),
-            "working_ion": self.working_ion.symbol,
-            "nsteps": self.num_steps,
-            "framework": self.voltage_pairs[0].framework.to_data_dict,
-            "formula_charge": chg_comp.reduced_formula,
-            "id_charge": self.fully_charged_entry.entry_id,
-            "formula_discharge": dischg_comp.reduced_formula,
-            "id_discharge": self.fully_discharged_entry.entry_id,
-            "fracA_charge": chg_comp.get_atomic_fraction(ion),
-            "fracA_discharge": dischg_comp.get_atomic_fraction(ion),
-            "max_instability": self.get_max_instability(),
-            "min_instability": self.get_min_instability(),
-            "material_ids": [itr_ent.entry_id for itr_ent in self.get_all_entries()],
-            "stable_material_ids": [itr_ent.entry_id for itr_ent in self.get_stable_entries()],
-            "unstable_material_ids": [itr_ent.entry_id for itr_ent in self.get_unstable_entries()],
-        }
-
-        if all("decomposition_energy" in itr_ent.data for itr_ent in self.get_all_entries()):
-            d.update(
-                {
-                    "stability_charge": self.fully_charged_entry.data["decomposition_energy"],
-                    "stability_discharge": self.fully_discharged_entry.data["decomposition_energy"],
-                    "stability_data": {
-                        itr_ent.entry_id: itr_ent.data["decomposition_energy"] for itr_ent in self.get_all_entries()
-                    },
-                }
-            )
-
-        if all("muO2" in itr_ent.data for itr_ent in self.get_all_entries()):
-            d.update({"muO2_data": {itr_ent.entry_id: itr_ent.data["muO2"] for itr_ent in self.get_all_entries()}})
-
-        if print_subelectrodes:
-
-            def f_dict(c):
-                return c.get_summary_dict(print_subelectrodes=False)
-
-            d["adj_pairs"] = list(map(f_dict, self.get_sub_electrodes(adjacent_only=True)))
-            d["all_pairs"] = list(map(f_dict, self.get_sub_electrodes(adjacent_only=False)))
-        return d
-
     def __str__(self):
         return self.__repr__()
 
@@ -433,10 +373,10 @@ class InsertionElectrode(AbstractElectrode):
         output = []
         chg_form = self.fully_charged_entry.composition.reduced_formula
         dischg_form = self.fully_discharged_entry.composition.reduced_formula
-        output.append("InsertionElectrode with endpoints at {} and {}".format(chg_form, dischg_form))
-        output.append("Avg. volt. = {} V".format(self.get_average_voltage()))
-        output.append("Grav. cap. = {} mAh/g".format(self.get_capacity_grav()))
-        output.append("Vol. cap. = {}".format(self.get_capacity_vol()))
+        output.append(f"InsertionElectrode with endpoints at {chg_form} and {dischg_form}")
+        output.append(f"Avg. volt. = {self.get_average_voltage()} V")
+        output.append(f"Grav. cap. = {self.get_capacity_grav()} mAh/g")
+        output.append(f"Vol. cap. = {self.get_capacity_vol()}")
         return "\n".join(output)
 
     @classmethod
@@ -451,7 +391,7 @@ class InsertionElectrode(AbstractElectrode):
         from monty.json import MontyDecoder
 
         dec = MontyDecoder()
-        return cls(
+        return InsertionElectrode(  # pylint: disable=E1120
             dec.process_decoded(d["entries"]),
             dec.process_decoded(d["working_ion_entry"]),
         )
@@ -508,23 +448,23 @@ class InsertionVoltagePair(AbstractVoltagePair):
 
         # check that the ion is just a single element
         if not working_ion_entry.composition.is_element:
-            raise ValueError("VoltagePair: The working ion specified must be " "an element")
+            raise ValueError("VoltagePair: The working ion specified must be an element")
 
         # check that at least one of the entries contains the working element
         if (
             not comp_charge.get_atomic_fraction(working_element) > 0
             and not comp_discharge.get_atomic_fraction(working_element) > 0
         ):
-            raise ValueError("VoltagePair: The working ion must be present in " "one of the entries")
+            raise ValueError("VoltagePair: The working ion must be present in one of the entries")
 
         # check that the entries do not contain the same amount of the workin
         # element
         if comp_charge.get_atomic_fraction(working_element) == comp_discharge.get_atomic_fraction(working_element):
-            raise ValueError("VoltagePair: The working ion atomic percentage " "cannot be the same in both the entries")
+            raise ValueError("VoltagePair: The working ion atomic percentage cannot be the same in both the entries")
 
         # check that the frameworks of the entries are equivalent
         if not frame_charge_comp.reduced_formula == frame_discharge_comp.reduced_formula:
-            raise ValueError("VoltagePair: the specified entries must have the" " same compositional framework")
+            raise ValueError("VoltagePair: the specified entries must have the same compositional framework")
 
         # Initialize normalization factors, charged and discharged entries
 
@@ -567,7 +507,7 @@ class InsertionVoltagePair(AbstractVoltagePair):
         _frac_charge = comp_charge.get_atomic_fraction(working_element)
         _frac_discharge = comp_discharge.get_atomic_fraction(working_element)
 
-        vpair = cls(
+        vpair = InsertionVoltagePair(  # pylint: disable=E1123
             voltage=_voltage,
             mAh=_mAh,
             mass_charge=_mass_charge,
@@ -579,7 +519,7 @@ class InsertionVoltagePair(AbstractVoltagePair):
             working_ion_entry=working_ion_entry,
             entry_charge=entry_charge,
             entry_discharge=entry_discharge,
-            _framework_formula=framework.reduced_formula,
+            framework_formula=framework.reduced_formula,
         )
 
         # Step 4: add (optional) hull and muO2 data
@@ -593,11 +533,11 @@ class InsertionVoltagePair(AbstractVoltagePair):
 
     def __repr__(self):
         output = [
-            "Insertion voltage pair with working ion {}".format(self.working_ion_entry.composition.reduced_formula),
-            "V = {}, mAh = {}".format(self.voltage, self.mAh),
-            "mass_charge = {}, mass_discharge = {}".format(self.mass_charge, self.mass_discharge),
-            "vol_charge = {}, vol_discharge = {}".format(self.vol_charge, self.vol_discharge),
-            "frac_charge = {}, frac_discharge = {}".format(self.frac_charge, self.frac_discharge),
+            f"Insertion voltage pair with working ion {self.working_ion_entry.composition.reduced_formula}",
+            f"V = {self.voltage}, mAh = {self.mAh}",
+            f"mass_charge = {self.mass_charge}, mass_discharge = {self.mass_discharge}",
+            f"vol_charge = {self.vol_charge}, vol_discharge = {self.vol_discharge}",
+            f"frac_charge = {self.frac_charge}, frac_discharge = {self.frac_discharge}",
         ]
         return "\n".join(output)
 
