@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -9,19 +8,18 @@ import re
 import warnings
 from collections import Counter
 from enum import Enum
-from io import open
 from itertools import combinations, product
 from pathlib import Path
-from typing import Callable, Optional, Union, Dict, Tuple, List
+from typing import Callable, Dict, List, Literal, Optional, Tuple, Union
 
 import numpy as np
 from monty.json import MSONable
 
 from pymatgen.core.units import SUPPORTED_UNIT_NAMES, FloatWithUnit, Length, Mass, Unit
-from pymatgen.util.string import formula_double_format, Stringify
+from pymatgen.util.string import Stringify, formula_double_format
 
 # Loads element data from json file
-with open(str(Path(__file__).absolute().parent / "periodic_table.json"), "rt") as f:
+with open(str(Path(__file__).absolute().parent / "periodic_table.json")) as f:
     _pt_data = json.load(f)
 
 _pt_row_sizes = (2, 8, 8, 18, 18, 32, 32)
@@ -63,8 +61,14 @@ class ElementBase(Enum):
         .. attribute:: van_der_waals_radius
 
             Van der Waals radius for the element. This is the empirical
-            value. Data is obtained from
-            http://en.wikipedia.org/wiki/Atomic_radii_of_the_elements_(data_page).
+            value determined from critical reviews of X-ray diffraction, gas kinetic
+            collision cross-section, and other experimental data by Bondi and later
+            workers. The uncertainty in these values is on the order of 0.1 Å.
+
+            Data are obtained from
+
+            "Atomic Radii of the Elements" in CRC Handbook of Chemistry and Physics,
+                91st Ed.; Haynes, W.M., Ed.; CRC Press: Boca Raton, FL, 2010.
 
         .. attribute:: mendeleev_no
 
@@ -166,8 +170,18 @@ class ElementBase(Enum):
         .. attribute:: coefficient_of_linear_thermal_expansion
 
             Coefficient of linear thermal expansion
+
+        .. attribute:: ground_level
+
+            Ground level for element
+
+        .. attribute:: ionization_energies
+
+            List of ionization energies. First value is the first ionization energy, second is the second ionization
+            energy, etc. Note that this is zero-based indexing! So Element.ionization_energies[0] refer to the 1st
+            ionization energy. Values are from the NIST Atomic Spectra Database. Missing values are None.
         """
-        self.symbol = "%s" % symbol
+        self.symbol = symbol
         d = _pt_data[symbol]
 
         # Store key variables for quick access
@@ -200,14 +214,16 @@ class ElementBase(Enum):
     @property
     def atomic_radius(self) -> Optional[FloatWithUnit]:
         """
-        Returns: The atomic radius of the element in Ångstroms.
+        Returns:
+            float | None: The atomic radius of the element in Ångstroms.
         """
         return self._atomic_radius
 
     @property
     def atomic_mass(self) -> Optional[FloatWithUnit]:
         """
-        Returns: The atomic mass of the element in amu.
+        Returns:
+            float | None: The atomic mass of the element in amu.
         """
         return self._atomic_mass
 
@@ -239,12 +255,14 @@ class ElementBase(Enum):
             "coefficient_of_linear_thermal_expansion",
             "ground_state_term_symbol",
             "valence",
+            "ground_level",
+            "ionization_energies",
         ]:
             kstr = item.capitalize().replace("_", " ")
             val = self._data.get(kstr, None)
             if str(val).startswith("no data"):
                 val = None
-            elif isinstance(val, dict):
+            elif isinstance(val, (list, dict)):
                 pass
             else:
                 try:
@@ -276,7 +294,7 @@ class ElementBase(Enum):
                             # Ignore error. val will just remain a string.
                             pass
             return val
-        raise AttributeError("Element has no attribute %s!" % item)
+        raise AttributeError(f"Element has no attribute {item}!")
 
     @property
     def data(self) -> dict:
@@ -285,6 +303,20 @@ class ElementBase(Enum):
         """
         return self._data.copy()
 
+    @property
+    def ionization_energy(self) -> float:
+        """
+        First ionization energy of element.
+        """
+        return self._data["Ionization energies"][0]
+
+    @property
+    def electron_affinity(self) -> float:
+        """
+        First ionization energy of element.
+        """
+        return self._data["Electron affinity"]
+
     @property
     def electronic_structure(self) -> str:
         """
@@ -364,18 +396,18 @@ class ElementBase(Enum):
     @property
     def oxidation_states(self) -> Tuple:
         """Tuple of all known oxidation states"""
-        return tuple(self._data.get("Oxidation states", list()))
+        return tuple(self._data.get("Oxidation states", []))
 
     @property
     def common_oxidation_states(self) -> Tuple:
         """Tuple of common oxidation states"""
-        return tuple(self._data.get("Common oxidation states", list()))
+        return tuple(self._data.get("Common oxidation states", []))
 
     @property
     def icsd_oxidation_states(self) -> Tuple:
         """Tuple of all oxidation states with at least 10 instances in
         ICSD database AND at least 1% of entries for that element"""
-        return tuple(self._data.get("ICSD oxidation states", list()))
+        return tuple(self._data.get("ICSD oxidation states", []))
 
     @property
     def metallic_radius(self) -> float:
@@ -457,8 +489,8 @@ class ElementBase(Enum):
         e_config_combs = list(combinations(range(n), v_e))
 
         # Total ML = sum(ml1, ml2), Total MS = sum(ms1, ms2)
-        TL = [sum([ml_ms[comb[e]][0] for e in range(v_e)]) for comb in e_config_combs]
-        TS = [sum([ml_ms[comb[e]][1] for e in range(v_e)]) for comb in e_config_combs]
+        TL = [sum(ml_ms[comb[e]][0] for e in range(v_e)) for comb in e_config_combs]
+        TS = [sum(ml_ms[comb[e]][1] for e in range(v_e)) for comb in e_config_combs]
         comb_counter = Counter(zip(TL, TS))
 
         term_symbols = []
@@ -560,23 +592,41 @@ class ElementBase(Enum):
         for sym, data in _pt_data.items():
             if data["Atomic no"] == z:
                 return Element(sym)
-        raise ValueError("No element with this atomic number %s" % z)
+        raise ValueError(f"No element with this atomic number {z}")
 
     @staticmethod
     def from_row_and_group(row: int, group: int) -> "Element":
         """
         Returns an element from a row and group number.
+        Important Note: For lanthanoids and actinoids, the row number must
+        be 8 and 9, respectively, and the group number must be
+        between 3 (La, Ac) and 17 (Lu, Lr). This is different than the
+        value for Element(symbol).row and Element(symbol).group for these
+        elements.
 
         Args:
-            row (int): Row number
-            group (int): Group number
+            row (int): (pseudo) row number. This is the
+                standard row number except for the lanthanoids
+                and actinoids for which it is 8 or 9, respectively.
+            group (int): (pseudo) group number. This is the
+                standard group number except for the lanthanoids
+                and actinoids for which it is 3 (La, Ac) to 17 (Lu, Lr).
 
         .. note::
             The 18 group number system is used, i.e., Noble gases are group 18.
         """
         for sym in _pt_data.keys():
             el = Element(sym)
-            if el.row == row and el.group == group:
+            if 57 <= el.Z <= 71:
+                el_pseudorow = 8
+                el_pseudogroup = (el.Z - 54) % 32
+            elif 89 <= el.Z <= 103:
+                el_pseudorow = 9
+                el_pseudogroup = (el.Z - 54) % 32
+            else:
+                el_pseudorow = el.row
+                el_pseudogroup = el.group
+            if el_pseudorow == row and el_pseudogroup == group:
                 return el
         raise ValueError("No element with this row and group!")
 
@@ -598,13 +648,15 @@ class ElementBase(Enum):
     def row(self) -> int:
         """
         Returns the periodic table row of the element.
+        Note: For lanthanoids and actinoids, the row is always 6 or 7,
+        respectively.
         """
         z = self.Z
         total = 0
         if 57 <= z <= 71:
-            return 8
+            return 6
         if 89 <= z <= 103:
-            return 9
+            return 7
         for i, size in enumerate(_pt_row_sizes):
             total += size
             if total >= z:
@@ -615,12 +667,14 @@ class ElementBase(Enum):
     def group(self) -> int:
         """
         Returns the periodic table group of the element.
+        Note: For lanthanoids and actinoids, the group is always 3.
         """
         z = self.Z
         if z == 1:
             return 1
         if z == 2:
             return 18
+
         if 3 <= z <= 18:
             if (z - 2) % 8 == 0:
                 return 18
@@ -633,6 +687,9 @@ class ElementBase(Enum):
                 return 18
             return (z - 18) % 18
 
+        if (57 <= z <= 71) or (89 <= z <= 103):
+            return 3
+
         if (z - 54) % 32 == 0:
             return 18
         if (z - 54) % 32 >= 18:
@@ -773,7 +830,7 @@ class ElementBase(Enum):
         """
         Ordering according to Table VI of "Nomenclature of Inorganic Chemistry
         (IUPAC Recommendations 2005)". This ordering effectively follows the
-        groups and rows of the periodic table, except the Lanthanides, Actanides
+        groups and rows of the periodic table, except the Lanthanides, Actinides
         and hydrogen.
         """
         return self._data["IUPAC ordering"]
@@ -820,7 +877,7 @@ class ElementBase(Enum):
                 except ValueError:
                     el = None  # type: ignore
                 if el and ((not filter_function) or filter_function(el)):
-                    rowstr.append("{:3s}".format(el.symbol))
+                    rowstr.append(f"{el.symbol:3s}")
                 else:
                     rowstr.append("   ")
             print(" ".join(rowstr))
@@ -998,7 +1055,7 @@ class Species(MSONable, Stringify):
         self._properties = properties if properties else {}
         for k, _ in self._properties.items():
             if k not in Species.supported_properties:
-                raise ValueError("{} is not a supported property".format(k))
+                raise ValueError(f"{k} is not a supported property")
 
     def __getattr__(self, a):
         # overriding getattr doesn't play nice with pickle, so we
@@ -1071,12 +1128,12 @@ class Species(MSONable, Stringify):
             d = self._el.data
             oxstr = str(int(self._oxi_state))
             if oxstr in d.get("Ionic radii hs", {}):
-                warnings.warn("No default ionic radius for %s. Using hs data." % self)
+                warnings.warn(f"No default ionic radius for {self}. Using hs data.")
                 return d["Ionic radii hs"][oxstr]
             if oxstr in d.get("Ionic radii ls", {}):
-                warnings.warn("No default ionic radius for %s. Using ls data." % self)
+                warnings.warn(f"No default ionic radius for {self}. Using ls data.")
                 return d["Ionic radii ls"][oxstr]
-        warnings.warn("No ionic radius for {}!".format(self))
+        warnings.warn(f"No ionic radius for {self}!")
         return None
 
     @property
@@ -1099,7 +1156,7 @@ class Species(MSONable, Stringify):
             A Species object.
 
         Raises:
-            ValueError if species_string cannot be intepreted.
+            ValueError if species_string cannot be interpreted.
         """
 
         # e.g. Fe2+,spin=5
@@ -1145,7 +1202,7 @@ class Species(MSONable, Stringify):
             else:
                 output += formula_double_format(-self.oxi_state) + "-"
         for p, v in self._properties.items():
-            output += ",%s=%s" % (p, v)
+            output += f",{p}={v}"
         return output
 
     def to_pretty_string(self) -> str:
@@ -1181,10 +1238,15 @@ class Species(MSONable, Stringify):
             return quad_mom.get(isotopes[0], 0.0)
 
         if isotope not in quad_mom:
-            raise ValueError("No quadrupole moment for isotope {}".format(isotope))
+            raise ValueError(f"No quadrupole moment for isotope {isotope}")
         return quad_mom.get(isotope, 0.0)
 
-    def get_shannon_radius(self, cn: str, spin: str = "", radius_type: str = "ionic") -> float:
+    def get_shannon_radius(
+        self,
+        cn: str,
+        spin: Literal["", "Low Spin", "High Spin"] = "",
+        radius_type: Literal["ionic", "crystal"] = "ionic",
+    ) -> float:
         """
         Get the local environment specific ionic radius for species.
 
@@ -1207,39 +1269,40 @@ class Species(MSONable, Stringify):
             k, data = list(radii.items())[0]  # type: ignore
             if k != spin:
                 warnings.warn(
-                    "Specified spin state of %s not consistent with database "
-                    "spin of %s. Only one spin data available, and "
-                    "that value is returned." % (spin, k)
+                    f"Specified spin state of {spin} not consistent with database "
+                    f"spin of {k}. Only one spin data available, and that value is returned."
                 )
         else:
             data = radii[spin]
-        return data["%s_radius" % radius_type]
+        return data[f"{radius_type}_radius"]
 
-    def get_crystal_field_spin(self, coordination: str = "oct", spin_config: str = "high") -> float:
+    def get_crystal_field_spin(
+        self, coordination: Literal["oct", "tet"] = "oct", spin_config: Literal["low", "high"] = "high"
+    ) -> float:
         """
         Calculate the crystal field spin based on coordination and spin
         configuration. Only works for transition metal species.
 
         Args:
-            coordination (str): Only oct and tet are supported at the moment.
-            spin_config (str): Supported keywords are "high" or "low".
+            coordination ("oct" | "tet"): Tetrahedron or octahedron crystal site coordination
+            spin_config ("low" | "high"): Whether the species is in a high or low spin state
 
         Returns:
             Crystal field spin in Bohr magneton.
 
         Raises:
             AttributeError if species is not a valid transition metal or has
-            an invalid oxidation state.
+                an invalid oxidation state.
             ValueError if invalid coordination or spin_config.
         """
         if coordination not in ("oct", "tet") or spin_config not in ("high", "low"):
             raise ValueError("Invalid coordination or spin config.")
         elec = self.full_electronic_structure
         if len(elec) < 4 or elec[-1][1] != "s" or elec[-2][1] != "d":
-            raise AttributeError("Invalid element {} for crystal field calculation.".format(self.symbol))
+            raise AttributeError(f"Invalid element {self.symbol} for crystal field calculation.")
         nelectrons = elec[-1][2] + elec[-2][2] - self.oxi_state
         if nelectrons < 0 or nelectrons > 10:
-            raise AttributeError("Invalid oxidation state {} for element {}".format(self.oxi_state, self.symbol))
+            raise AttributeError(f"Invalid oxidation state {self.oxi_state} for element {self.symbol}")
         if spin_config == "high":
             if nelectrons <= 5:
                 return nelectrons
@@ -1335,7 +1398,7 @@ class DummySpecies(Species):
 
         for i in range(1, min(2, len(symbol)) + 1):
             if Element.is_valid_symbol(symbol[:i]):
-                raise ValueError("{} contains {}, which is a valid element " "symbol.".format(symbol, symbol[:i]))
+                raise ValueError(f"{symbol} contains {symbol[:i]}, which is a valid element symbol.")
 
         # Set required attributes for DummySpecies to function like a Species in
         # most instances.
@@ -1344,7 +1407,7 @@ class DummySpecies(Species):
         self._properties = properties if properties else {}
         for k, _ in self._properties.items():
             if k not in Species.supported_properties:
-                raise ValueError("{} is not a supported property".format(k))
+                raise ValueError(f"{k} is not a supported property")
 
     def __getattr__(self, a):
         # overriding getattr doens't play nice with pickle, so we
@@ -1485,7 +1548,7 @@ class DummySpecies(Species):
             else:
                 output += formula_double_format(-self.oxi_state) + "-"
         for p, v in self._properties.items():
-            output += ",%s=%s" % (p, v)
+            output += f",{p}={v}"
         return output
 
 
@@ -1495,8 +1558,6 @@ class Specie(Species):
     to maintain backwards compatibility.
     """
 
-    pass
-
 
 class DummySpecie(DummySpecies):
     """
@@ -1504,8 +1565,6 @@ class DummySpecie(DummySpecies):
     to maintain backwards compatibility.
     """
 
-    pass
-
 
 def get_el_sp(obj) -> Union[Element, Species, DummySpecies]:
     """
@@ -1551,4 +1610,4 @@ def get_el_sp(obj) -> Union[Element, Species, DummySpecies]:
             try:
                 return DummySpecies.from_string(obj)
             except Exception:
-                raise ValueError("Can't parse Element or String from type" " %s: %s." % (type(obj), obj))
+                raise ValueError(f"Can't parse Element or String from type {type(obj)}: {obj}.")
