@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -160,15 +159,13 @@ class DOS(Spectrum):
         Returns a string which can be easily plotted (using gnuplot).
         """
         if Spin.down in self.densities:
-            stringarray = ["#{:30s} {:30s} {:30s}".format("Energy", "DensityUp", "DensityDown")]
+            stringarray = [f"#{'Energy':30s} {'DensityUp':30s} {'DensityDown':30s}"]
             for i, energy in enumerate(self.energies):
-                stringarray.append(
-                    "{:.5f} {:.5f} {:.5f}".format(energy, self.densities[Spin.up][i], self.densities[Spin.down][i])
-                )
+                stringarray.append(f"{energy:.5f} {self.densities[Spin.up][i]:.5f} {self.densities[Spin.down][i]:.5f}")
         else:
-            stringarray = ["#{:30s} {:30s}".format("Energy", "DensityUp")]
+            stringarray = [f"#{'Energy':30s} {'DensityUp':30s}"]
             for i, energy in enumerate(self.energies):
-                stringarray.append("{:.5f} {:.5f}".format(energy, self.densities[Spin.up][i]))
+                stringarray.append(f"{energy:.5f} {self.densities[Spin.up][i]:.5f}")
         return "\n".join(stringarray)
 
 
@@ -368,15 +365,13 @@ class Dos(MSONable):
         Returns a string which can be easily plotted (using gnuplot).
         """
         if Spin.down in self.densities:
-            stringarray = ["#{:30s} {:30s} {:30s}".format("Energy", "DensityUp", "DensityDown")]
+            stringarray = [f"#{'Energy':30s} {'DensityUp':30s} {'DensityDown':30s}"]
             for i, energy in enumerate(self.energies):
-                stringarray.append(
-                    "{:.5f} {:.5f} {:.5f}".format(energy, self.densities[Spin.up][i], self.densities[Spin.down][i])
-                )
+                stringarray.append(f"{energy:.5f} {self.densities[Spin.up][i]:.5f} {self.densities[Spin.down][i]:.5f}")
         else:
-            stringarray = ["#{:30s} {:30s}".format("Energy", "DensityUp")]
+            stringarray = [f"#{'Energy':30s} {'DensityUp':30s}"]
             for i, energy in enumerate(self.energies):
-                stringarray.append("{:.5f} {:.5f}".format(energy, self.densities[Spin.up][i]))
+                stringarray.append(f"{energy:.5f} {self.densities[Spin.up][i]:.5f}")
         return "\n".join(stringarray)
 
     @classmethod
@@ -398,8 +393,8 @@ class Dos(MSONable):
             "@module": self.__class__.__module__,
             "@class": self.__class__.__name__,
             "efermi": self.efermi,
-            "energies": list(self.energies),
-            "densities": {str(spin): list(dens) for spin, dens in self.densities.items()},
+            "energies": self.energies.tolist(),
+            "densities": {str(spin): dens.tolist() for spin, dens in self.densities.items()},
         }
 
 
@@ -441,7 +436,7 @@ class FermiDos(Dos, MSONable):
             if hasattr(dos, "structure"):
                 structure = dos.structure
             else:
-                raise ValueError("Structure object is not provided and not " "present in dos")
+                raise ValueError("Structure object is not provided and not present in dos")
 
         self.structure = structure
         self.nelecs = nelecs or self.structure.composition.total_electrons
@@ -499,7 +494,7 @@ class FermiDos(Dos, MSONable):
         )
         vb_integral = np.sum(
             self.tdos[: self.idx_vbm + 1]
-            * (1 - f0(self.energies[: self.idx_vbm + 1], fermi_level, temperature))
+            * f0(-self.energies[: self.idx_vbm + 1], -fermi_level, temperature)
             * self.de[: self.idx_vbm + 1],
             axis=0,
         )
@@ -592,12 +587,12 @@ class FermiDos(Dos, MSONable):
         for _ in range(precision):
             frange = np.arange(-nstep, nstep + 1) * step + fermi
             calc_doping = np.array([self.get_doping(f, temperature) for f in frange])
-            relative_error = np.abs(calc_doping / concentration - 1.0)
+            relative_error = np.abs(calc_doping / concentration - 1.0)  # type: ignore
             fermi = frange[np.argmin(relative_error)]
             step /= 10.0
 
         if min(relative_error) > rtol:
-            raise ValueError("Could not find fermi within {}% of concentration={}".format(rtol * 100, concentration))
+            raise ValueError(f"Could not find fermi within {rtol * 100}% of concentration={concentration}")
         return fermi
 
     @classmethod
@@ -620,8 +615,8 @@ class FermiDos(Dos, MSONable):
             "@module": self.__class__.__module__,
             "@class": self.__class__.__name__,
             "efermi": self.efermi,
-            "energies": list(self.energies),
-            "densities": {str(spin): list(dens) for spin, dens in self.densities.items()},
+            "energies": self.energies.tolist(),
+            "densities": {str(spin): dens.tolist() for spin, dens in self.densities.items()},
             "structure": self.structure,
             "nelecs": self.nelecs,
         }
@@ -697,7 +692,7 @@ class CompleteDos(Dos):
         Returns:
             dict of {orbital: Dos}, e.g. {"s": Dos object, ...}
         """
-        spd_dos: Dict[Orbital, Dict[Spin, ArrayLike]] = dict()
+        spd_dos: Dict[Orbital, Dict[Spin, ArrayLike]] = {}
         for orb, pdos in self.pdos[site].items():
             orbital_type = _get_orb_type(orb)
             if orbital_type in spd_dos:
@@ -774,7 +769,7 @@ class CompleteDos(Dos):
             el: Element in Structure.composition associated with CompleteDos
 
         Returns:
-            dict of {Element: {"S": densities, "P": densities, "D": densities}}
+            dict of {orbital: Dos}, e.g. {"s": Dos object, ...}
         """
         el = get_el_sp(el)
         el_dos = {}
@@ -840,8 +835,8 @@ class CompleteDos(Dos):
             "@class": self.__class__.__name__,
             "efermi": self.efermi,
             "structure": self.structure.as_dict(),
-            "energies": list(self.energies),
-            "densities": {str(spin): list(dens) for spin, dens in self.densities.items()},
+            "energies": self.energies.tolist(),
+            "densities": {str(spin): dens.tolist() for spin, dens in self.densities.items()},
             "pdos": [],
         }
         if len(self.pdos) > 0:
