@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 """
@@ -11,7 +10,7 @@ import collections
 import logging
 import os
 import sys
-from collections import OrderedDict, defaultdict, namedtuple
+from collections import defaultdict, namedtuple
 
 import numpy as np
 from monty.collections import AttrDict, Namespace
@@ -57,11 +56,11 @@ def _read_nlines(filename, nlines):
     If nlines is < 0, the entire file is read.
     """
     if nlines < 0:
-        with open(filename, "r") as fh:
+        with open(filename) as fh:
             return fh.readlines()
 
     lines = []
-    with open(filename, "r") as fh:
+    with open(filename) as fh:
         for lineno, line in enumerate(fh):
             if lineno == nlines:
                 break
@@ -87,7 +86,7 @@ def l2str(l):
     try:
         return _l2str[l]
     except KeyError:
-        return "Unknown angular momentum, received l = %s" % l
+        return f"Unknown angular momentum, received l = {l}"
 
 
 def str2l(s):
@@ -136,36 +135,34 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
 
     def __repr__(self):
         try:
-            return "<%s at %s>" % (
-                self.__class__.__name__,
-                os.path.relpath(self.filepath),
-            )
+            return f"<{self.__class__.__name__} at {os.path.relpath(self.filepath)}>"
         except Exception:
             # relpath can fail if the code is executed in demon mode.
-            return "<%s at %s>" % (self.__class__.__name__, self.filepath)
+            return f"<{self.__class__.__name__} at {self.filepath}>"
 
     def __str__(self):
         return self.to_string()
 
     def to_string(self, verbose=0):
         """String representation."""
+        # pylint: disable=E1101
         lines = []
         app = lines.append
-        app("<%s: %s>" % (self.__class__.__name__, self.basename))
+        app(f"<{self.__class__.__name__}: {self.basename}>")
         app("  summary: " + self.summary.strip())
-        app("  number of valence electrons: %s" % self.Z_val)
-        app("  maximum angular momentum: %s" % l2str(self.l_max))
-        app("  angular momentum for local part: %s" % l2str(self.l_local))
-        app("  XC correlation: %s" % self.xc)
-        app("  supports spin-orbit: %s" % self.supports_soc)
+        app(f"  number of valence electrons: {self.Z_val}")
+        app(f"  maximum angular momentum: {l2str(self.l_max)}")
+        app(f"  angular momentum for local part: {l2str(self.l_local)}")
+        app(f"  XC correlation: {self.xc}")
+        app(f"  supports spin-orbit: {self.supports_soc}")
 
         if self.isnc:
-            app("  radius for non-linear core correction: %s" % self.nlcc_radius)
+            app(f"  radius for non-linear core correction: {self.nlcc_radius}")
 
         if self.has_hints:
             for accuracy in ("low", "normal", "high"):
                 hint = self.hint_for_accuracy(accuracy=accuracy)
-                app("  hint for %s accuracy: %s" % (accuracy, str(hint)))
+                app(f"  hint for {accuracy} accuracy: {hint}")
 
         return "\n".join(lines)
 
@@ -177,11 +174,13 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
     @property
     def filepath(self):
         """Absolute path to pseudopotential file."""
+        # pylint: disable=E1101
         return os.path.abspath(self.path)
 
     @property
     def basename(self):
         """File basename."""
+        # pylint: disable=E1101
         return os.path.basename(self.filepath)
 
     @property
@@ -240,9 +239,10 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
 
     def compute_md5(self):
         """Compute and erturn MD5 hash value."""
+        # pylint: disable=E1101
         import hashlib
 
-        with open(self.path, "rt") as fh:
+        with open(self.path) as fh:
             text = fh.read()
             m = hashlib.md5(text.encode("utf-8"))
             return m.hexdigest()
@@ -258,6 +258,7 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
     @pmg_serialize
     def as_dict(self, **kwargs):
         """Return dictionary for MSONable protocol."""
+        # pylint: disable=E1101
         return dict(
             basename=self.basename,
             type=self.type,
@@ -293,6 +294,7 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
             tmpdir: If None, a new temporary directory is created and files are copied here
                 else tmpdir is used.
         """
+        # pylint: disable=E1101
         import shutil
         import tempfile
 
@@ -316,11 +318,13 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
     @property
     def has_dojo_report(self):
         """True if the pseudo has an associated `DOJO_REPORT` section."""
+        # pylint: disable=E1101
         return hasattr(self, "dojo_report") and bool(self.dojo_report)
 
     @property
     def djrepo_path(self):
         """The path of the djrepo file. None if file does not exist."""
+        # pylint: disable=E1101
         root, ext = os.path.splitext(self.filepath)
         path = root + ".djrepo"
         return path
@@ -336,13 +340,14 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
         Args:
             accuracy: ["low", "normal", "high"]
         """
+        # pylint: disable=E1101
         if not self.has_dojo_report:
             return Hint(ecut=0.0, pawecutdg=0.0)
 
         # Get hints from dojoreport. Try first in hints then in ppgen_hints.
         if "hints" in self.dojo_report:
             return Hint.from_dict(self.dojo_report["hints"][accuracy])
-        elif "ppgen_hints" in self.dojo_report:
+        if "ppgen_hints" in self.dojo_report:
             return Hint.from_dict(self.dojo_report["ppgen_hints"][accuracy])
         return Hint(ecut=0.0, pawecutdg=0.0)
 
@@ -397,16 +402,14 @@ class Pseudo(MSONable, metaclass=abc.ABCMeta):
 
         filepath = task.outdir.has_abiext("_PSPS.nc")
         if not filepath:
-            logger.critical("Cannot find PSPS.nc file in %s" % task.outdir)
+            logger.critical(f"Cannot find PSPS.nc file in {task.outdir}")
             return None
 
         # Open the PSPS.nc file.
         try:
             return PspsFile(filepath)
         except Exception as exc:
-            logger.critical(
-                "Exception while reading PSPS file at %s:\n%s" % (filepath, str(exc))
-            )
+            logger.critical(f"Exception while reading PSPS file at {filepath}:\n{exc}")
             return None
 
 
@@ -499,30 +502,35 @@ class AbinitPseudo(Pseudo):
 
     @property
     def Z(self):
+        # pylint: disable=E1101
         return self._zatom
 
     @property
     def Z_val(self):
+        # pylint: disable=E1101
         return self._zion
 
     @property
     def l_max(self):
+        # pylint: disable=E1101
         return self._lmax
 
     @property
     def l_local(self):
+        # pylint: disable=E1101
         return self._lloc
 
     @property
     def supports_soc(self):
-        # Treate ONCVPSP pseudos
+        # Treat ONCVPSP pseudos
+        # pylint: disable=E1101
         if self._pspcod == 8:
             switch = self.header["extension_switch"]
             if switch in (0, 1):
                 return False
             if switch in (2, 3):
                 return True
-            raise ValueError("Don't know how to handle extension_switch: %s" % switch)
+            raise ValueError(f"Don't know how to handle extension_switch: {switch}")
 
         # TODO Treat HGH HGHK pseudos
 
@@ -539,23 +547,28 @@ class NcAbinitPseudo(NcPseudo, AbinitPseudo):
 
     @property
     def Z(self):
+        # pylint: disable=E1101
         return self._zatom
 
     @property
     def Z_val(self):
         """Number of valence electrons."""
+        # pylint: disable=E1101
         return self._zion
 
     @property
     def l_max(self):
+        # pylint: disable=E1101
         return self._lmax
 
     @property
     def l_local(self):
+        # pylint: disable=E1101
         return self._lloc
 
     @property
     def nlcc_radius(self):
+        # pylint: disable=E1101
         return self._rchrg
 
 
@@ -564,6 +577,7 @@ class PawAbinitPseudo(PawPseudo, AbinitPseudo):
 
     @property
     def paw_radius(self):
+        # pylint: disable=E1101
         return self._r_cut
 
     # def orbitals(self):
@@ -585,9 +599,8 @@ class Hint:
 
     def __str__(self):
         if self.pawecutdg is not None:
-            return "ecut: %s, pawecutdg: %s" % (self.ecut, self.pawecutdg)
-        else:
-            return "ecut: %s" % (self.ecut)
+            return f"ecut: {self.ecut}, pawecutdg: {self.pawecutdg}"
+        return f"ecut: {self.ecut}"
 
     @pmg_serialize
     def as_dict(self):
@@ -623,7 +636,7 @@ def _dict_from_lines(lines, key_nums, sep=None):
         key_nums = list(key_nums)
 
     if len(lines) != len(key_nums):
-        err_msg = "lines = %s\n key_num =  %s" % (str(lines), str(key_nums))
+        err_msg = f"lines = {lines}\n key_num =  {key_nums}"
         raise ValueError(err_msg)
 
     kwargs = Namespace()
@@ -642,11 +655,11 @@ def _dict_from_lines(lines, key_nums, sep=None):
         if sep is not None:
             check = keys[0][0]
             if check != sep:
-                raise ValueError("Expecting separator %s, got %s" % (sep, check))
+                raise ValueError(f"Expecting separator {sep}, got {check}")
             keys[0] = keys[0][1:]
 
         if len(values) != len(keys):
-            msg = "line: %s\n len(keys) != len(value)\nkeys: %s\n values: %s" % (
+            msg = "line: {}\n len(keys) != len(value)\nkeys: {}\n values: {}".format(
                 line,
                 keys,
                 values,
@@ -684,17 +697,16 @@ def _int_from_str(string):
     int_num = int(float_num)
     if float_num == int_num:
         return int_num
-    else:
-        # Needed to handle pseudos with fractional charge
-        int_num = np.rint(float_num)
-        logger.warning("Converting float %s to int %s" % (float_num, int_num))
-        return int_num
+    # Needed to handle pseudos with fractional charge
+    int_num = np.rint(float_num)
+    logger.warning(f"Converting float {float_num} to int {int_num}")
+    return int_num
 
 
 class NcAbinitHeader(AbinitHeader):
     """The abinit header found in the NC pseudopotential files."""
 
-    _attr_desc = namedtuple("att", "default astype")
+    _attr_desc = namedtuple("_attr_desc", "default astype")
 
     _VARS = {
         # Mandatory
@@ -708,9 +720,7 @@ class NcAbinitHeader(AbinitHeader):
         "r2well": _attr_desc(None, float),
         "mmax": _attr_desc(None, float),
         # Optional variables for non linear-core correction. HGH does not have it.
-        "rchrg": _attr_desc(
-            0.0, float
-        ),  # radius at which the core charge vanish (i.e. cut-off in a.u.)
+        "rchrg": _attr_desc(0.0, float),  # radius at which the core charge vanish (i.e. cut-off in a.u.)
         "fchrg": _attr_desc(0.0, float),
         "qchrg": _attr_desc(0.0, float),
     }
@@ -732,14 +742,12 @@ class NcAbinitHeader(AbinitHeader):
             if value is None:
                 value = default
                 if default is None:
-                    raise RuntimeError("Attribute %s must be specified" % key)
+                    raise RuntimeError(f"Attribute {key} must be specified")
             else:
                 try:
                     value = astype(value)
                 except Exception:
-                    raise RuntimeError(
-                        "Conversion Error for key %s, value %s" % (key, value)
-                    )
+                    raise RuntimeError(f"Conversion Error for key {key}, value {value}")
 
             self[key] = value
 
@@ -872,7 +880,7 @@ class NcAbinitHeader(AbinitHeader):
         # Parse the section with the projectors.
         # 0   4.085   6.246    0   2.8786493        l,e99.0,e99.9,nproj,rcpsp
         # .00000000    .0000000000    .0000000000    .00000000   rms,ekb1,ekb2,epsatm
-        projectors = OrderedDict()
+        projectors = {}
         for idx in range(2 * (lmax + 1)):
             line = lines[idx]
             if idx % 2 == 0:
@@ -896,7 +904,7 @@ class NcAbinitHeader(AbinitHeader):
 class PawAbinitHeader(AbinitHeader):
     """The abinit header found in the PAW pseudopotential files."""
 
-    _attr_desc = namedtuple("att", "default astype")
+    _attr_desc = namedtuple("_attr_desc", "default astype")
 
     _VARS = {
         "zatom": _attr_desc(None, _int_from_str),
@@ -933,19 +941,17 @@ class PawAbinitHeader(AbinitHeader):
             if value is None:
                 value = default
                 if default is None:
-                    raise RuntimeError("Attribute %s must be specified" % key)
+                    raise RuntimeError(f"Attribute {key} must be specified")
             else:
                 try:
                     value = astype(value)
                 except Exception:
-                    raise RuntimeError(
-                        "Conversion Error for key %s, with value %s" % (key, value)
-                    )
+                    raise RuntimeError(f"Conversion Error for key {key}, with value {value}")
 
             self[key] = value
 
         if kwargs:
-            raise RuntimeError("kwargs should be empty but got %s" % str(kwargs))
+            raise RuntimeError(f"kwargs should be empty but got {str(kwargs)}")
 
     @staticmethod
     def paw_header(filename, ppdesc):
@@ -1001,9 +1007,7 @@ class PawAbinitHeader(AbinitHeader):
         """
         supported_formats = ["paw3", "paw4", "paw5"]
         if ppdesc.format not in supported_formats:
-            raise NotImplementedError(
-                "format %s not in %s" % (ppdesc.format, supported_formats)
-            )
+            raise NotImplementedError(f"format {ppdesc.format} not in {supported_formats}")
 
         lines = _read_nlines(filename, -1)
 
@@ -1050,29 +1054,27 @@ class PseudoParser:
     ppdesc = namedtuple("ppdesc", "pspcod name psp_type format")
 
     # TODO Recheck
-    _PSPCODES = OrderedDict(
-        {
-            1: ppdesc(1, "TM", "NC", None),
-            2: ppdesc(2, "GTH", "NC", None),
-            3: ppdesc(3, "HGH", "NC", None),
-            4: ppdesc(4, "Teter", "NC", None),
-            # 5: ppdesc(5, "NC",     , None),
-            6: ppdesc(6, "FHI", "NC", None),
-            7: ppdesc(6, "PAW_abinit_text", "PAW", None),
-            8: ppdesc(8, "ONCVPSP", "NC", None),
-            10: ppdesc(10, "HGHK", "NC", None),
-        }
-    )
+    _PSPCODES = {
+        1: ppdesc(1, "TM", "NC", None),
+        2: ppdesc(2, "GTH", "NC", None),
+        3: ppdesc(3, "HGH", "NC", None),
+        4: ppdesc(4, "Teter", "NC", None),
+        # 5: ppdesc(5, "NC",     , None),
+        6: ppdesc(6, "FHI", "NC", None),
+        7: ppdesc(6, "PAW_abinit_text", "PAW", None),
+        8: ppdesc(8, "ONCVPSP", "NC", None),
+        10: ppdesc(10, "HGHK", "NC", None),
+    }
     del ppdesc
 
-    # renumber functionals from oncvpsp todo confrim that 3 is 2
+    # renumber functionals from oncvpsp todo confirm that 3 is 2
     # _FUNCTIONALS = {1: {'n': 4, 'name': 'Wigner'},
     #                2: {'n': 5, 'name': 'HL'},
     #                3: {'n': 2, 'name': 'PWCA'},
     #                4: {'n': 11, 'name': 'PBE'}}
 
     def __init__(self):
-        # List of files that have been parsed succesfully.
+        # List of files that have been parsed successfully.
         self._parsed_paths = []
 
         # List of files that could not been parsed.
@@ -1099,12 +1101,7 @@ class PseudoParser:
         for fname in os.listdir(dirname):
             root, ext = os.path.splitext(fname)
             path = os.path.join(dirname, fname)
-            if (
-                ext in exclude_exts
-                or fname in exclude_fnames
-                or fname.startswith(".")
-                or not os.path.isfile(path)
-            ):
+            if ext in exclude_exts or fname in exclude_fnames or fname.startswith(".") or not os.path.isfile(path):
                 continue
             paths.append(path)
 
@@ -1137,50 +1134,43 @@ class PseudoParser:
         if filename.endswith(".xml"):
             raise self.Error("XML pseudo not supported yet")
 
-        else:
-            # Assume file with the abinit header.
-            lines = _read_nlines(filename, 80)
+        # Assume file with the abinit header.
+        lines = _read_nlines(filename, 80)
 
-            for lineno, line in enumerate(lines):
+        for lineno, line in enumerate(lines):
 
-                if lineno == 2:
-                    try:
-                        tokens = line.split()
-                        pspcod, pspxc = map(int, tokens[:2])
-                    except Exception:
-                        msg = "%s: Cannot parse pspcod, pspxc in line\n %s" % (
-                            filename,
-                            line,
-                        )
-                        logger.critical(msg)
-                        return None
+            if lineno == 2:
+                try:
+                    tokens = line.split()
+                    pspcod, pspxc = map(int, tokens[:2])
+                except Exception:
+                    msg = f"{filename}: Cannot parse pspcod, pspxc in line\n {line}"
+                    logger.critical(msg)
+                    return None
 
-                    # if tokens[-1].strip().replace(" ","") not in ["pspcod,pspxc,lmax,lloc,mmax,r2well",
-                    #                              "pspcod,pspxc,lmax,llocal,mmax,r2well"]:
-                    #    raise self.Error("%s: Invalid line\n %s"  % (filename, line))
+                # if tokens[-1].strip().replace(" ","") not in ["pspcod,pspxc,lmax,lloc,mmax,r2well",
+                #                              "pspcod,pspxc,lmax,llocal,mmax,r2well"]:
+                #    raise self.Error("%s: Invalid line\n %s"  % (filename, line))
+                #    return None
+
+                if pspcod not in self._PSPCODES:
+                    raise self.Error(f"{filename}: Don't know how to handle pspcod {pspcod}\n")
+
+                ppdesc = self._PSPCODES[pspcod]
+
+                if pspcod == 7:
+                    # PAW -> need to know the format pspfmt
+                    tokens = lines[lineno + 1].split()
+                    pspfmt, creatorID = tokens[:2]
+                    # if tokens[-1].strip() != "pspfmt,creatorID":
+                    #    raise self.Error("%s: Invalid line\n %s" % (filename, line))
                     #    return None
 
-                    if pspcod not in self._PSPCODES:
-                        raise self.Error(
-                            "%s: Don't know how to handle pspcod %s\n"
-                            % (filename, pspcod)
-                        )
+                    ppdesc = ppdesc._replace(format=pspfmt)
 
-                    ppdesc = self._PSPCODES[pspcod]
+                return ppdesc
 
-                    if pspcod == 7:
-                        # PAW -> need to know the format pspfmt
-                        tokens = lines[lineno + 1].split()
-                        pspfmt, creatorID = tokens[:2]
-                        # if tokens[-1].strip() != "pspfmt,creatorID":
-                        #    raise self.Error("%s: Invalid line\n %s" % (filename, line))
-                        #    return None
-
-                        ppdesc = ppdesc._replace(format=pspfmt)
-
-                    return ppdesc
-
-            return None
+        return None
 
     def parse(self, filename):
         """
@@ -1198,7 +1188,7 @@ class PseudoParser:
         ppdesc = self.read_ppdesc(path)
 
         if ppdesc is None:
-            logger.critical("Cannot find ppdesc in %s" % path)
+            logger.critical(f"Cannot find ppdesc in {path}")
             return None
 
         psp_type = ppdesc.psp_type
@@ -1231,11 +1221,21 @@ class PseudoParser:
 
 # TODO use RadialFunction from pseudo_dojo.
 class RadialFunction(namedtuple("RadialFunction", "mesh values")):
-    pass
+    """
+    Radial Function class.
+    """
 
 
 class PawXmlSetup(Pseudo, PawPseudo):
+    """
+    Setup class for PawXml.
+    """
+
     def __init__(self, filepath):
+        """
+        :param filepath:
+        """
+        # pylint: disable=E1101
         self.path = os.path.abspath(filepath)
 
         # Get the XML root (this trick is used to that the object is pickleable).
@@ -1249,9 +1249,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
 
         # self._symbol = atom_attrib["symbol"]
         self._zatom = int(float(atom_attrib["Z"]))
-        self.core, self.valence = map(
-            float, [atom_attrib["core"], atom_attrib["valence"]]
-        )
+        self.core, self.valence = map(float, [atom_attrib["core"], atom_attrib["valence"]])
 
         # Build xc from header.
         xc_info = root.find("xc_functional").attrib
@@ -1282,7 +1280,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
         # In this way, we know that only the first two bound states (with f and n attributes)
         # should be used for constructing an initial guess for the wave functions.
 
-        self.valence_states = OrderedDict()
+        self.valence_states = {}
         for node in root.find("valence_states"):
             attrib = AttrDict(node.attrib)
             assert attrib.id not in self.valence_states
@@ -1308,6 +1306,9 @@ class PawXmlSetup(Pseudo, PawPseudo):
 
     @lazy_property
     def root(self):
+        """
+        Root tree of XML.
+        """
         from xml.etree import cElementTree as Et
 
         tree = Et.parse(self.filepath)
@@ -1380,12 +1381,13 @@ class PawXmlSetup(Pseudo, PawPseudo):
             mesh = [(i / n + a) ** 5 / a - a ** 4 for i in indices]
 
         else:
-            raise ValueError("Unknown grid type: %s" % eq)
+            raise ValueError(f"Unknown grid type: {eq}")
 
         return np.array(mesh)
 
     def _parse_radfunc(self, func_name):
-        """Parse the first occurence of func_name in the XML file."""
+        """Parse the first occurrence of func_name in the XML file."""
+        # pylint: disable=E1101
         node = self.root.find(func_name)
         grid = node.attrib["grid"]
         values = np.array([float(s) for s in node.text.split()])
@@ -1394,6 +1396,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
 
     def _parse_all_radfuncs(self, func_name):
         """Parse all the nodes with tag func_name in the XML file."""
+        # pylint: disable=E1101
         for node in self.root.findall(func_name):
             grid = node.attrib["grid"]
             values = np.array([float(s) for s in node.text.split()])
@@ -1415,7 +1418,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
     @lazy_property
     def ae_partial_waves(self):
         """Dictionary with the AE partial waves indexed by state."""
-        ae_partial_waves = OrderedDict()
+        ae_partial_waves = {}
         for mesh, values, attrib in self._parse_all_radfuncs("ae_partial_wave"):
             state = attrib["state"]
             # val_state = self.valence_states[state]
@@ -1426,7 +1429,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
     @property
     def pseudo_partial_waves(self):
         """Dictionary with the pseudo partial waves indexed by state."""
-        pseudo_partial_waves = OrderedDict()
+        pseudo_partial_waves = {}
         for (mesh, values, attrib) in self._parse_all_radfuncs("pseudo_partial_wave"):
             state = attrib["state"]
             # val_state = self.valence_states[state]
@@ -1437,7 +1440,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
     @lazy_property
     def projector_functions(self):
         """Dictionary with the PAW projectors indexed by state."""
-        projector_functions = OrderedDict()
+        projector_functions = {}
         for (mesh, values, attrib) in self._parse_all_radfuncs("projector_function"):
             state = attrib["state"]
             # val_state = self.valence_states[state]
@@ -1491,6 +1494,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
 
         Returns: `matplotlib` figure
         """
+        # pylint: disable=E1101
         ax, fig, plt = get_ax_fig_plt(ax)
 
         ax.grid(True)
@@ -1501,14 +1505,10 @@ class PawXmlSetup(Pseudo, PawPseudo):
         # ax.annotate("$r_c$", xy=(self.paw_radius + 0.1, 0.1))
 
         for state, rfunc in self.pseudo_partial_waves.items():
-            ax.plot(
-                rfunc.mesh, rfunc.mesh * rfunc.values, lw=2, label="PS-WAVE: " + state
-            )
+            ax.plot(rfunc.mesh, rfunc.mesh * rfunc.values, lw=2, label="PS-WAVE: " + state)
 
         for state, rfunc in self.ae_partial_waves.items():
-            ax.plot(
-                rfunc.mesh, rfunc.mesh * rfunc.values, lw=2, label="AE-WAVE: " + state
-            )
+            ax.plot(rfunc.mesh, rfunc.mesh * rfunc.values, lw=2, label="AE-WAVE: " + state)
 
         ax.legend(loc="best", shadow=True, fontsize=fontsize)
 
@@ -1524,6 +1524,7 @@ class PawXmlSetup(Pseudo, PawPseudo):
 
         Returns: `matplotlib` figure
         """
+        # pylint: disable=E1101
         ax, fig, plt = get_ax_fig_plt(ax)
         ax.grid(True)
         ax.set_xlabel("r [Bohr]")
@@ -1630,13 +1631,13 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
                         if p:
                             pseudos.append(p)
                         else:
-                            logger.info("Skipping file %s" % f)
+                            logger.info(f"Skipping file {f}")
                     except Exception:
-                        logger.info("Skipping file %s" % f)
+                        logger.info(f"Skipping file {f}")
             if not pseudos:
-                logger.warning("No pseudopotentials parsed from folder %s" % top)
+                logger.warning(f"No pseudopotentials parsed from folder {top}")
                 return None
-            logger.info("Creating PseudoTable with %i pseudopotentials" % len(pseudos))
+            logger.info(f"Creating PseudoTable with {len(pseudos)} pseudopotentials")
 
         else:
             if exts is None:
@@ -1646,7 +1647,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
                 try:
                     pseudos.append(Pseudo.from_file(p))
                 except Exception as exc:
-                    logger.critical("Error in %s:\n%s" % (p, exc))
+                    logger.critical(f"Error in {p}:\n{exc}")
 
         return cls(pseudos).sort_by_z()
 
@@ -1677,9 +1678,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
             symbols = [p.symbol for p in pseudo_list]
             symbol = symbols[0]
             if any(symb != symbol for symb in symbols):
-                raise ValueError(
-                    "All symbols must be equal while they are: %s" % str(symbols)
-                )
+                raise ValueError(f"All symbols must be equal while they are: {str(symbols)}")
 
             setattr(self, symbol, pseudo_list)
 
@@ -1693,8 +1692,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
             for znum in iterator_from_slice(Z):
                 pseudos.extend(self._pseudos_with_z[znum])
             return self.__class__(pseudos)
-        else:
-            return self.__class__(self._pseudos_with_z[Z])
+        return self.__class__(self._pseudos_with_z[Z])
 
     def __len__(self):
         return len(list(self.__iter__()))
@@ -1702,11 +1700,10 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
     def __iter__(self):
         """Process the elements in Z order."""
         for z in self.zlist:
-            for pseudo in self._pseudos_with_z[z]:
-                yield pseudo
+            yield from self._pseudos_with_z[z]
 
     def __repr__(self):
-        return "<%s at %s>" % (self.__class__.__name__, id(self))
+        return f"<{self.__class__.__name__} at {id(self)}>"
 
     def __str__(self):
         return self.to_table()
@@ -1776,7 +1773,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
 
             table.all_combinations_for_elements(["Li", "F"])
         """
-        d = OrderedDict()
+        d = {}
         for symbol in element_symbols:
             d[symbol] = self.select_symbols(symbol, ret_list=True)
 
@@ -1794,43 +1791,33 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
                 if multiple occurrences are found. Use allow_multi to prevent this.
 
         Raises:
-            ValueError if symbol is not found or multiple occurences are present and not allow_multi
+            ValueError if symbol is not found or multiple occurrences are present and not allow_multi
         """
         pseudos = self.select_symbols(symbol, ret_list=True)
         if not pseudos or (len(pseudos) > 1 and not allow_multi):
-            raise ValueError(
-                "Found %d occurrences of symbol %s" % (len(pseudos), symbol)
-            )
+            raise ValueError(f"Found {len(pseudos)} occurrences of symbol {symbol}")
 
         if not allow_multi:
             return pseudos[0]
-        else:
-            return pseudos
+        return pseudos
 
     def pseudos_with_symbols(self, symbols):
         """
         Return the pseudos with the given chemical symbols.
 
         Raises:
-            ValueError if one of the symbols is not found or multiple occurences are present.
+            ValueError if one of the symbols is not found or multiple occurrences are present.
         """
         pseudos = self.select_symbols(symbols, ret_list=True)
         found_symbols = [p.symbol for p in pseudos]
-        duplicated_elements = [
-            s for s, o in collections.Counter(found_symbols).items() if o > 1
-        ]
+        duplicated_elements = [s for s, o in collections.Counter(found_symbols).items() if o > 1]
 
         if duplicated_elements:
-            raise ValueError(
-                "Found multiple occurrences of symbol(s) %s"
-                % ", ".join(duplicated_elements)
-            )
+            raise ValueError(f"Found multiple occurrences of symbol(s) {', '.join(duplicated_elements)}")
         missing_symbols = [s for s in symbols if s not in found_symbols]
 
         if missing_symbols:
-            raise ValueError(
-                "Missing data for symbol(s) %s" % ", ".join(missing_symbols)
-            )
+            raise ValueError(f"Missing data for symbol(s) {', '.join(missing_symbols)}")
 
         return pseudos
 
@@ -1848,9 +1835,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
 
         if exclude:
             if not all(s.startswith("-") for s in symbols):
-                raise ValueError(
-                    "When excluding symbols, all strings must start with `-`"
-                )
+                raise ValueError("When excluding symbols, all strings must start with `-`")
             symbols = [s[1:] for s in symbols]
 
         symbols = set(symbols)
@@ -1867,8 +1852,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
 
         if ret_list:
             return pseudos
-        else:
-            return self.__class__(pseudos)
+        return self.__class__(pseudos)
 
     def get_pseudos_for_structure(self, structure):
         """
@@ -1879,7 +1863,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
 
         Raises:
             `ValueError` if one of the chemical symbols is not found or
-            multiple occurences are present in the table.
+            multiple occurrences are present in the table.
         """
         return self.pseudos_with_symbols(structure.symbol_set)
 
@@ -1902,9 +1886,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
         for p in self:
             if filter_function is not None and filter_function(p):
                 continue
-            table.append(
-                [p.basename, p.symbol, p.Z_val, p.l_max, p.l_local, p.xc, p.type]
-            )
+            table.append([p.basename, p.symbol, p.Z_val, p.l_max, p.l_local, p.xc, p.type])
         return tabulate(
             table,
             headers=["basename", "symbol", "Z_val", "l_max", "l_local", "XC", "type"],
@@ -1927,9 +1909,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
             attrs.append((i, a))
 
         # Sort attrs, and build new table with sorted pseudos.
-        return self.__class__(
-            [self[a[0]] for a in sorted(attrs, key=lambda t: t[1], reverse=reverse)]
-        )
+        return self.__class__([self[a[0]] for a in sorted(attrs, key=lambda t: t[1], reverse=reverse)])
 
     def sort_by_z(self):
         """Return a new :class:`PseudoTable` with pseudos sorted by Z"""
@@ -1961,7 +1941,7 @@ class PseudoTable(collections.abc.Sequence, MSONable, metaclass=abc.ABCMeta):
 
     def select_family(self, family):
         """
-        Return PseudoTable with element beloging to the specified family, e.g. familiy="alkaline"
+        Return PseudoTable with element belonging to the specified family, e.g. family="alkaline"
         """
         # e.g element.is_alkaline
         return self.__class__([p for p in self if getattr(p.element, "is_" + family)])
