@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -20,35 +19,28 @@ more info.
 import itertools
 import re
 import warnings
-from collections import OrderedDict
 from io import StringIO
 from pathlib import Path
 
-try:
-    import ruamel.yaml as yaml
-except ImportError:
-    try:
-        import ruamel_yaml as yaml  # type: ignore  # noqa
-    except ImportError:
-        import yaml  # type: ignore # noqa
 import numpy as np
 import pandas as pd
 from monty.dev import deprecated
 from monty.json import MSONable
 from monty.serialization import loadfn
+from ruamel.yaml import YAML
 
-from pymatgen.core.periodic_table import Element
 from pymatgen.core.lattice import Lattice
-from pymatgen.core.structure import Molecule, Structure
 from pymatgen.core.operations import SymmOp
+from pymatgen.core.periodic_table import Element
+from pymatgen.core.structure import Molecule, Structure
 from pymatgen.util.io_utils import clean_lines
 
 __author__ = "Kiran Mathew, Zhi Deng, Tingzheng Hou"
 __copyright__ = "Copyright 2018, The Materials Virtual Lab"
-__version__ = "1.0"
-__maintainer__ = "Zhi Deng"
-__email__ = "z4deng@eng.ucsd.edu"
-__date__ = "Aug 1, 2018"
+__version__ = "2.0"
+__maintainer__ = "Tingzheng Hou"
+__email__ = "tingzheng_hou@berkeley.edu"
+__date__ = "May 29, 2021"
 
 MODULE_DIR = Path(__file__).resolve().parent
 
@@ -134,14 +126,14 @@ class LammpsBox(MSONable):
         assert bounds_arr.shape == (
             3,
             2,
-        ), "Expecting a (3, 2) array for bounds," " got {}".format(bounds_arr.shape)
+        ), f"Expecting a (3, 2) array for bounds, got {bounds_arr.shape}"
         self.bounds = bounds_arr.tolist()
         matrix = np.diag(bounds_arr[:, 1] - bounds_arr[:, 0])
 
         self.tilt = None
         if tilt is not None:
             tilt_arr = np.array(tilt)
-            assert tilt_arr.shape == (3,), "Expecting a (3,) array for box_tilt," " got {}".format(tilt_arr.shape)
+            assert tilt_arr.shape == (3,), f"Expecting a (3,) array for box_tilt, got {tilt_arr.shape}"
             self.tilt = tilt_arr.tolist()
             matrix[1, 0] = tilt_arr[0]
             matrix[2, 0] = tilt_arr[1]
@@ -344,7 +336,7 @@ class LammpsData(MSONable):
             site_properties=site_properties,
         )
 
-    def get_string(self, distance=6, velocity=8, charge=4):
+    def get_string(self, distance=6, velocity=8, charge=4, hybrid=True):
         """
         Returns the string representation of LammpsData, essentially
         the string to be written to a file. Support hybrid style
@@ -357,7 +349,12 @@ class LammpsData(MSONable):
             velocity (int): No. of significant figures to output for
                 velocities. Default to 8.
             charge (int): No. of significant figures to output for
-                charges. Default to 3.
+                charges. Default to 4.
+            hybrid (bool): Whether to write hybrid coeffs types.
+                Default to True. If the data object has no hybrid
+                coeffs types and has large coeffs section, one may
+                use False to speedup the process. Otherwise the
+                default is recommended.
 
         Returns:
             String representation
@@ -373,9 +370,9 @@ class LammpsData(MSONable):
 """
         box = self.box.get_string(distance)
 
-        body_dict = OrderedDict()
+        body_dict = {}
         body_dict["Masses"] = self.masses
-        types = OrderedDict()
+        types = {}
         types["atom"] = len(self.masses)
         if self.force_field:
             all_ff_kws = SECTION_KEYWORDS["ff"] + SECTION_KEYWORDS["class2"]
@@ -386,7 +383,7 @@ class LammpsData(MSONable):
                     types[kw.lower()[:-7]] = len(self.force_field[kw])
 
         body_dict["Atoms"] = self.atoms
-        counts = OrderedDict()
+        counts = {}
         counts["atoms"] = len(self.atoms)
         if self.velocities is not None:
             body_dict["Velocities"] = self.velocities
@@ -441,12 +438,16 @@ class LammpsData(MSONable):
         parts = []
         for k, v in body_dict.items():
             index = k != "PairIJ Coeffs"
-            if k in [
-                "Bond Coeffs",
-                "Angle Coeffs",
-                "Dihedral Coeffs",
-                "Improper Coeffs",
-            ]:
+            if (
+                k
+                in [
+                    "Bond Coeffs",
+                    "Angle Coeffs",
+                    "Dihedral Coeffs",
+                    "Improper Coeffs",
+                ]
+                and hybrid
+            ):
                 listofdf = np.array_split(v, len(v.index))
                 df_string = ""
                 for i, df in enumerate(listofdf):
@@ -499,7 +500,7 @@ class LammpsData(MSONable):
             velocity (int): No. of significant figures to output for
                 velocities. Default to 8.
             charge (int): No. of significant figures to output for
-                charges. Default to 3.
+                charges. Default to 4.
 
         """
         with open(filename, "w") as f:
@@ -522,7 +523,7 @@ class LammpsData(MSONable):
             atom_labels ([str]): List of strings (must be different
                 from one another) for labelling each atom type found in
                 Masses section. Default to None, where the labels are
-                automaticaly added based on either element guess or
+                automatically added based on either element guess or
                 dummy specie assignment.
             guess_element (bool): Whether to guess the element based on
                 its atomic mass. Default to True, otherwise dummy
@@ -562,14 +563,14 @@ class LammpsData(MSONable):
             atomic_numbers = np.argmin(diff, axis=1) + 1
             symbols = [Element.from_Z(an).symbol for an in atomic_numbers]
         else:
-            symbols = ["Q%s" % a for a in map(chr, range(97, 97 + len(unique_masses)))]
+            symbols = [f"Q{a}" for a in map(chr, range(97, 97 + len(unique_masses)))]
         for um, s in zip(unique_masses, symbols):
             masses.loc[masses["mass"] == um, "element"] = s
         if atom_labels is None:  # add unique labels based on elements
             for el, vc in masses["element"].value_counts().iteritems():
-                masses.loc[masses["element"] == el, "label"] = ["%s%d" % (el, c) for c in range(1, vc + 1)]
+                masses.loc[masses["element"] == el, "label"] = [f"{el}{c}" for c in range(1, vc + 1)]
         assert masses["label"].nunique(dropna=False) == len(masses), "Expecting unique atom label for each type"
-        mass_info = [tuple([r["label"], r["mass"]]) for _, r in masses.iterrows()]
+        mass_info = [(row.label, row.mass) for row in masses.itertuples()]
 
         nonbond_coeffs, topo_coeffs = None, None
         if self.force_field:
@@ -606,9 +607,9 @@ class LammpsData(MSONable):
                     topo_idx = topo[0] - 1
                     indices = list(topo[1:])
                     mids = atoms_df.loc[indices]["molecule-ID"].unique()
-                    assert len(mids) == 1, (
-                        "Do not support intermolecular topology formed " "by atoms with different molecule-IDs"
-                    )
+                    assert (
+                        len(mids) == 1
+                    ), "Do not support intermolecular topology formed by atoms with different molecule-IDs"
                     label = label_topo(indices)
                     topo_coeffs[ff_kw][topo_idx]["types"].append(label)
                     if data_by_mols[mids[0]].get(k):
@@ -671,7 +672,7 @@ class LammpsData(MSONable):
         parts = np.split(lines, section_marks)
 
         float_group = r"([0-9eE.+-]+)"
-        header_pattern = dict()
+        header_pattern = {}
         header_pattern["counts"] = r"^\s*(\d+)\s+([a-zA-Z]+)$"
         header_pattern["types"] = r"^\s*(\d+)\s+([a-zA-Z]+)\s+types$"
         header_pattern["bounds"] = r"^\s*{}$".format(r"\s+".join([float_group] * 2 + [r"([xyz])lo \3hi"]))
@@ -706,24 +707,24 @@ class LammpsData(MSONable):
                     if line.strip()
                 ]
                 df = pd.concat(df_list, ignore_index=True)
-                names = ["id"] + ["coeff%d" % i for i in range(1, df.shape[1])]
+                names = ["id"] + [f"coeff{i}" for i in range(1, df.shape[1])]
             else:
                 df = pd.read_csv(sio, header=None, comment="#", delim_whitespace=True)
                 if kw == "PairIJ Coeffs":
-                    names = ["id1", "id2"] + ["coeff%d" % i for i in range(1, df.shape[1] - 1)]
-                    df.index.name = None
+                    names = ["id1", "id2"] + [f"coeff{i}" for i in range(1, df.shape[1] - 1)]
+                    df.index.name = None  # pylint: disable=E1101
                 elif kw in SECTION_HEADERS:
                     names = ["id"] + SECTION_HEADERS[kw]
                 elif kw == "Atoms":
                     names = ["id"] + ATOMS_HEADERS[atom_style]
-                    if df.shape[1] == len(names):
+                    if df.shape[1] == len(names):  # pylint: disable=E1101
                         pass
-                    elif df.shape[1] == len(names) + 3:
+                    elif df.shape[1] == len(names) + 3:  # pylint: disable=E1101
                         names += ["nx", "ny", "nz"]
                     else:
-                        raise ValueError("Format in Atoms section inconsistent" " with atom_style %s" % atom_style)
+                        raise ValueError(f"Format in Atoms section inconsistent with atom_style {atom_style}")
                 else:
-                    raise NotImplementedError("Parser for %s section" " not implemented" % kw)
+                    raise NotImplementedError(f"Parser for {kw} section not implemented")
             df.columns = names
             if sort_id:
                 sort_by = "id" if kw != "PairIJ Coeffs" else ["id1", "id2"]
@@ -743,7 +744,7 @@ class LammpsData(MSONable):
             if (
                 name in ["Velocities"] + SECTION_KEYWORDS["topology"] and not seen_atoms
             ):  # Atoms must appear earlier than these
-                raise RuntimeError(err_msg + "%s section appears before" " Atoms section" % name)
+                raise RuntimeError(err_msg + f"{name} section appears before Atoms section")
             body.update({name: section})
 
         err_msg += "Nos. of {} do not match between header and {} section"
@@ -782,7 +783,7 @@ class LammpsData(MSONable):
             atom_style (str): Output atom_style. Default to "full".
 
         """
-        atom_types = set.union(*[t.species for t in topologies])
+        atom_types = set.union(*(t.species for t in topologies))
         assert atom_types.issubset(ff.maps["Atoms"].keys()), "Unknown atom type found in topologies"
 
         items = dict(box=box, atom_style=atom_style, masses=ff.masses, force_field=ff.force_field)
@@ -822,7 +823,7 @@ class LammpsData(MSONable):
             df = pd.DataFrame(np.concatenate(topo_collector[k]), columns=SECTION_HEADERS[k][1:])
             df["type"] = list(map(ff.maps[k].get, topo_labels[k]))
             if any(pd.isnull(df["type"])):  # Throw away undefined topologies
-                warnings.warn("Undefined %s detected and removed" % k.lower())
+                warnings.warn(f"Undefined {k.lower()} detected and removed")
                 df.dropna(subset=["type"], inplace=True)
                 df.reset_index(drop=True, inplace=True)
             df.index += 1
@@ -877,60 +878,6 @@ class LammpsData(MSONable):
         topo = Topology(boxed_s)
         return cls.from_ff_and_topologies(box=box, ff=ff, topologies=[topo], atom_style=atom_style)
 
-    @classmethod
-    def from_dict(cls, d):
-        """
-        Constructor that reads in a dictionary.
-
-        Args:
-            d (dict): Dictionary to read.
-        """
-
-        def decode_df(s):
-            return pd.read_json(s, orient="split")
-
-        items = dict()
-        items["box"] = LammpsBox.from_dict(d["box"])
-        items["masses"] = decode_df(d["masses"])
-        items["atoms"] = decode_df(d["atoms"])
-        items["atom_style"] = d["atom_style"]
-
-        velocities = d["velocities"]
-        if velocities:
-            velocities = decode_df(velocities)
-        items["velocities"] = velocities
-        force_field = d["force_field"]
-        if force_field:
-            force_field = {k: decode_df(v) for k, v in force_field.items()}
-        items["force_field"] = force_field
-        topology = d["topology"]
-        if topology:
-            topology = {k: decode_df(v) for k, v in topology.items()}
-        items["topology"] = topology
-        return cls(**items)
-
-    def as_dict(self):
-        """
-        Returns the LammpsData as a dict.
-
-        """
-
-        def encode_df(df):
-            return df.to_json(orient="split")
-
-        d = dict()
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
-        d["box"] = self.box.as_dict()
-        d["masses"] = encode_df(self.masses)
-        d["atoms"] = encode_df(self.atoms)
-        d["atom_style"] = self.atom_style
-
-        d["velocities"] = None if self.velocities is None else encode_df(self.velocities)
-        d["force_field"] = None if not self.force_field else {k: encode_df(v) for k, v in self.force_field.items()}
-        d["topology"] = None if not self.topology else {k: encode_df(v) for k, v in self.topology.items()}
-        return d
-
 
 class Topology(MSONable):
     """
@@ -1081,11 +1028,11 @@ class ForceField(MSONable):
         """
 
         Args:
-            mass_into (list): List of atomic mass info. Elements,
+            mass_info (list): List of atomic mass info. Elements,
                 strings (symbols) and floats are all acceptable for the
                 values, with the first two converted to the atomic mass
                 of an element. It is recommended to use
-                OrderedDict.items() to prevent key duplications.
+                dict.items() to prevent key duplications.
                 [("C", 12.01), ("H", Element("H")), ("O", "O"), ...]
             nonbond_coeffs [coeffs]: List of pair or pairij
                 coefficients, of which the sequence must be sorted
@@ -1160,7 +1107,7 @@ class ForceField(MSONable):
         pair_df = pd.DataFrame(self.nonbond_coeffs)
         assert self._is_valid(pair_df), "Invalid nonbond coefficients with rows varying in length"
         npair, ncoeff = pair_df.shape
-        pair_df.columns = ["coeff%d" % i for i in range(1, ncoeff + 1)]
+        pair_df.columns = [f"coeff{i}" for i in range(1, ncoeff + 1)]
         nm = len(self.mass_info)
         ncomb = int(nm * (nm + 1) / 2)
         if npair == nm:
@@ -1194,12 +1141,12 @@ class ForceField(MSONable):
             distinct_types.append(d["types"])
             for k in class2_data.keys():
                 class2_data[k].append(d[k])
-        distinct_types = [set(itertools.chain(*[find_eq_types(t, kw) for t in dt])) for dt in distinct_types]
-        type_counts = sum([len(dt) for dt in distinct_types])
+        distinct_types = [set(itertools.chain(*(find_eq_types(t, kw) for t in dt))) for dt in distinct_types]
+        type_counts = sum(len(dt) for dt in distinct_types)
         type_union = set.union(*distinct_types)
-        assert len(type_union) == type_counts, "Duplicated items found " "under different coefficients in %s" % kw
+        assert len(type_union) == type_counts, f"Duplicated items found under different coefficients in {kw}"
         atoms = set(np.ravel(list(itertools.chain(*distinct_types))))
-        assert atoms.issubset(self.maps["Atoms"].keys()), "Undefined atom type found in %s" % kw
+        assert atoms.issubset(self.maps["Atoms"].keys()), f"Undefined atom type found in {kw}"
         mapper = {}
         for i, dt in enumerate(distinct_types):
             for t in dt:
@@ -1209,7 +1156,7 @@ class ForceField(MSONable):
             df = pd.DataFrame(data)
             assert self._is_valid(df), "Invalid coefficients with rows varying in length"
             n, c = df.shape
-            df.columns = ["coeff%d" % i for i in range(1, c + 1)]
+            df.columns = [f"coeff{i}" for i in range(1, c + 1)]
             df.index = range(1, n + 1)
             return df
 
@@ -1231,9 +1178,9 @@ class ForceField(MSONable):
             "nonbond_coeffs": self.nonbond_coeffs,
             "topo_coeffs": self.topo_coeffs,
         }
-        yml = yaml.YAML(typ="safe")
         with open(filename, "w") as f:
-            yml.dump(d, f)
+            yaml = YAML()
+            yaml.dump(d, f)
 
     @classmethod
     def from_file(cls, filename):
@@ -1244,9 +1191,9 @@ class ForceField(MSONable):
             filename (str): Filename.
 
         """
-        yml = yaml.YAML(typ="safe")
-        with open(filename, "r") as f:
-            d = yml.load(f)
+        with open(filename) as f:
+            yaml = YAML()
+            d = yaml.load(f)
         return cls.from_dict(d)
 
     @classmethod
@@ -1268,7 +1215,7 @@ class ForceField(MSONable):
 class CombinedData(LammpsData):
     """
     Object for a collective set of data for a series of LAMMPS data file.
-    velocities not yet implementd.
+    velocities not yet implemented.
     """
 
     def __init__(
@@ -1281,27 +1228,38 @@ class CombinedData(LammpsData):
     ):
         """
         Args:
-            list_of_molecules: a list of LammpsData of a single cluster.
-            list_of_names: a list of name for each cluster.
-            list_of_numbers: a list of Integer for counts of each molecule
-                coordinates (pandas.DataFrame): DataFrame with with four
-                columns ["atom", "x", "y", "z"] for coordinates of atoms.
+            list_of_molecules: A list of LammpsData objects of a chemical cluster.
+                 Each LammpsData object (cluster) may contain one or more molecule ID.
+            list_of_names: A list of name (string) for each cluster. The characters in each name are
+                restricted to word characters ([a-zA-Z0-9_]). If names with any non-word characters
+                are passed in, the special characters will be substituted by '_'.
+            list_of_numbers: A list of Integer for counts of each molecule
+            coordinates (pandas.DataFrame): DataFrame at least containing
+                columns of ["x", "y", "z"] for coordinates of atoms.
             atom_style (str): Output atom_style. Default to "full".
 
         """
 
-        max_xyz = coordinates[["x", "y", "z"]].max().max()
-        min_xyz = coordinates[["x", "y", "z"]].min().min()
+        self._list_of_molecules = list_of_molecules
+        self._list_of_names = list_of_names
+        self._list_of_numbers = list_of_numbers
+        self._coordinates = coordinates
+        self._coordinates.index = self._coordinates.index.map(int)
+        max_xyz = self._coordinates[["x", "y", "z"]].max().max()
+        min_xyz = self._coordinates[["x", "y", "z"]].min().min()
         self.box = LammpsBox(np.array(3 * [[min_xyz - 0.5, max_xyz + 0.5]]))
         self.atom_style = atom_style
-        self.n = sum(list_of_numbers)
-        self.names = list_of_names
-        self.mols = list_of_molecules
-        self.nums = list_of_numbers
+        self.n = sum(self._list_of_numbers)
+        self.names = []
+        for name in self._list_of_names:
+            self.names.append("_".join(re.findall(r"\w+", name)))
+        self.mols = self._list_of_molecules
+        self.nums = self._list_of_numbers
         self.masses = pd.concat([mol.masses.copy() for mol in self.mols], ignore_index=True)
         self.masses.index += 1
         all_ff_kws = SECTION_KEYWORDS["ff"] + SECTION_KEYWORDS["class2"]
-        ff_kws = [k for k in all_ff_kws if k in self.mols[0].force_field]
+        appeared_kws = {k for mol in self.mols if mol.force_field is not None for k in mol.force_field}
+        ff_kws = [k for k in all_ff_kws if k in appeared_kws]
         self.force_field = {}
         for kw in ff_kws:
             self.force_field[kw] = pd.concat(
@@ -1309,22 +1267,27 @@ class CombinedData(LammpsData):
                 ignore_index=True,
             )
             self.force_field[kw].index += 1
+        if not bool(self.force_field):
+            self.force_field = None
 
         self.atoms = pd.DataFrame()
         mol_count = 0
         type_count = 0
+        self.mols_per_data = []
         for i, mol in enumerate(self.mols):
             atoms_df = mol.atoms.copy()
             atoms_df["molecule-ID"] += mol_count
             atoms_df["type"] += type_count
+            mols_in_data = len(atoms_df["molecule-ID"].unique())
+            self.mols_per_data.append(mols_in_data)
             for j in range(self.nums[i]):
                 self.atoms = self.atoms.append(atoms_df, ignore_index=True)
-                atoms_df["molecule-ID"] += 1
+                atoms_df["molecule-ID"] += mols_in_data
             type_count += len(mol.masses)
-            mol_count += self.nums[i]
+            mol_count += self.nums[i] * mols_in_data
         self.atoms.index += 1
-        assert len(self.atoms) == len(coordinates), "Wrong number of coordinates."
-        self.atoms.update(coordinates)
+        assert len(self.atoms) == len(self._coordinates), "Wrong number of coordinates."
+        self.atoms.update(self._coordinates)
 
         self.velocities = None
         assert self.mols[0].velocities is None, "Velocities not supported"
@@ -1334,7 +1297,7 @@ class CombinedData(LammpsData):
         count = {"Bonds": 0, "Angles": 0, "Dihedrals": 0, "Impropers": 0}
         for i, mol in enumerate(self.mols):
             for kw in SECTION_KEYWORDS["topology"]:
-                if kw in mol.topology:
+                if bool(mol.topology) and kw in mol.topology:
                     if kw not in self.topology:
                         self.topology[kw] = pd.DataFrame()
                     topo_df = mol.topology[kw].copy()
@@ -1350,6 +1313,68 @@ class CombinedData(LammpsData):
         for kw in SECTION_KEYWORDS["topology"]:
             if kw in self.topology:
                 self.topology[kw].index += 1
+        if not bool(self.topology):
+            self.topology = None
+
+    @property
+    def structure(self):
+        """
+        Exports a periodic structure object representing the simulation
+        box.
+        Return:
+            Structure
+        """
+        ld_cp = self.as_lammpsdata()
+        return ld_cp.structure
+
+    def disassemble(self, atom_labels=None, guess_element=True, ff_label="ff_map"):
+        """
+        Breaks down each LammpsData in CombinedData to building blocks
+        (LammpsBox, ForceField and a series of Topology).
+        RESTRICTIONS APPLIED:
+        1. No complex force field defined not just on atom
+            types, where the same type or equivalent types of topology
+            may have more than one set of coefficients.
+        2. No intermolecular topologies (with atoms from different
+            molecule-ID) since a Topology object includes data for ONE
+            molecule or structure only.
+        Args:
+            atom_labels ([str]): List of strings (must be different
+                from one another) for labelling each atom type found in
+                Masses section. Default to None, where the labels are
+                automatically added based on either element guess or
+                dummy specie assignment.
+            guess_element (bool): Whether to guess the element based on
+                its atomic mass. Default to True, otherwise dummy
+                species "Qa", "Qb", ... will be assigned to various
+                atom types. The guessed or assigned elements will be
+                reflected on atom labels if atom_labels is None, as
+                well as on the species of molecule in each Topology.
+            ff_label (str): Site property key for labeling atoms of
+                different types. Default to "ff_map".
+        Returns:
+            [(LammpsBox, ForceField, [Topology]), ...]
+        """
+        disassembles = []
+        for mol in self.mols:
+            disassembles.append(
+                mol.disassemble(atom_labels=atom_labels, guess_element=guess_element, ff_label=ff_label)
+            )
+        return disassembles
+
+    @classmethod
+    def from_ff_and_topologies(cls):
+        """
+        Unsupported constructor for CombinedData objects.
+        """
+        raise AttributeError("Unsupported constructor for CombinedData objects.")
+
+    @classmethod
+    def from_structure(cls):
+        """
+        Unsupported constructor for CombinedData objects.
+        """
+        raise AttributeError("Unsupported constructor for CombinedData objects.")
 
     @classmethod
     def parse_xyz(cls, filename):
@@ -1383,17 +1408,17 @@ class CombinedData(LammpsData):
             coordinate_file (str): The filename of xyz coordinates.
             list_of_numbers (list): A list of numbers specifying counts for each
                 clusters parsed from files.
-            filenames (str): A series of filenames in string format.
+            filenames (str): A series of LAMMPS data filenames in string format.
         """
         names = []
         mols = []
         styles = []
         coordinates = cls.parse_xyz(filename=coordinate_file)
         for i in range(0, len(filenames)):
-            exec("cluster%d = LammpsData.from_file(filenames[i])" % (i + 1))
-            names.append("cluster%d" % (i + 1))
-            mols.append(eval("cluster%d" % (i + 1)))
-            styles.append(eval("cluster%d" % (i + 1)).atom_style)
+            exec(f"cluster{i + 1} = LammpsData.from_file(filenames[i])")
+            names.append(f"cluster{i + 1}")
+            mols.append(eval(f"cluster{i + 1}"))
+            styles.append(eval(f"cluster{i + 1}").atom_style)
         style = set(styles)
         assert len(style) == 1, "Files have different atom styles."
         return cls.from_lammpsdata(mols, names, list_of_numbers, coordinates, style.pop())
@@ -1405,11 +1430,12 @@ class CombinedData(LammpsData):
         The input LammpsData objects are used non-destructively.
 
         Args:
-            mols: a list of LammpsData of a single cluster.
+            mols: a list of LammpsData of a chemical cluster.Each LammpsData object (cluster)
+                may contain one or more molecule ID.
             names: a list of name for each cluster.
             list_of_numbers: a list of Integer for counts of each molecule
-                coordinates (pandas.DataFrame): DataFrame with with four
-                columns ["atom", "x", "y", "z"] for coordinates of atoms.
+            coordinates (pandas.DataFrame): DataFrame at least containing
+                columns of ["x", "y", "z"] for coordinates of atoms.
             atom_style (str): Output atom_style. Default to "full".
         """
         styles = []
@@ -1422,10 +1448,14 @@ class CombinedData(LammpsData):
             assert atom_style == style_return, "Data have different atom_style as specified."
         return cls(mols, names, list_of_numbers, coordinates, style_return)
 
-    def get_string(self, distance=6, velocity=8, charge=4):
+    def get_string(self, distance=6, velocity=8, charge=4, hybrid=True):
         """
         Returns the string representation of CombinedData, essentially
-        the string to be written to a file. Combination info is included.
+        the string to be written to a file. Combination info is included
+        as a comment. For single molecule ID data, the info format is:
+            num name
+        For data with multiple molecule ID, the format is:
+            num(mols_per_data) name
 
         Args:
             distance (int): No. of significant figures to output for
@@ -1434,20 +1464,56 @@ class CombinedData(LammpsData):
             velocity (int): No. of significant figures to output for
                 velocities. Default to 8.
             charge (int): No. of significant figures to output for
-                charges. Default to 3.
+                charges. Default to 4.
+            hybrid (bool): Whether to write hybrid coeffs types.
+                Default to True. If the data object has no hybrid
+                coeffs types and has large coeffs section, one may
+                use False to speedup the process. Otherwise the
+                default is recommended.
 
         Returns:
             String representation
         """
-        lines = LammpsData.get_string(self, distance, velocity, charge).splitlines()
-        info = "# " + " + ".join(str(a) + " " + b for a, b in zip(self.nums, self.names))
+        lines = LammpsData.get_string(self, distance, velocity, charge, hybrid).splitlines()
+        info = "# " + " + ".join(
+            (str(a) + " " + b) if c == 1 else (str(a) + "(" + str(c) + ") " + b)
+            for a, b, c in zip(self.nums, self.names, self.mols_per_data)
+        )
         lines.insert(1, info)
         return "\n".join(lines)
 
+    def as_lammpsdata(self):
+        """
+        Convert a CombinedData object to a LammpsData object. attributes are deepcopied.
+
+        box (LammpsBox): Simulation box.
+        force_field (dict): Data for force field sections. Optional
+            with default to None. Only keywords in force field and
+            class 2 force field are valid keys, and each value is a
+            DataFrame.
+        topology (dict): Data for topology sections. Optional with
+            default to None. Only keywords in topology are valid
+            keys, and each value is a DataFrame.
+
+        """
+        items = {}
+        items["box"] = LammpsBox(self.box.bounds, self.box.tilt)
+        items["masses"] = self.masses.copy()
+        items["atoms"] = self.atoms.copy()
+        items["atom_style"] = self.atom_style
+        items["velocities"] = None  # Velocities not supported
+        if self.force_field:
+            all_ff_kws = SECTION_KEYWORDS["ff"] + SECTION_KEYWORDS["class2"]
+            items["force_field"] = {k: v.copy() for k, v in self.force_field.items() if k in all_ff_kws}
+
+        if self.topology:
+            items["topology"] = {k: v.copy() for k, v in self.topology.items() if k in SECTION_KEYWORDS["topology"]}
+        return LammpsData(**items)
+
 
 @deprecated(
     LammpsData.from_structure,
-    "structure_2_lmpdata has been deprecated " "in favor of LammpsData.from_structure",
+    "structure_2_lmpdata has been deprecated in favor of LammpsData.from_structure",
 )
 def structure_2_lmpdata(structure, ff_elements=None, atom_style="charge", is_sort=False):
     """
