@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -6,8 +5,7 @@
 Classes for reading/manipulating/writing QChem input files.
 """
 import logging
-from typing import Union, Dict, List, Optional
-from typing_extensions import Literal
+from typing import Dict, List, Literal, Optional, Tuple, Union
 
 from monty.io import zopen
 from monty.json import MSONable
@@ -43,7 +41,10 @@ class QCInput(MSONable):
         solvent: Optional[Dict] = None,
         smx: Optional[Dict] = None,
         scan: Optional[Dict[str, List]] = None,
+        van_der_waals: Optional[Dict[str, float]] = None,
+        vdw_mode: str = "atomic",
         plots: Optional[Dict] = None,
+        nbo: Optional[Dict] = None,
     ):
         """
         Args:
@@ -56,7 +57,7 @@ class QCInput(MSONable):
                 Ex. rem = {'method': 'rimp2', 'basis': '6-31*G++' ... }
             opt (dict of lists):
                 A dictionary of opt sections, where each opt section is a key and the corresponding
-                values are a list of strings. Stings must be formatted as instructed by the QChem manual.
+                values are a list of strings. Strings must be formatted as instructed by the QChem manual.
                 The different opt sections are: CONSTRAINT, FIXED, DUMMY, and CONNECT
                 Ex. opt = {"CONSTRAINT": ["tors 2 3 4 5 25.0", "tors 2 5 7 9 80.0"], "FIXED": ["2 XY"]}
             pcm (dict):
@@ -76,6 +77,18 @@ class QCInput(MSONable):
                 CANNOT be
                 more than two.
                 Ex. scan = {"stre": ["3 6 1.5 1.9 0.1"], "tors": ["1 2 3 4 -180 180 15"]}
+            van_der_waals (dict):
+                A dictionary of custom van der Waals radii to be used when constructing cavities for the PCM
+                model or when computing, e.g. Mulliken charges. They keys are strs whose meaning depends on
+                the value of vdw_mode, and the values are the custom radii in angstroms.
+            vdw_mode (str): Method of specifying custom van der Waals radii - 'atomic' or 'sequential'.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., 12 = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
+            plots (dict):
+                    A dictionary of all the input parameters for the plots section of QChem input file.
+            nbo (dict):
+                    A dictionary of all the input parameters for the nbo section of QChem input file.
 
         """
         self.molecule = molecule
@@ -85,7 +98,10 @@ class QCInput(MSONable):
         self.solvent = lower_and_check_unique(solvent)
         self.smx = lower_and_check_unique(smx)
         self.scan = lower_and_check_unique(scan)
+        self.van_der_waals = lower_and_check_unique(van_der_waals)
+        self.vdw_mode = vdw_mode
         self.plots = lower_and_check_unique(plots)
+        self.nbo = lower_and_check_unique(nbo)
 
         # Make sure rem is valid:
         #   - Has a basis
@@ -148,10 +164,18 @@ class QCInput(MSONable):
         if self.scan:
             combined_list.append(self.scan_template(self.scan))
             combined_list.append("")
+        # section for van_der_waals radii
+        if self.van_der_waals:
+            combined_list.append(self.van_der_waals_template(self.van_der_waals, self.vdw_mode))
+            combined_list.append("")
         # plots section
         if self.plots:
             combined_list.append(self.plots_template(self.plots))
             combined_list.append("")
+        # nbo section
+        if self.nbo is not None:
+            combined_list.append(self.nbo_template(self.nbo))
+            combined_list.append("")
         return "\n".join(combined_list)
 
     @staticmethod
@@ -163,7 +187,7 @@ class QCInput(MSONable):
         Returns:
             (str) String representation of multi job input file.
         """
-        multi_job_string = str()
+        multi_job_string = ""
         for i, job_i in enumerate(job_list):
             if i < len(job_list) - 1:
                 multi_job_string += job_i.__str__() + "\n@@@\n\n"
@@ -191,7 +215,10 @@ class QCInput(MSONable):
         solvent = None
         smx = None
         scan = None
+        vdw = None
+        vdw_mode = "atomic"
         plots = None
+        nbo = None
         if "opt" in sections:
             opt = cls.read_opt(string)
         if "pcm" in sections:
@@ -202,9 +229,25 @@ class QCInput(MSONable):
             smx = cls.read_smx(string)
         if "scan" in sections:
             scan = cls.read_scan(string)
+        if "van_der_waals" in sections:
+            vdw_mode, vdw = cls.read_vdw(string)
         if "plots" in sections:
             plots = cls.read_plots(string)
-        return cls(molecule, rem, opt=opt, pcm=pcm, solvent=solvent, smx=smx, scan=scan, plots=plots)
+        if "nbo" in sections:
+            nbo = cls.read_nbo(string)
+        return cls(
+            molecule,
+            rem,
+            opt=opt,
+            solvent=solvent,
+            pcm=pcm,
+            smx=smx,
+            scan=scan,
+            van_der_waals=vdw,
+            vdw_mode=vdw_mode,
+            plots=plots,
+            nbo=nbo,
+        )
 
     def write_file(self, filename: str):
         """
@@ -276,9 +319,7 @@ class QCInput(MSONable):
             else:
                 raise ValueError('The only acceptable text value for molecule is "read"')
         else:
-            mol_list.append(
-                " {charge} {spin_mult}".format(charge=int(molecule.charge), spin_mult=molecule.spin_multiplicity)
-            )
+            mol_list.append(f" {int(molecule.charge)} {molecule.spin_multiplicity}")
             for site in molecule.sites:
                 mol_list.append(
                     " {atom}     {x: .10f}     {y: .10f}     {z: .10f}".format(
@@ -300,7 +341,7 @@ class QCInput(MSONable):
         rem_list = []
         rem_list.append("$rem")
         for key, value in rem.items():
-            rem_list.append("   {key} = {value}".format(key=key, value=value))
+            rem_list.append(f"   {key} = {value}")
         rem_list.append("$end")
         return "\n".join(rem_list)
 
@@ -319,11 +360,11 @@ class QCInput(MSONable):
         opt_list.append("$opt")
         # loops over all opt sections
         for key, value in opt.items():
-            opt_list.append("{section}".format(section=key))
+            opt_list.append(f"{key}")
             # loops over all values within the section
             for i in value:
-                opt_list.append("   {val}".format(val=i))
-            opt_list.append("END{section}".format(section=key))
+                opt_list.append(f"   {i}")
+            opt_list.append(f"END{key}")
             opt_list.append("")
         # this deletes the empty space after the last section
         del opt_list[-1]
@@ -344,7 +385,7 @@ class QCInput(MSONable):
         pcm_list = []
         pcm_list.append("$pcm")
         for key, value in pcm.items():
-            pcm_list.append("   {key} {value}".format(key=key, value=value))
+            pcm_list.append(f"   {key} {value}")
         pcm_list.append("$end")
         return "\n".join(pcm_list)
 
@@ -362,7 +403,7 @@ class QCInput(MSONable):
         solvent_list = []
         solvent_list.append("$solvent")
         for key, value in solvent.items():
-            solvent_list.append("   {key} {value}".format(key=key, value=value))
+            solvent_list.append(f"   {key} {value}")
         solvent_list.append("$end")
         return "\n".join(solvent_list)
 
@@ -379,9 +420,9 @@ class QCInput(MSONable):
         smx_list.append("$smx")
         for key, value in smx.items():
             if value == "tetrahydrofuran":
-                smx_list.append("   {key} {value}".format(key=key, value="thf"))
+                smx_list.append(f"   {key} thf")
             else:
-                smx_list.append("   {key} {value}".format(key=key, value=value))
+                smx_list.append(f"   {key} {value}")
         smx_list.append("$end")
         return "\n".join(smx_list)
 
@@ -395,18 +436,49 @@ class QCInput(MSONable):
         Returns:
             String representing Q-Chem input format for scan section
         """
-        scan_list = list()
+        scan_list = []
         scan_list.append("$scan")
-        total_vars = sum([len(v) for v in scan.values()])
+        total_vars = sum(len(v) for v in scan.values())
         if total_vars > 2:
-            raise ValueError("Q-Chem only supports PES_SCAN with two or less " "variables.")
+            raise ValueError("Q-Chem only supports PES_SCAN with two or less variables.")
         for var_type, variables in scan.items():
-            if variables not in [None, list()]:
+            if variables not in [None, []]:
                 for var in variables:
-                    scan_list.append("   {var_type} {var}".format(var_type=var_type, var=var))
+                    scan_list.append(f"   {var_type} {var}")
         scan_list.append("$end")
         return "\n".join(scan_list)
 
+    @staticmethod
+    def van_der_waals_template(radii: Dict[str, float], mode: str = "atomic") -> str:
+        """
+        Args:
+            radii (dict): Dictionary with custom van der Waals radii, in
+                Angstroms, keyed by either atomic number or sequential
+                atom number (see 'mode' kwarg).
+                Ex: {1: 1.20, 12: 1.70}
+            mode: 'atomic' or 'sequential'. In 'atomic' mode (default), dict keys
+                represent the atomic number associated with each radius (e.g., '12' = carbon).
+                In 'sequential' mode, dict keys represent the sequential position of
+                a single specific atom in the input structure.
+                **NOTE: keys must be given as strings even though they are numbers!**
+
+        Returns:
+            String representing Q-Chem input format for van_der_waals section
+        """
+        vdw_list = []
+        vdw_list.append("$van_der_waals")
+        if mode == "atomic":
+            vdw_list.append("1")
+        elif mode == "sequential":
+            vdw_list.append("2")
+        else:
+            raise ValueError(f"Invalid value {mode} given for 'mode' kwarg.")
+
+        for num, radius in radii.items():
+            vdw_list.append(f"   {num} {radius}")
+        vdw_list.append("$end")
+        return "\n".join(vdw_list)
+
     @staticmethod
     def plots_template(plots: Dict) -> str:
         """
@@ -419,10 +491,26 @@ class QCInput(MSONable):
         plots_list = []
         plots_list.append("$plots")
         for key, value in plots.items():
-            plots_list.append("   {key} {value}".format(key=key, value=value))
+            plots_list.append(f"   {key} {value}")
         plots_list.append("$end")
         return "\n".join(plots_list)
 
+    @staticmethod
+    def nbo_template(nbo: Dict) -> str:
+        """
+        Args:
+            nbo ():
+
+        Returns:
+            (str)
+        """
+        nbo_list = []
+        nbo_list.append("$nbo")
+        for key, value in nbo.items():
+            nbo_list.append(f"   {key} = {value}")
+        nbo_list.append("$end")
+        return "\n".join(nbo_list)
+
     @staticmethod
     def find_sections(string: str) -> List:
         """
@@ -434,7 +522,7 @@ class QCInput(MSONable):
         Returns:
             List of sections.
         """
-        patterns = {"sections": r"^\s*?\$([a-z]+)", "multiple_jobs": r"(@@@)"}
+        patterns = {"sections": r"^\s*?\$([a-z_]+)", "multiple_jobs": r"(@@@)"}
         matches = read_pattern(string, patterns)
         # list of the sections present
         sections = [val[0] for val in matches["sections"]]
@@ -498,7 +586,7 @@ class QCInput(MSONable):
             (dict) rem
         """
         header = r"^\s*\$rem"
-        row = r"\s*([a-zA-Z\_]+)\s*=?\s*(\S+)"
+        row = r"\s*([a-zA-Z\_\d]+)\s*=?\s*(\S+)"
         footer = r"^\s*\$end"
         rem_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
         return dict(rem_table[0])
@@ -580,11 +668,37 @@ class QCInput(MSONable):
         footer = r"^\s*\$end"
         pcm_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
         if not pcm_table:
-            print("No valid PCM inputs found. Note that there should be no '=' chracters in PCM input lines.")
+            print("No valid PCM inputs found. Note that there should be no '=' characters in PCM input lines.")
             return {}
 
         return dict(pcm_table[0])
 
+    @staticmethod
+    def read_vdw(string: str) -> Tuple[str, Dict]:
+        """
+        Read van der Waals parameters from string.
+
+        Args:
+            string (str): String
+
+        Returns:
+            (str, dict) vdW mode ('atomic' or 'sequential') and dict of van der Waals radii.
+        """
+        header = r"^\s*\$van_der_waals"
+        row = r"[^\d]*(\d+).?(\d+.\d+)?.*"
+        footer = r"^\s*\$end"
+        vdw_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
+        if not vdw_table:
+            print("No valid vdW inputs found. Note that there should be no '=' characters in vdW input lines.")
+            return "", {}
+
+        if vdw_table[0][0][0] == 2:
+            mode = "sequential"
+        else:
+            mode = "atomic"
+
+        return mode, dict(vdw_table[0][1:])
+
     @staticmethod
     def read_solvent(string: str) -> Dict:
         """
@@ -601,7 +715,7 @@ class QCInput(MSONable):
         footer = r"^\s*\$end"
         solvent_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
         if not solvent_table:
-            print("No valid solvent inputs found. Note that there should be no '=' chracters in solvent input lines.")
+            print("No valid solvent inputs found. Note that there should be no '=' characters in solvent input lines.")
             return {}
 
         return dict(solvent_table[0])
@@ -622,7 +736,7 @@ class QCInput(MSONable):
         footer = r"^\s*\$end"
         smx_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
         if not smx_table:
-            print("No valid smx inputs found. Note that there should be no '=' chracters in smx input lines.")
+            print("No valid smx inputs found. Note that there should be no '=' characters in smx input lines.")
             return {}
         smx = {}
         for key, val in smx_table[0]:
@@ -646,13 +760,13 @@ class QCInput(MSONable):
         row = r"\s*(stre|bend|tors|STRE|BEND|TORS)\s+((?:[\-\.0-9]+\s*)+)"
         footer = r"^\s*\$end"
         scan_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
-        if scan_table == list():
-            print("No valid scan inputs found. Note that there should be no '=' chracters in scan input lines.")
-            return dict()
+        if scan_table == []:
+            print("No valid scan inputs found. Note that there should be no '=' characters in scan input lines.")
+            return {}
 
-        stre = list()
-        bend = list()
-        tors = list()
+        stre = []
+        bend = []
+        tors = []
         for row in scan_table[0]:
             if row[0].lower() == "stre":
                 stre.append(row[1].replace("\n", "").rstrip())
@@ -682,9 +796,32 @@ class QCInput(MSONable):
         footer = r"^\s*\$end"
         plots_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
         if plots_table == []:
-            print("No valid plots inputs found. Note that there should be no '=' chracters in plots input lines.")
+            print("No valid plots inputs found. Note that there should be no '=' characters in plots input lines.")
             return {}
         plots = {}
         for key, val in plots_table[0]:
             plots[key] = val
         return plots
+
+    @staticmethod
+    def read_nbo(string: str) -> Dict:
+        """
+        Read nbo parameters from string.
+
+        Args:
+            string (str): String
+
+        Returns:
+            (dict) nbo parameters.
+        """
+        header = r"^\s*\$nbo"
+        row = r"\s*([a-zA-Z\_]+)\s*=?\s*(\S+)"
+        footer = r"^\s*\$end"
+        nbo_table = read_table_pattern(string, header_pattern=header, row_pattern=row, footer_pattern=footer)
+        if nbo_table == []:
+            print("No valid nbo inputs found.")
+            return {}
+        nbo = {}
+        for key, val in nbo_table[0]:
+            nbo[key] = val
+        return nbo
