@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -12,10 +11,11 @@ import math
 import os
 import re
 import warnings
-from typing import List, Dict, Any
+from typing import Any, Dict, List, Union
 
 import networkx as nx
 import numpy as np
+import pandas as pd
 from monty.io import zopen
 from monty.json import MSONable, jsanitize
 
@@ -36,6 +36,7 @@ from .utils import process_parsed_coords, read_pattern, read_table_pattern
 __author__ = "Samuel Blau, Brandon Wood, Shyam Dwaraknath, Evan Spotte-Smith"
 __copyright__ = "Copyright 2018, The Materials Project"
 __version__ = "0.1"
+__credits__ = "Gabe Gomes"
 
 logger = logging.getLogger(__name__)
 
@@ -51,11 +52,11 @@ class QCOutput(MSONable):
             filename (str): Filename to parse
         """
         self.filename = filename
-        self.data = dict()  # type: Dict[str, Any]
+        self.data = {}  # type: Dict[str, Any]
         self.data["errors"] = []
         self.data["warnings"] = {}
         self.text = ""
-        with zopen(filename, "rt") as f:
+        with zopen(filename, mode="rt", encoding="ISO-8859-1") as f:
             self.text = f.read()
 
         # Check if output file contains multiple output files. If so, print an error message and exit
@@ -218,6 +219,53 @@ class QCOutput(MSONable):
                         spin_contamination[ii] = abs(correct_s2 - entry)
                     self.data["warnings"]["spin_contamination"] = spin_contamination
 
+        # Parse additional data from coupled-cluster calculations
+        self.data["coupled_cluster"] = read_pattern(
+            self.text, {"key": r"CCMAN2: suite of methods based on coupled cluster"}
+        ).get("key")
+        if self.data.get("coupled_cluster", []):
+            temp_dict = read_pattern(
+                self.text,
+                {
+                    "SCF": r"\s+SCF energy\s+=\s+([\d\-\.]+)",
+                    "MP2": r"\s+MP2 energy\s+=\s+([\d\-\.]+)",
+                    "CCSD_correlation": r"\s+CCSD correlation energy\s+=\s+([\d\-\.]+)",
+                    "CCSD": r"\s+CCSD total energy\s+=\s+([\d\-\.]+)",
+                    "CCSD(T)_correlation": r"\s+CCSD\(T\) correlation energy\s+=\s+([\d\-\.]+)",
+                    "CCSD(T)": r"\s+CCSD\(T\) total energy\s+=\s+([\d\-\.]+)",
+                },
+            )
+
+            if temp_dict.get("SCF") is None:
+                self.data["hf_scf_energy"] = None
+            else:
+                self.data["hf_scf_energy"] = float(temp_dict["SCF"][0][0])
+
+            if temp_dict.get("MP2") is None:
+                self.data["mp2_energy"] = None
+            else:
+                self.data["mp2_energy"] = float(temp_dict["MP2"][0][0])
+
+            if temp_dict.get("CCSD_correlation") is None:
+                self.data["ccsd_correlation_energy"] = None
+            else:
+                self.data["ccsd_correlation_energy"] = float(temp_dict["CCSD_correlation"][0][0])
+
+            if temp_dict.get("CCSD") is None:
+                self.data["ccsd_total_energy"] = None
+            else:
+                self.data["ccsd_total_energy"] = float(temp_dict["CCSD"][0][0])
+
+            if temp_dict.get("CCSD(T)_correlation") is None:
+                self.data["ccsd(t)_correlation_energy"] = None
+            else:
+                self.data["ccsd(t)_correlation_energy"] = float(temp_dict["CCSD(T)_correlation"][0][0])
+
+            if temp_dict.get("CCSD(T)") is None:
+                self.data["ccsd(t)_total_energy"] = None
+            else:
+                self.data["ccsd(t)_total_energy"] = float(temp_dict["CCSD(T)"][0][0])
+
         # Check if the calculation is a geometry optimization. If so, parse the relevant output
         self.data["optimization"] = read_pattern(self.text, {"key": r"(?i)\s*job(?:_)*type\s*(?:=)*\s*opt"}).get("key")
         if self.data.get("optimization", []):
@@ -228,7 +276,7 @@ class QCOutput(MSONable):
         self.data["transition_state"] = read_pattern(self.text, {"key": r"(?i)\s*job(?:_)*type\s*(?:=)*\s*ts"}).get(
             "key"
         )
-        if self.data.get("transition_state", list()):
+        if self.data.get("transition_state", []):
             self._read_optimization_data()
 
         # Check if the calculation contains a constraint in an $opt section.
@@ -264,6 +312,7 @@ class QCOutput(MSONable):
         if self.data.get("frequency_job", []):
             self._read_frequency_data()
 
+        # Check if the calculation is a single point. If so, parse the relevant output
         self.data["single_point_job"] = read_pattern(
             self.text,
             {"key": r"(?i)\s*job(?:_)*type\s*(?:=)*\s*sp"},
@@ -272,20 +321,29 @@ class QCOutput(MSONable):
         if self.data.get("single_point_job", []):
             self._read_single_point_data()
 
+        # Check if the calculation is a force calculation. If so, parse the relevant output
         self.data["force_job"] = read_pattern(
             self.text,
             {"key": r"(?i)\s*job(?:_)*type\s*(?:=)*\s*force"},
             terminate_on_match=True,
         ).get("key")
-        if self.data.get("force", []):
+        if self.data.get("force_job", []):
             self._read_force_data()
 
+        # Check if the calculation is a PES scan. If so, parse the relevant output
         self.data["scan_job"] = read_pattern(
             self.text, {"key": r"(?i)\s*job(?:_)*type\s*(?:=)*\s*pes_scan"}, terminate_on_match=True
         ).get("key")
         if self.data.get("scan_job", []):
             self._read_scan_data()
 
+        # Check if an NBO calculation was performed. If so, parse the relevant output
+        self.data["nbo_data"] = read_pattern(
+            self.text, {"key": r"N A T U R A L   A T O M I C   O R B I T A L"}, terminate_on_match=True
+        ).get("key")
+        if self.data.get("nbo_data", []):
+            self._read_nbo_data()
+
         # If the calculation did not finish and no errors have been identified yet, check for other errors
         if not self.data.get("completion", []) and self.data.get("errors") == []:
             self._check_completion_errors()
@@ -294,11 +352,11 @@ class QCOutput(MSONable):
     def multiple_outputs_from_file(cls, filename, keep_sub_files=True):
         """
         Parses a QChem output file with multiple calculations
-        # 1.) Seperates the output into sub-files
+        # 1.) Separates the output into sub-files
             e.g. qcout -> qcout.0, qcout.1, qcout.2 ... qcout.N
-            a.) Find delimeter for multiple calcualtions
-            b.) Make seperate output sub-files
-        2.) Creates seperate QCCalcs for each one from the sub-files
+            a.) Find delimiter for multiple calculations
+            b.) Make separate output sub-files
+        2.) Creates separate QCCalcs for each one from the sub-files
         """
         to_return = []
         with zopen(filename, "rt") as f:
@@ -306,9 +364,8 @@ class QCOutput(MSONable):
         if text[0] == "":
             text = text[1:]
         for i, sub_text in enumerate(text):
-            temp = open(filename + "." + str(i), "w")
-            temp.write(sub_text)
-            temp.close()
+            with open(filename + "." + str(i), "w") as temp:
+                temp.write(sub_text)
             tempOutput = cls(filename + "." + str(i))
             to_return.append(tempOutput)
             if not keep_sub_files:
@@ -412,9 +469,6 @@ class QCOutput(MSONable):
                 r"Q-Chem Inc\. Pittsburgh\s+)*\-+)*\n"
             )
             table_pattern = (
-                r"(?:\n[a-zA-Z_\s/]+\.C::(?:WARNING energy changes are now smaller than effective "
-                r"accuracy\.)*(?:\s+calculation will continue, but THRESH should be increased)*"
-                r"(?:\s+or SCF_CONVERGENCE decreased\. )*(?:\s+effective_thresh = [\d\-\.]+e[\d\-]+)*)*"
                 r"(?:\s*Nonlocal correlation = [\d\-\.]+e[\d\-]+)*"
                 r"(?:\s*Inaccurate integrated density:\n\s+Number of electrons\s+=\s+[\d\-\.]+\n\s+"
                 r"Numerical integral\s+=\s+[\d\-\.]+\n\s+Relative error\s+=\s+[\d\-\.]+\s+\%\n)*\s*\d+\s+"
@@ -427,6 +481,11 @@ class QCOutput(MSONable):
                 r"(?:RMS Gradient)*\s+\-+(?:\s*\-+\s+OpenMP\s+Integral\s+computing\s+Module\s+"
                 r"(?:Release:\s+version\s+[\d\-\.]+\,\s+\w+\s+[\d\-\.]+\, "
                 r"Q-Chem Inc\. Pittsburgh\s+)*\-+)*\n)*"
+                r"(?:(\n\s*[a-z\dA-Z_\s/]+\.C|\n\s*GDM)::WARNING energy changes are now smaller than effective "
+                r"accuracy\.\s*(\n\s*[a-z\dA-Z_\s/]+\.C|\n\s*GDM)::\s+calculation will continue, but THRESH s"
+                r"hould be increased\s*"
+                r"(\n\s*[a-z\dA-Z_\s/]+\.C|\n\s*GDM)::\s+or SCF_CONVERGENCE decrea"
+                r"sed\.\s*(\n\s*[a-z\dA-Z_\s/]+\.C|\n\s*GDM)::\s+effective_thresh = [\d\-\.]+e[\d\-]+)*"
             )
         else:
             if "SCF_failed_to_converge" in self.data.get("errors"):
@@ -679,13 +738,6 @@ class QCOutput(MSONable):
         ):
             self.data["warnings"]["diagonalizing_BBt"] = True
 
-        # Check for bad Roothaan step
-        for scf in self.data["SCF"]:
-            if abs(scf[0][0] - scf[1][0]) > 10.0:
-                self.data["warnings"]["bad_roothaan"] = True
-                if abs(scf[0][0] - scf[1][0]) > 100.0:
-                    self.data["warnings"]["very_bad_roothaan"] = True
-
     def _read_geometries(self):
         """
         Parses all geometries from an optimization trajectory.
@@ -741,7 +793,7 @@ class QCOutput(MSONable):
                     charge=self.data.get("charge"),
                     spin_multiplicity=self.data.get("multiplicity"),
                 )
-                self.data["molecules_from_optimized_geometries"] = list()
+                self.data["molecules_from_optimized_geometries"] = []
                 for geom in self.data["optimized_geometries"]:
                     mol = Molecule(
                         species=self.data.get("species"),
@@ -875,8 +927,23 @@ class QCOutput(MSONable):
 
     def _read_frequency_data(self):
         """
-        Parses frequencies, enthalpy, entropy, and mode vectors.
+        Parses cpscf_nseg, frequencies, enthalpy, entropy, and mode vectors.
         """
+        if read_pattern(self.text, {"key": r"Calculating MO derivatives via CPSCF"}, terminate_on_match=True).get(
+            "key"
+        ) == [[]]:
+            temp_cpscf_nseg = read_pattern(
+                self.text,
+                {"key": r"CPSCF will be done in([\d\s]+)segments to save memory"},
+                terminate_on_match=True,
+            ).get("key")
+            if temp_cpscf_nseg is None:
+                self.data["cpscf_nseg"] = 1
+            else:
+                self.data["cpscf_nseg"] = int(temp_cpscf_nseg[0][0])
+        else:
+            self.data["cpscf_nseg"] = 0
+
         raman = False
         if read_pattern(self.text, {"key": r"doraman\s*(?:=)*\s*true"}, terminate_on_match=True).get("key") == [[]]:
             raman = True
@@ -1072,6 +1139,11 @@ class QCOutput(MSONable):
             self.data["energy_trajectory"] = real_energy_trajectory
 
         self._read_geometries()
+        if have_babel:
+            self.data["structure_change"] = check_for_structure_changes(
+                self.data["initial_molecule"],
+                self.data["molecule_from_last_geometry"],
+            )
         self._read_gradients()
 
         if len(self.data.get("errors")) == 0:
@@ -1096,7 +1168,7 @@ class QCOutput(MSONable):
             footer_pattern=footer_pattern,
         )
 
-        self.data["scan_energies"] = list()
+        self.data["scan_energies"] = []
         if len(single_data) == 0:
             double_data = read_table_pattern(
                 self.text,
@@ -1129,7 +1201,7 @@ class QCOutput(MSONable):
             footer_pattern=scan_inputs_foot,
         )
 
-        self.data["scan_variables"] = {"stre": list(), "bend": list(), "tors": list()}
+        self.data["scan_variables"] = {"stre": [], "bend": [], "tors": []}
         for row in constraints_meta[0]:
             var_type = row[0].lower()
             self.data["scan_variables"][var_type].append(
@@ -1145,7 +1217,7 @@ class QCOutput(MSONable):
             self.text,
             {"key": r"\s*(Distance\(Angs\)|Angle|Dihedral)\:\s*((?:[0-9]+\s+)+)+([\.0-9]+)\s+([\.0-9]+)"},
         ).get("key")
-        self.data["scan_constraint_sets"] = {"stre": list(), "bend": list(), "tors": list()}
+        self.data["scan_constraint_sets"] = {"stre": [], "bend": [], "tors": []}
         if temp_constraint is not None:
             for entry in temp_constraint:
                 atoms = [int(i) for i in entry[1].split()]
@@ -1237,6 +1309,16 @@ class QCOutput(MSONable):
         for key in pcm_keys:
             self.data["solvent_data"][key] = None
 
+    def _read_nbo_data(self):
+        """
+        Parses NBO output
+        """
+        dfs = nbo_parser(self.filename)
+        nbo_data = {}
+        for key, value in dfs.items():
+            nbo_data[key] = [df.to_dict() for df in value]
+        self.data["nbo_data"] = nbo_data
+
     def _check_completion_errors(self):
         """
         Parses potential errors that can cause jobs to crash
@@ -1272,6 +1354,15 @@ class QCOutput(MSONable):
             == [[]]
         ):
             self.data["errors"] += ["premature_end_FileMan_error"]
+        elif (
+            read_pattern(
+                self.text,
+                {"key": r"need to increase the array of NLebdevPts"},
+                terminate_on_match=True,
+            ).get("key")
+            == [[]]
+        ):
+            self.data["errors"] += ["NLebdevPts"]
         elif read_pattern(self.text, {"key": r"method not available"}, terminate_on_match=True).get("key") == [[]]:
             self.data["errors"] += ["method_not_available"]
         elif (
@@ -1309,6 +1400,33 @@ class QCOutput(MSONable):
             == [[]]
         ):
             self.data["errors"] += ["driver_error"]
+        elif (
+            read_pattern(
+                self.text,
+                {"key": r"Basis not supported for the above atom"},
+                terminate_on_match=True,
+            ).get("key")
+            == [[]]
+        ):
+            self.data["errors"] += ["basis_not_supported"]
+        elif (
+            read_pattern(
+                self.text,
+                {"key": r"Unable to find relaxed density"},
+                terminate_on_match=True,
+            ).get("key")
+            == [[]]
+        ):
+            self.data["errors"] += ["failed_cpscf"]
+        elif (
+            read_pattern(
+                self.text,
+                {"key": r"Out of Iterations- IterZ"},
+                terminate_on_match=True,
+            ).get("key")
+            == [[]]
+        ):
+            self.data["errors"] += ["failed_cpscf"]
         elif (
             read_pattern(
                 self.text,
@@ -1403,3 +1521,411 @@ def check_for_structure_changes(mol1: Molecule, mol2: Molecule) -> str:
     if last_graph.number_of_edges() > initial_graph.number_of_edges():
         return "more_bonds"
     return "bond_change"
+
+
+def jump_to_header(lines: List[str], header: str) -> List[str]:
+    """
+    Given a list of lines, truncate the start of the list so that the first line
+    of the new list contains the header.
+
+    Args:
+            lines: List of lines.
+            header: Substring to match.
+
+    Returns:
+            Truncated lines.
+
+    Raises:
+            RuntimeError
+    """
+
+    # Search for the header
+    for i, line in enumerate(lines):
+        if header in line.strip():
+            return lines[i:]
+
+    # Search failed
+    raise RuntimeError(f"Header {header} could not be found in the lines.")
+
+
+def get_percentage(line: str, orbital: str) -> str:
+    """
+    Retrieve the percent character of an orbital.
+
+    Args:
+            line: Line containing orbital and percentage.
+            orbital: Type of orbital (s, p, d, f).
+
+    Returns:
+            Percentage of character.
+
+    Raises:
+            n/a
+    """
+
+    # Locate orbital in line
+    index = line.find(orbital)
+    line = line[index:]
+
+    # Locate the first open bracket
+    index = line.find("(")
+    line = line[index:]
+
+    # Isolate the percentage
+    return line[1:7].strip()
+
+
+def z_int(string: str) -> int:
+    """
+    Convert string to integer.
+    If string empty, return -1.
+
+    Args:
+            string: Input to be cast to int.
+
+    Returns:
+            Int representation.
+
+    Raises:
+            n/a
+    """
+    try:
+        return int(string)
+    except ValueError:
+        return -1
+
+
+def parse_natural_populations(lines: List[str]) -> List[pd.DataFrame]:
+    """
+    Parse the natural populations section of NBO output.
+
+    Args:
+            lines: QChem output lines.
+
+    Returns:
+            Data frame of formatted output.
+
+    Raises:
+            RuntimeError
+    """
+
+    no_failures = True
+    pop_dfs = []
+
+    while no_failures:
+
+        # Natural populations
+        try:
+            lines = jump_to_header(lines, "Summary of Natural Population Analysis:")
+        except RuntimeError:
+            no_failures = False
+
+        if no_failures:
+            # Jump to column names
+            lines = lines[4:]
+            columns = lines[0].split()
+
+            # Jump to values
+            lines = lines[2:]
+            data = []
+            for line in lines:
+
+                # Termination condition
+                if "=" in line:
+                    break
+
+                # Extract the values
+                values = line.split()
+                if len(values[0]) > 2:
+                    values.insert(0, values[0][0:-3])
+                    values[1] = values[1][-3:]
+                data.append(
+                    [
+                        str(values[0]),
+                        int(values[1]),
+                        float(values[2]),
+                        float(values[3]),
+                        float(values[4]),
+                        float(values[5]),
+                        float(values[6]),
+                    ]
+                )
+                if len(columns) == 8:
+                    data[-1].append(float(values[7]))
+
+            # Store values in a dataframe
+            pop_dfs.append(pd.DataFrame(data=data, columns=columns))
+    return pop_dfs
+
+
+def parse_hybridization_character(lines: List[str]) -> List[pd.DataFrame]:
+    """
+    Parse the hybridization character section of NBO output.
+
+    Args:
+            lines: QChem output lines.
+
+    Returns:
+            Data frames of formatted output.
+
+    Raises:
+            RuntimeError
+    """
+
+    # Orbitals
+    orbitals = ["s", "p", "d", "f"]
+
+    no_failures = True
+    lp_and_bd_dfs = []
+
+    while no_failures:
+
+        # NBO Analysis
+        try:
+            lines = jump_to_header(lines, "(Occupancy)   Bond orbital/ Coefficients/ Hybrids")
+        except RuntimeError:
+            try:
+                lines = jump_to_header(lines, "(Occupancy)   Bond orbital / Coefficients / Hybrids")
+            except RuntimeError:
+                no_failures = False
+
+        if no_failures:
+
+            # Jump to values
+            lines = lines[2:]
+
+            # Save the data for different types of orbitals
+            lp_data = []
+            bd_data = []
+
+            # Iterate over the lines
+            i = -1
+            while True:
+                i += 1
+                line = lines[i]
+
+                # Termination conditions
+                if "NHO DIRECTIONALITY AND BOND BENDING" in line:
+                    break
+                if "Archival summary:" in line:
+                    break
+
+                # Lone pair
+                if "LP" in line or "LV" in line:
+                    LPentry = {orbital: 0.0 for orbital in orbitals}  # type: Dict[str, Union[str, int, float]]
+                    LPentry["bond index"] = line[0:4].strip()
+                    LPentry["occupancy"] = line[7:14].strip()
+                    LPentry["type"] = line[16:19].strip()
+                    LPentry["orbital index"] = line[20:22].strip()
+                    LPentry["atom symbol"] = line[23:25].strip()
+                    LPentry["atom number"] = line[25:28].strip()
+
+                    # Populate the orbital percentages
+                    for orbital in orbitals:
+                        if orbital in line:
+                            LPentry[orbital] = get_percentage(line, orbital)
+
+                    # Move one line down
+                    i += 1
+                    line = lines[i]
+
+                    # Populate the orbital percentages
+                    for orbital in orbitals:
+                        if orbital in line:
+                            LPentry[orbital] = get_percentage(line, orbital)
+
+                    # Save the entry
+                    lp_data.append(LPentry)
+
+                # Bonding
+                if "BD" in line:
+                    BDentry = {
+                        f"atom {i} {orbital}": 0.0 for orbital in orbitals for i in range(1, 3)
+                    }  # type: Dict[str, Union[str, int, float]]
+                    BDentry["bond index"] = line[0:4].strip()
+                    BDentry["occupancy"] = line[7:14].strip()
+                    BDentry["type"] = line[16:19].strip()
+                    BDentry["orbital index"] = line[20:22].strip()
+                    BDentry["atom 1 symbol"] = line[23:25].strip()
+                    BDentry["atom 1 number"] = line[25:28].strip()
+                    BDentry["atom 2 symbol"] = line[29:31].strip()
+                    BDentry["atom 2 number"] = line[31:34].strip()
+
+                    # Move one line down
+                    i += 1
+                    line = lines[i]
+
+                    BDentry["atom 1 polarization"] = line[16:22].strip()
+                    BDentry["atom 1 pol coeff"] = line[24:33].strip()
+
+                    # Populate the orbital percentages
+                    for orbital in orbitals:
+                        if orbital in line:
+                            BDentry[f"atom 1 {orbital}"] = get_percentage(line, orbital)
+
+                    # Move one line down
+                    i += 1
+                    line = lines[i]
+
+                    # Populate the orbital percentages
+                    for orbital in orbitals:
+                        if orbital in line:
+                            BDentry[f"atom 1 {orbital}"] = get_percentage(line, orbital)
+
+                    # Move down until you see an orbital
+                    while "s" not in line:
+                        i += 1
+                        line = lines[i]
+
+                    BDentry["atom 2 polarization"] = line[16:22].strip()
+                    BDentry["atom 2 pol coeff"] = line[24:33].strip()
+
+                    # Populate the orbital percentages
+                    for orbital in orbitals:
+                        if orbital in line:
+                            BDentry[f"atom 2 {orbital}"] = get_percentage(line, orbital)
+
+                    # Move one line down
+                    i += 1
+                    line = lines[i]
+
+                    # Populate the orbital percentages
+                    for orbital in orbitals:
+                        if orbital in line:
+                            BDentry[f"atom 2 {orbital}"] = get_percentage(line, orbital)
+
+                    # Save the entry
+                    bd_data.append(BDentry)
+
+            # Store values in a dataframe
+            lp_and_bd_dfs.append(pd.DataFrame(data=lp_data))
+            lp_and_bd_dfs.append(pd.DataFrame(data=bd_data))
+
+    return lp_and_bd_dfs
+
+
+def parse_perturbation_energy(lines: List[str]) -> List[pd.DataFrame]:
+    """
+    Parse the perturbation energy section of NBO output.
+
+    Args:
+            lines: QChem output lines.
+
+    Returns:
+            Data frame of formatted output.
+
+    Raises:
+            RuntimeError
+    """
+
+    no_failures = True
+    e2_dfs = []
+
+    while no_failures:
+
+        # 2nd order perturbation theory analysis
+        try:
+            lines = jump_to_header(
+                lines,
+                "SECOND ORDER PERTURBATION THEORY ANALYSIS OF FOCK MATRIX IN NBO BASIS",
+            )
+        except RuntimeError:
+            no_failures = False
+
+        if no_failures:
+
+            # Jump to values
+            i = -1
+            while True:
+                i += 1
+                line = lines[i]
+                if "within" in line:
+                    lines = lines[i:]
+                    break
+
+            # Extract 2nd order data
+            e2_data = []
+            for line in lines:
+
+                # Termination condition
+                if "NATURAL BOND ORBITALS" in line:
+                    break
+
+                # Skip conditions
+                if line.strip() == "":
+                    continue
+                if "unit" in line:
+                    continue
+                if "None" in line:
+                    continue
+
+                # Extract the values
+                entry = {}  # type: Dict[str, Union[str, int, float]]
+                if line[4] == ".":
+                    entry["donor bond index"] = int(line[0:4].strip())
+                    entry["donor type"] = str(line[5:9].strip())
+                    entry["donor orbital index"] = int(line[10:12].strip())
+                    entry["donor atom 1 symbol"] = str(line[13:15].strip())
+                    entry["donor atom 1 number"] = int(line[15:17].strip())
+                    entry["donor atom 2 symbol"] = str(line[18:20].strip())
+                    entry["donor atom 2 number"] = z_int(line[20:22].strip())
+                    entry["acceptor bond index"] = int(line[25:31].strip())
+                    entry["acceptor type"] = str(line[32:36].strip())
+                    entry["acceptor orbital index"] = int(line[37:39].strip())
+                    entry["acceptor atom 1 symbol"] = str(line[40:42].strip())
+                    entry["acceptor atom 1 number"] = int(line[42:44].strip())
+                    entry["acceptor atom 2 symbol"] = str(line[45:47].strip())
+                    entry["acceptor atom 2 number"] = z_int(line[47:49].strip())
+                    entry["perturbation energy"] = float(line[50:62].strip())
+                    entry["energy difference"] = float(line[62:70].strip())
+                    entry["fock matrix element"] = float(line[70:79].strip())
+                elif line[5] == ".":
+                    entry["donor bond index"] = int(line[0:5].strip())
+                    entry["donor type"] = str(line[6:10].strip())
+                    entry["donor orbital index"] = int(line[11:13].strip())
+                    entry["donor atom 1 symbol"] = str(line[14:16].strip())
+                    entry["donor atom 1 number"] = int(line[16:19].strip())
+                    entry["donor atom 2 symbol"] = str(line[20:22].strip())
+                    entry["donor atom 2 number"] = z_int(line[22:25].strip())
+                    entry["acceptor bond index"] = int(line[25:33].strip())
+                    entry["acceptor type"] = str(line[34:38].strip())
+                    entry["acceptor orbital index"] = int(line[39:41].strip())
+                    entry["acceptor atom 1 symbol"] = str(line[42:44].strip())
+                    entry["acceptor atom 1 number"] = int(line[44:47].strip())
+                    entry["acceptor atom 2 symbol"] = str(line[48:50].strip())
+                    entry["acceptor atom 2 number"] = z_int(line[50:53].strip())
+                    entry["perturbation energy"] = float(line[53:63].strip())
+                    entry["energy difference"] = float(line[63:71].strip())
+                    entry["fock matrix element"] = float(line[71:79].strip())
+                e2_data.append(entry)
+
+            # Store values in a dataframe
+            e2_dfs.append(pd.DataFrame(data=e2_data))
+
+    return e2_dfs
+
+
+def nbo_parser(filename: str) -> Dict[str, List[pd.DataFrame]]:
+    """
+    Parse all the important sections of NBO output.
+
+    Args:
+            filename: Path to QChem NBO output.
+
+    Returns:
+            Data frames of formatted output.
+
+    Raises:
+            RuntimeError
+    """
+
+    # Open the lines
+    with zopen(filename, mode="rt", encoding="ISO-8859-1") as f:
+        lines = f.readlines()
+
+    # Compile the dataframes
+    dfs = {}
+    dfs["natural_populations"] = parse_natural_populations(lines)
+    dfs["hybridization_character"] = parse_hybridization_character(lines)
+    dfs["perturbation_energy"] = parse_perturbation_energy(lines)
+    return dfs
