@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -17,6 +16,7 @@ import copy
 import itertools
 import logging
 import math
+import warnings
 from collections import defaultdict
 from fractions import Fraction
 from math import cos, sin
@@ -55,6 +55,7 @@ class SpacegroupAnalyzer:
         self._symprec = symprec
         self._angle_tol = angle_tolerance
         self._structure = structure
+        self._siteprops = structure.site_properties
         latt = structure.lattice.matrix
         positions = structure.frac_coords
         unique_species = []
@@ -75,11 +76,10 @@ class SpacegroupAnalyzer:
             elif site.is_ordered and hasattr(site.specie, "spin"):
                 magmoms.append(site.specie.spin)
             else:
-                magmoms.append(0)
+                magmoms.append(0)  # needed for spglib
 
         self._unique_species = unique_species
         self._numbers = zs
-        # For now, we are setting magmom to zero.
         self._cell = latt, positions, zs, magmoms
 
         self._space_group_data = spglib.get_symmetry_dataset(
@@ -149,26 +149,22 @@ class SpacegroupAnalyzer:
         """
         n = self._space_group_data["number"]
 
-        def f(i, j):
-            return i <= n <= j
+        if 0 < n < 3:
+            return "triclinic"
+        if n < 16:
+            return "monoclinic"
+        if n < 75:
+            return "orthorhombic"
+        if n < 143:
+            return "tetragonal"
+        if n < 168:
+            return "trigonal"
+        if n < 195:
+            return "hexagonal"
+        if n < 231:
+            return "cubic"
 
-        cs = {
-            "triclinic": (1, 2),
-            "monoclinic": (3, 15),
-            "orthorhombic": (16, 74),
-            "tetragonal": (75, 142),
-            "trigonal": (143, 167),
-            "hexagonal": (168, 194),
-            "cubic": (195, 230),
-        }
-
-        crystal_sytem = None
-
-        for k, v in cs.items():
-            if f(*v):
-                crystal_sytem = k
-                break
-        return crystal_sytem
+        raise ValueError("Invalid space group")
 
     def get_lattice_type(self):
         """
@@ -293,36 +289,68 @@ class SpacegroupAnalyzer:
         )
         return SymmetrizedStructure(self._structure, sg, ds["equivalent_atoms"], ds["wyckoffs"])
 
-    def get_refined_structure(self):
+    def get_refined_structure(self, keep_site_properties=False):
         """
         Get the refined structure based on detected symmetry. The refined
         structure is a *conventional* cell setting with atoms moved to the
         expected symmetry positions.
 
+        Args:
+            keep_site_properties (bool): Whether to keep the input site properties (including
+                magnetic moments) on the sitesthat are still present after the refinement. Note:
+                This is disabled by default because the magnetic moments are not always directly
+                transferable between unit cell definitions. For instance, long-range magnetic
+                ordering or antiferromagnetic character may no longer be present (or exist in
+                the same way) in the returned structure. If keep_site_properties is True,
+                each site retains the same site property as in the original structure without
+                further adjustment.
+
         Returns:
             Refined structure.
         """
         # Atomic positions have to be specified by scaled positions for spglib.
         lattice, scaled_positions, numbers = spglib.refine_cell(self._cell, self._symprec, self._angle_tol)
-
         species = [self._unique_species[i - 1] for i in numbers]
-        s = Structure(lattice, species, scaled_positions)
+        if keep_site_properties:
+            site_properties = {}
+            for k, v in self._siteprops.items():
+                site_properties[k] = [v[i - 1] for i in numbers]
+        else:
+            site_properties = None
+        s = Structure(lattice, species, scaled_positions, site_properties=site_properties)
         return s.get_sorted_structure()
 
-    def find_primitive(self):
+    def find_primitive(self, keep_site_properties=False):
         """
         Find a primitive version of the unit cell.
 
+        Args:
+            keep_site_properties (bool): Whether to keep the input site properties (including
+                magnetic moments) on the sitesthat are still present after the refinement. Note:
+                This is disabled by default because the magnetic moments are not always directly
+                transferable between unit cell definitions. For instance, long-range magnetic
+                ordering or antiferromagnetic character may no longer be present (or exist in
+                the same way) in the returned structure. If keep_site_properties is True,
+                each site retains the same site property as in the original structure without
+                further adjustment.
+
         Returns:
             A primitive cell in the input cell is searched and returned
             as an Structure object. If no primitive cell is found, None is
             returned.
         """
         lattice, scaled_positions, numbers = spglib.find_primitive(self._cell, symprec=self._symprec)
-
         species = [self._unique_species[i - 1] for i in numbers]
+        if keep_site_properties:
+            site_properties = {}
+            for k, v in self._siteprops.items():
+                site_properties[k] = [v[i - 1] for i in numbers]
+        else:
+            site_properties = None
 
-        return Structure(lattice, species, scaled_positions, to_unit_cell=True).get_reduced_structure()
+        return Structure(
+            lattice, species, scaled_positions, to_unit_cell=True, site_properties=site_properties
+        ).get_reduced_structure()
 
     def get_ir_reciprocal_mesh(self, mesh=(10, 10, 10), is_shift=(0, 0, 0)):
         """
@@ -358,6 +386,10 @@ class SpacegroupAnalyzer:
         Challenges and tools. Computational Materials Science,
         49(2), 299-312. doi:10.1016/j.commatsci.2010.05.010
 
+        Args:
+            international_monoclinic (bool): Whether to convert to proper international convention
+                such that beta is the non-right angle.
+
         Returns:
             Transformation matrix to go from conventional to primitive cell
         """
@@ -390,7 +422,7 @@ class SpacegroupAnalyzer:
 
         return transf
 
-    def get_primitive_standard_structure(self, international_monoclinic=True):
+    def get_primitive_standard_structure(self, international_monoclinic=True, keep_site_properties=False):
         """
         Gives a structure with a primitive cell according to certain standards
         the standards are defined in Setyawan, W., & Curtarolo, S. (2010).
@@ -398,10 +430,24 @@ class SpacegroupAnalyzer:
         Challenges and tools. Computational Materials Science,
         49(2), 299-312. doi:10.1016/j.commatsci.2010.05.010
 
+        Args:
+            international_monoclinic (bool): Whether to convert to proper international convention
+                such that beta is the non-right angle.
+            keep_site_properties (bool): Whether to keep the input site properties (including
+                magnetic moments) on the sitesthat are still present after the refinement. Note:
+                This is disabled by default because the magnetic moments are not always directly
+                transferable between unit cell definitions. For instance, long-range magnetic
+                ordering or antiferromagnetic character may no longer be present (or exist in
+                the same way) in the returned structure. If keep_site_properties is True,
+                each site retains the same site property as in the original structure without
+                further adjustment.
+
         Returns:
             The structure in a primitive standardized cell
         """
-        conv = self.get_conventional_standard_structure(international_monoclinic=international_monoclinic)
+        conv = self.get_conventional_standard_structure(
+            international_monoclinic=international_monoclinic, keep_site_properties=keep_site_properties
+        )
         lattice = self.get_lattice_type()
 
         if "P" in self.get_space_group_symbol() or lattice == "hexagonal":
@@ -456,7 +502,7 @@ class SpacegroupAnalyzer:
 
         return Structure.from_sites(new_sites)
 
-    def get_conventional_standard_structure(self, international_monoclinic=True):
+    def get_conventional_standard_structure(self, international_monoclinic=True, keep_site_properties=False):
         """
         Gives a structure with a conventional cell according to certain
         standards. The standards are defined in Setyawan, W., & Curtarolo,
@@ -464,18 +510,32 @@ class SpacegroupAnalyzer:
         Challenges and tools. Computational Materials Science,
         49(2), 299-312. doi:10.1016/j.commatsci.2010.05.010
         They basically enforce as much as possible
-        norm(a1)<norm(a2)<norm(a3)
+        norm(a1)<norm(a2)<norm(a3). NB This is not necessarily the same as the
+        standard settings within the International Tables of Crystallography,
+        for which get_refined_structure should be used instead.
+
+        Args:
+            international_monoclinic (bool): Whether to convert to proper international convention
+                such that beta is the non-right angle.
+            keep_site_properties (bool): Whether to keep the input site properties (including
+                magnetic moments) on the sitesthat are still present after the refinement. Note:
+                This is disabled by default because the magnetic moments are not always directly
+                transferable between unit cell definitions. For instance, long-range magnetic
+                ordering or antiferromagnetic character may no longer be present (or exist in
+                the same way) in the returned structure. If keep_site_properties is True,
+                each site retains the same site property as in the original structure without
+                further adjustment.
 
         Returns:
             The structure in a conventional standardized cell
         """
         tol = 1e-5
-        struct = self.get_refined_structure()
+        struct = self.get_refined_structure(keep_site_properties=keep_site_properties)
         latt = struct.lattice
         latt_type = self.get_lattice_type()
         sorted_lengths = sorted(latt.abc)
         sorted_dic = sorted(
-            [{"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [0, 1, 2]],
+            ({"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [0, 1, 2]),
             key=lambda k: k["length"],
         )
 
@@ -487,7 +547,7 @@ class SpacegroupAnalyzer:
                 transf[2] = [0, 0, 1]
                 a, b = sorted(latt.abc[:2])
                 sorted_dic = sorted(
-                    [{"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [0, 1]],
+                    ({"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [0, 1]),
                     key=lambda k: k["length"],
                 )
                 for i in range(2):
@@ -499,7 +559,7 @@ class SpacegroupAnalyzer:
                 transf[2] = [1, 0, 0]
                 a, b = sorted(latt.abc[1:])
                 sorted_dic = sorted(
-                    [{"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [1, 2]],
+                    ({"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [1, 2]),
                     key=lambda k: k["length"],
                 )
                 for i in range(2):
@@ -525,7 +585,7 @@ class SpacegroupAnalyzer:
             latt = Lattice.tetragonal(a, c)
         elif latt_type in ("hexagonal", "rhombohedral"):
             # for the conventional cell representation,
-            # we allways show the rhombohedral lattices as hexagonal
+            # we always show the rhombohedral lattices as hexagonal
 
             # check first if we have the refined structure shows a rhombohedral
             # cell
@@ -552,7 +612,7 @@ class SpacegroupAnalyzer:
                 transf = np.zeros(shape=(3, 3))
                 transf[2] = [0, 0, 1]
                 sorted_dic = sorted(
-                    [{"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [0, 1]],
+                    ({"vec": latt.matrix[i], "length": latt.abc[i], "orig_index": i} for i in [0, 1]),
                     key=lambda k: k["length"],
                 )
                 a = sorted_dic[0]["length"]
@@ -664,9 +724,10 @@ class SpacegroupAnalyzer:
         elif latt_type == "triclinic":
             # we use a LLL Minkowski-like reduction for the triclinic cells
             struct = struct.get_reduced_structure("LLL")
+            latt = struct.lattice
 
             a, b, c = latt.lengths
-            alpha, beta, gamma = [math.pi * i / 180 for i in latt.angles]
+            alpha, beta, gamma = (math.pi * i / 180 for i in latt.angles)
             new_matrix = None
             test_matrix = [
                 [a, 0, 0],
@@ -799,7 +860,7 @@ class SpacegroupAnalyzer:
                     weights.append(mapping.count(mapping[i]))
                     break
         if (len(mapped) != len(set(mapping))) or (not all(v == 1 for v in mapped.values())):
-            raise ValueError("Unable to find 1:1 corresponding between input " "kpoints and irreducible grid!")
+            raise ValueError("Unable to find 1:1 corresponding between input kpoints and irreducible grid!")
         return [w / sum(weights) for w in weights]
 
     def is_laue(self):
@@ -836,7 +897,7 @@ class PointGroupAnalyzer:
 
         a. Linear molecules have one zero eigenvalue. Possible symmetry
            operations are C*v or D*v
-        b. Asymetric top molecules have all different eigenvalues. The
+        b. Asymmetric top molecules have all different eigenvalues. The
            maximum rotational symmetry in such molecules is 2
         c. Symmetric top molecules have 1 unique eigenvalue, which gives a
            unique rotation axis.  All axial point groups are possible
@@ -851,7 +912,7 @@ class PointGroupAnalyzer:
 
     inversion_op = SymmOp.inversion()
 
-    def __init__(self, mol, tolerance=0.3, eigen_tolerance=0.01, matrix_tol=0.1):
+    def __init__(self, mol, tolerance=0.3, eigen_tolerance=0.01, matrix_tolerance=0.1):
         """
         The default settings are usually sufficient.
 
@@ -861,14 +922,14 @@ class PointGroupAnalyzer:
                 symmetrically equivalent. Defaults to 0.3 Angstrom.
             eigen_tolerance (float): Tolerance to compare eigen values of
                 the inertia tensor. Defaults to 0.01.
-            matrix_tol (float): Tolerance used to generate the full set of
+            matrix_tolerance (float): Tolerance used to generate the full set of
                 symmetry operations of the point group.
         """
         self.mol = mol
         self.centered_mol = mol.get_centered_molecule()
         self.tol = tolerance
         self.eig_tol = eigen_tolerance
-        self.mat_tol = matrix_tol
+        self.mat_tol = matrix_tolerance
         self._analyze()
         if self.sch_symbol in ["C1v", "C1h"]:
             self.sch_symbol = "Cs"
@@ -925,7 +986,7 @@ class PointGroupAnalyzer:
 
     def _proc_asym_top(self):
         """
-        Handles assymetric top molecules, which cannot contain rotational
+        Handles asymmetric top molecules, which cannot contain rotational
         symmetry larger than 2.
         """
         self._check_R2_axes_asym()
@@ -941,7 +1002,7 @@ class PointGroupAnalyzer:
 
     def _proc_sym_top(self):
         """
-        Handles symetric top molecules which has one unique eigenvalue whose
+        Handles symmetric top molecules which has one unique eigenvalue whose
         corresponding principal axis is a unique rotational axis.  More complex
         handling required to look for R2 axes perpendicular to this unique
         axis.
@@ -952,10 +1013,10 @@ class PointGroupAnalyzer:
             ind = 0
         else:
             ind = 1
-        logger.debug("Eigenvalues = %s." % self.eigvals)
+        logger.debug(f"Eigenvalues = {self.eigvals}.")
         unique_axis = self.principal_axes[ind]
         self._check_rot_sym(unique_axis)
-        logger.debug("Rotation symmetries = %s" % self.rot_sym)
+        logger.debug(f"Rotation symmetries = {self.rot_sym}")
         if len(self.rot_sym) > 0:
             self._check_perpendicular_r2_axis(unique_axis)
 
@@ -987,7 +1048,7 @@ class PointGroupAnalyzer:
         Handles cyclic group molecules.
         """
         main_axis, rot = max(self.rot_sym, key=lambda v: v[1])
-        self.sch_symbol = "C{}".format(rot)
+        self.sch_symbol = f"C{rot}"
         mirror_type = self._find_mirror(main_axis)
         if mirror_type == "h":
             self.sch_symbol += "h"
@@ -995,7 +1056,7 @@ class PointGroupAnalyzer:
             self.sch_symbol += "v"
         elif mirror_type == "":
             if self.is_valid_op(SymmOp.rotoreflection(main_axis, angle=180 / rot)):
-                self.sch_symbol = "S{}".format(2 * rot)
+                self.sch_symbol = f"S{2 * rot}"
 
     def _proc_dihedral(self):
         """
@@ -1003,7 +1064,7 @@ class PointGroupAnalyzer:
         and a main axis.
         """
         main_axis, rot = max(self.rot_sym, key=lambda v: v[1])
-        self.sch_symbol = "D{}".format(rot)
+        self.sch_symbol = f"D{rot}"
         mirror_type = self._find_mirror(main_axis)
         if mirror_type == "h":
             self.sch_symbol += "h"
@@ -1013,7 +1074,7 @@ class PointGroupAnalyzer:
     def _check_R2_axes_asym(self):
         """
         Test for 2-fold rotation along the principal axes. Used to handle
-        asymetric top molecules.
+        asymmetric top molecules.
         """
         for v in self.principal_axes:
             op = SymmOp.from_axis_angle_and_translation(v, 180)
@@ -1301,7 +1362,7 @@ class PointGroupAnalyzer:
 
         def all_equivalent_atoms_of_i(i, eq_sets, ops):
             """WORKS INPLACE on operations"""
-            visited = set([i])
+            visited = {i}
             tmp_eq_sets = {j: (eq_sets[j] - visited) for j in eq_sets[i]}
 
             while tmp_eq_sets:
@@ -1520,6 +1581,11 @@ def generate_full_symmops(symmops, tol):
             d = np.abs(full - op) < tol
             if not np.any(np.all(np.all(d, axis=2), axis=1)):
                 full.append(op)
+            if len(full) > 1000:
+                warnings.warn(
+                    f"{len(full)} matrices have been generated. The tol may be too small. Please terminate"
+                    f" and rerun with a different tolerance."
+                )
 
     d = np.abs(full - UNIT) < tol
     if not np.any(np.all(np.all(d, axis=2), axis=1)):
@@ -1581,7 +1647,7 @@ class SpacegroupOperations(list):
         return False
 
     def __str__(self):
-        return "{} ({}) spacegroup".format(self.int_symbol, self.int_number)
+        return f"{self.int_symbol} ({self.int_number}) spacegroup"
 
 
 class PointGroupOperations(list):
