@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -49,7 +48,7 @@ try:
 except ImportError:
     hiphive = None
 
-__author__ = "Shyue Ping Ong, Stephen Dacek, Anubhav Jain, Matthew Horton, " "Alex Ganose"
+__author__ = "Shyue Ping Ong, Stephen Dacek, Anubhav Jain, Matthew Horton, Alex Ganose"
 
 logger = logging.getLogger(__name__)
 
@@ -84,12 +83,12 @@ class ChargeBalanceTransformation(AbstractTransformation):
         num_in_structure = structure.composition[specie]
         removal_fraction = num_to_remove / num_in_structure
         if removal_fraction < 0:
-            raise ValueError("addition of specie not yet supported by " "ChargeBalanceTransformation")
+            raise ValueError("addition of specie not yet supported by ChargeBalanceTransformation")
         trans = SubstitutionTransformation({self.charge_balance_sp: {self.charge_balance_sp: 1 - removal_fraction}})
         return trans.apply_transformation(structure)
 
     def __str__(self):
-        return "Charge Balance Transformation : " + "Species to remove = {}".format(str(self.charge_balance_sp))
+        return f"Charge Balance Transformation : Species to remove = {self.charge_balance_sp}"
 
     def __repr__(self):
         return self.__str__()
@@ -139,7 +138,7 @@ class SuperTransformation(AbstractTransformation):
             Structures with all transformations applied.
         """
         if not return_ranked_list:
-            raise ValueError("SuperTransformation has no single best structure" " output. Must use return_ranked_list")
+            raise ValueError("SuperTransformation has no single best structure output. Must use return_ranked_list")
         structures = []
         for t in self._transformations:
             if t.is_one_to_many:
@@ -243,7 +242,7 @@ class MultipleSubstitutionTransformation:
                 sign = "+"
             else:
                 sign = "-"
-            dummy_sp = "X{}{}".format(str(charge), sign)
+            dummy_sp = f"X{charge}{sign}"
             mapping[self.sp_to_replace] = {
                 self.sp_to_replace: 1 - self.r_fraction,
                 dummy_sp: self.r_fraction,
@@ -262,13 +261,13 @@ class MultipleSubstitutionTransformation:
                     sign = "+"
                 else:
                     sign = "-"
-                st = SubstitutionTransformation({"X{}+".format(str(charge)): "{}{}{}".format(el, charge, sign)})
+                st = SubstitutionTransformation({f"X{charge}+": f"{el}{charge}{sign}"})
                 new_structure = st.apply_transformation(dummy_structure)
                 outputs.append({"structure": new_structure})
         return outputs
 
     def __str__(self):
-        return "Multiple Substitution Transformation : Substitution on " + "{}".format(self.sp_to_replace)
+        return "Multiple Substitution Transformation : Substitution on " + f"{self.sp_to_replace}"
 
     def __repr__(self):
         return self.__str__()
@@ -319,7 +318,7 @@ class EnumerateStructureTransformation(AbstractTransformation):
                 structure. The refined structure have cell parameters and
                 atomic positions shifted to the expected symmetry positions,
                 which makes it much less sensitive precision issues in enumlib.
-                If you are already starting from an experimental cif, refinment
+                If you are already starting from an experimental cif, refinement
                 should have already been done and it is not necessary. Defaults
                 to False.
             enum_precision_parameter (float): Finite precision parameter for
@@ -354,7 +353,7 @@ class EnumerateStructureTransformation(AbstractTransformation):
         self.timeout = timeout
 
         if max_cell_size and max_disordered_sites:
-            raise ValueError("Cannot set both max_cell_size and " "max_disordered_sites!")
+            raise ValueError("Cannot set both max_cell_size and max_disordered_sites!")
 
     def apply_transformation(self, structure, return_ranked_list=False):
         """
@@ -400,9 +399,9 @@ class EnumerateStructureTransformation(AbstractTransformation):
             structures = [structure.copy()]
 
         if self.max_disordered_sites:
-            ndisordered = sum([1 for site in structure if not site.is_ordered])
+            ndisordered = sum(1 for site in structure if not site.is_ordered)
             if ndisordered > self.max_disordered_sites:
-                raise ValueError("Too many disordered sites! ({} > {})".format(ndisordered, self.max_disordered_sites))
+                raise ValueError(f"Too many disordered sites! ({ndisordered} > {self.max_disordered_sites})")
             max_cell_sizes = range(
                 self.min_cell_size,
                 int(math.floor(self.max_disordered_sites / ndisordered)) + 1,
@@ -427,7 +426,7 @@ class EnumerateStructureTransformation(AbstractTransformation):
                 if structures:
                     break
             except EnumError:
-                warnings.warn("Unable to enumerate for max_cell_size = {}".format(max_cell_size))
+                warnings.warn(f"Unable to enumerate for max_cell_size = {max_cell_size}")
 
         if structures is None:
             raise ValueError("Unable to enumerate")
@@ -512,7 +511,7 @@ class SubstitutionPredictorTransformation(AbstractTransformation):
             Predicted Structures.
         """
         if not return_ranked_list:
-            raise ValueError("SubstitutionPredictorTransformation doesn't" " support returning 1 structure")
+            raise ValueError("SubstitutionPredictorTransformation doesn't support returning 1 structure")
 
         preds = self._substitutor.composition_prediction(structure.composition, to_this_composition=False)
         preds.sort(key=lambda x: x["probability"], reverse=True)
@@ -690,7 +689,7 @@ class MagOrderingTransformation(AbstractTransformation):
                 # this very hacky bit of code only works because we know
                 # that on disordered sites in this class, all species are the same
                 # but have different spins, and this is comma-delimited
-                sp = str(list(site.species.keys())[0]).split(",")[0]
+                sp = str(list(site.species.keys())[0]).split(",", maxsplit=1)[0]
                 if sp in mag_species_order_parameter:
                     mag_species_occurrences[sp] += 1
                 else:
@@ -753,9 +752,7 @@ class MagOrderingTransformation(AbstractTransformation):
             if satisfies_constraints.count(True) > 1:
                 # site should either not satisfy any constraints, or satisfy
                 # one constraint
-                raise ValueError(
-                    "Order parameter constraints conflict for site: {}, {}".format(str(site.specie), site.properties)
-                )
+                raise ValueError(f"Order parameter constraints conflict for site: {site.specie}, {site.properties}")
             if any(satisfies_constraints):
                 dummy_specie_idx = satisfies_constraints.index(True)
                 dummy_struct.append(dummy_species[dummy_specie_idx], site.coords, site.lattice)
@@ -773,7 +770,7 @@ class MagOrderingTransformation(AbstractTransformation):
             raise Exception("Something went wrong with enumeration.")
 
         sites_to_remove = []
-        logger.debug("Dummy species structure:\n{}".format(str(structure)))
+        logger.debug(f"Dummy species structure:\n{structure}")
         for idx, site in enumerate(structure):
             if isinstance(site.specie, DummySpecies):
                 sites_to_remove.append(idx)
@@ -785,7 +782,7 @@ class MagOrderingTransformation(AbstractTransformation):
                     include_index=True,
                 )
                 if len(neighbors) != 1:
-                    raise Exception("This shouldn't happen, found neighbors: {}".format(neighbors))
+                    raise Exception(f"This shouldn't happen, found neighbors: {neighbors}")
                 orig_site_idx = neighbors[0][2]
                 orig_specie = structure[orig_site_idx].specie
                 new_specie = Species(
@@ -799,7 +796,7 @@ class MagOrderingTransformation(AbstractTransformation):
                     properties=structure[orig_site_idx].properties,
                 )
         structure.remove_sites(sites_to_remove)
-        logger.debug("Structure with dummy species removed:\n{}".format(str(structure)))
+        logger.debug(f"Structure with dummy species removed:\n{structure}")
         return structure
 
     def _add_spin_magnitudes(self, structure):
@@ -818,7 +815,7 @@ class MagOrderingTransformation(AbstractTransformation):
                     # this very hacky bit of code only works because we know
                     # that on disordered sites in this class, all species are the same
                     # but have different spins, and this is comma-delimited
-                    sp = str(site.specie).split(",")[0]
+                    sp = str(site.specie).split(",", maxsplit=1)[0]
                     new_properties.update({"spin": sign * self.mag_species_spin.get(sp, 0)})
                     new_specie = Species(
                         site.specie.symbol,
@@ -826,7 +823,7 @@ class MagOrderingTransformation(AbstractTransformation):
                         new_properties,
                     )
                     structure.replace(idx, new_specie, properties=site.properties)
-        logger.debug("Structure with spin magnitudes:\n{}".format(str(structure)))
+        logger.debug(f"Structure with spin magnitudes:\n{structure}")
         return structure
 
     def apply_transformation(self, structure, return_ranked_list=False):
@@ -838,7 +835,7 @@ class MagOrderingTransformation(AbstractTransformation):
         """
 
         if not structure.is_ordered:
-            raise ValueError("Create an ordered approximation of " "your  input structure first.")
+            raise ValueError("Create an ordered approximation of your  input structure first.")
 
         # retrieve order parameters
         order_parameters = [MagOrderParameterConstraint.from_dict(op_dict) for op_dict in self.order_parameter]
@@ -900,7 +897,7 @@ class MagOrderingTransformation(AbstractTransformation):
             return SpacegroupAnalyzer(x, 0.1).get_space_group_number()
 
         out = []
-        for _, g in groupby(sorted([d["structure"] for d in alls], key=key), key):
+        for _, g in groupby(sorted((d["structure"] for d in alls), key=key), key):
             g = list(g)
             grouped = m.group_structures(g)
             out.extend([{"structure": g[0], "energy": self.energy_model.get_energy(g[0])} for g in grouped])
@@ -1017,7 +1014,7 @@ class DopingTransformation(AbstractTransformation):
             [{"structure": Structure, "energy": float}]
         """
         comp = structure.composition
-        logger.info("Composition: %s" % comp)
+        logger.info(f"Composition: {comp}")
 
         for sp in comp:
             try:
@@ -1052,12 +1049,12 @@ class DopingTransformation(AbstractTransformation):
                 sp for sp in compatible_species if sp in [get_el_sp(s) for s in self.allowed_doping_species]
             ]
 
-        logger.info("Compatible species: %s" % compatible_species)
+        logger.info(f"Compatible species: {compatible_species}")
 
         lengths = structure.lattice.abc
         scaling = [max(1, int(round(math.ceil(self.min_length / x)))) for x in lengths]
-        logger.info("Lengths are %s" % str(lengths))
-        logger.info("Scaling = %s" % str(scaling))
+        logger.info(f"Lengths are {str(lengths)}")
+        logger.info(f"Scaling = {str(scaling)}")
 
         all_structures = []
         t = EnumerateStructureTransformation(**self.kwargs)
@@ -1067,11 +1064,11 @@ class DopingTransformation(AbstractTransformation):
             nsp = supercell.composition[sp]
             if sp.oxi_state == ox:
                 supercell.replace_species({sp: {sp: (nsp - 1) / nsp, self.dopant: 1 / nsp}})
-                logger.info("Doping %s for %s at level %.3f" % (sp, self.dopant, 1 / nsp))
+                logger.info(f"Doping {sp} for {self.dopant} at level {1 / nsp:.3f}")
             elif self.codopant:
                 codopant = _find_codopant(sp, 2 * sp.oxi_state - ox)
                 supercell.replace_species({sp: {sp: (nsp - 2) / nsp, self.dopant: 1 / nsp, codopant: 1 / nsp}})
-                logger.info("Doping %s for %s + %s at level %.3f" % (sp, self.dopant, codopant, 1 / nsp))
+                logger.info(f"Doping {sp} for {self.dopant} + {codopant} at level {1 / nsp:.3f}")
             elif abs(sp.oxi_state) < abs(ox):
                 # Strategy: replace the target species with a
                 # combination of dopant and vacancy.
@@ -1079,7 +1076,7 @@ class DopingTransformation(AbstractTransformation):
                 # vacancy compensation species as it is likely to be lower in
                 # energy
                 sp_to_remove = min(
-                    [s for s in comp if s.oxi_state * ox > 0],
+                    (s for s in comp if s.oxi_state * ox > 0),
                     key=lambda ss: abs(ss.oxi_state),
                 )
 
@@ -1087,7 +1084,7 @@ class DopingTransformation(AbstractTransformation):
                     common_charge = lcm(int(abs(sp.oxi_state)), int(abs(ox)))
                     ndopant = common_charge / abs(ox)
                     nsp_to_remove = common_charge / abs(sp.oxi_state)
-                    logger.info("Doping %d %s with %d %s." % (nsp_to_remove, sp, ndopant, self.dopant))
+                    logger.info(f"Doping {nsp_to_remove} {sp} with {ndopant} {self.dopant}.")
                     supercell.replace_species(
                         {
                             sp: {
@@ -1129,9 +1126,7 @@ class DopingTransformation(AbstractTransformation):
                 common_charge = lcm(anion_ox, ox_diff)
                 ndopant = common_charge / ox_diff
                 nx_to_remove = common_charge / anion_ox
-                logger.info(
-                    "Doping %d %s with %s and removing %d %s." % (ndopant, sp, self.dopant, nx_to_remove, sp_to_remove)
-                )
+                logger.info(f"Doping {ndopant} {sp} with {self.dopant} and removing {nx_to_remove} {sp_to_remove}.")
                 supercell.replace_species(
                     {
                         sp: {sp: (nsp - ndopant) / nsp, self.dopant: ndopant / nsp},
@@ -1140,10 +1135,10 @@ class DopingTransformation(AbstractTransformation):
                 )
 
             ss = t.apply_transformation(supercell, return_ranked_list=self.max_structures_per_enum)
-            logger.info("%s distinct structures" % len(ss))
+            logger.info(f"{len(ss)} distinct structures")
             all_structures.extend(ss)
 
-        logger.info("Total %s doped structures" % len(all_structures))
+        logger.info(f"Total {len(all_structures)} doped structures")
         if return_ranked_list:
             return all_structures[:return_ranked_list]
 
@@ -1361,7 +1356,7 @@ class DisorderOrderedTransformation(AbstractTransformation):
             d = {}  # to be passed to Structure.replace_species()
             for sp_list in partition:
                 if len(sp_list) > 1:
-                    total_occ = sum([composition[sp] for sp in sp_list])
+                    total_occ = sum(composition[sp] for sp in sp_list)
                     merged_comp = {sp: composition[sp] / total_occ for sp in sp_list}
                     for sp in sp_list:
                         d[sp] = merged_comp
@@ -2219,7 +2214,7 @@ class MonteCarloRattleTransformation(AbstractTransformation):
         return transformed_structure
 
     def __str__(self):
-        return "{} : rattle_std = {}".format(__name__, self.rattle_std)
+        return f"{__name__} : rattle_std = {self.rattle_std}"
 
     def __repr__(self):
         return self.__str__()
