@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -18,6 +17,7 @@ import numpy as np
 from monty.json import MSONable
 
 from pymatgen.analysis.ewald import EwaldMinimizer, EwaldSummation
+from pymatgen.analysis.local_env import MinimumDistanceNN
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 from pymatgen.transformations.transformation_abc import AbstractTransformation
 
@@ -68,7 +68,7 @@ class InsertSitesTransformation(AbstractTransformation):
         return s.get_sorted_structure()
 
     def __str__(self):
-        return "InsertSiteTransformation : " + "species {}, coords {}".format(self.species, self.coords)
+        return "InsertSiteTransformation : " + f"species {self.species}, coords {self.coords}"
 
     def __repr__(self):
         return self.__str__()
@@ -119,7 +119,7 @@ class ReplaceSiteSpeciesTransformation(AbstractTransformation):
 
     def __str__(self):
         return "ReplaceSiteSpeciesTransformation :" + ", ".join(
-            ["{}->{}".format(k, v) + v for k, v in self.indices_species_map.items()]
+            [f"{k}->{v}" + v for k, v in self.indices_species_map.items()]
         )
 
     def __repr__(self):
@@ -218,18 +218,12 @@ class TranslateSitesTransformation(AbstractTransformation):
             for i, idx in enumerate(self.indices_to_move):
                 s.translate_sites(idx, self.translation_vector[i], self.vector_in_frac_coords)
         else:
-            s.translate_sites(
-                self.indices_to_move,
-                self.translation_vector,
-                self.vector_in_frac_coords,
-            )
+            s.translate_sites(self.indices_to_move, self.translation_vector, self.vector_in_frac_coords)
         return s
 
     def __str__(self):
         return "TranslateSitesTransformation for indices " + "{}, vect {} and vect_in_frac_coords = {}".format(
-            self.indices_to_move,
-            self.translation_vector,
-            self.vector_in_frac_coords,
+            self.indices_to_move, self.translation_vector, self.vector_in_frac_coords
         )
 
     def __repr__(self):
@@ -325,7 +319,7 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
         starttime = time.time()
         self.logger.debug("Performing initial ewald sum...")
         ewaldsum = EwaldSummation(structure)
-        self.logger.debug("Ewald sum took {} seconds.".format(time.time() - starttime))
+        self.logger.debug(f"Ewald sum took {time.time() - starttime} seconds.")
         starttime = time.time()
 
         ematrix = ewaldsum.total_energy_matrix
@@ -352,7 +346,7 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
             ematrix[maxindex, :] = 0
         s = structure.copy()
         s.remove_sites(to_delete)
-        self.logger.debug("Minimizing Ewald took {} seconds.".format(time.time() - starttime))
+        self.logger.debug(f"Minimizing Ewald took {time.time() - starttime} seconds.")
         return [{"energy": sum(sum(ematrix)), "structure": s.get_sorted_structure()}]
 
     def _complete_ordering(self, structure, num_remove_dict):
@@ -360,13 +354,13 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
         all_structures = []
         symprec = 0.2
         s = SpacegroupAnalyzer(structure, symprec=symprec)
-        self.logger.debug("Symmetry of structure is determined to be {}.".format(s.get_space_group_symbol()))
+        self.logger.debug(f"Symmetry of structure is determined to be {s.get_space_group_symbol()}.")
         sg = s.get_space_group_operations()
         tested_sites = []
         starttime = time.time()
         self.logger.debug("Performing initial ewald sum...")
         ewaldsum = EwaldSummation(structure)
-        self.logger.debug("Ewald sum took {} seconds.".format(time.time() - starttime))
+        self.logger.debug(f"Ewald sum took {time.time() - starttime} seconds.")
         starttime = time.time()
 
         allcombis = []
@@ -398,11 +392,11 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
             count += 1
             if count % 10 == 0:
                 timenow = time.time()
-                self.logger.debug("{} structures, {:.2f} seconds.".format(count, timenow - starttime))
-                self.logger.debug("Average time per combi = {} seconds".format((timenow - starttime) / count))
-                self.logger.debug("{} symmetrically distinct structures found.".format(len(all_structures)))
+                self.logger.debug(f"{count} structures, {timenow - starttime:.2f} seconds.")
+                self.logger.debug(f"Average time per combi = {(timenow - starttime) / count} seconds")
+                self.logger.debug(f"{len(all_structures)} symmetrically distinct structures found.")
 
-        self.logger.debug("Total symmetrically distinct structures found = {}".format(len(all_structures)))
+        self.logger.debug(f"Total symmetrically distinct structures found = {len(all_structures)}")
         all_structures = sorted(all_structures, key=lambda s: s["energy"])
         return all_structures
 
@@ -412,7 +406,7 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
         sums of the potential structures. This is on the order of 4 orders of
         magnitude faster when there are large numbers of permutations to
         consider. There are further optimizations possible (doing a smarter
-        search of permutations for example), but this wont make a difference
+        search of permutations for example), but this won't make a difference
         until the number of permutations is on the order of 30,000.
         """
         self.logger.debug("Performing fast ordering")
@@ -420,20 +414,15 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
         self.logger.debug("Performing initial ewald sum...")
 
         ewaldmatrix = EwaldSummation(structure).total_energy_matrix
-        self.logger.debug("Ewald sum took {} seconds.".format(time.time() - starttime))
+        self.logger.debug(f"Ewald sum took {time.time() - starttime} seconds.")
         starttime = time.time()
         m_list = []
         for indices, num in num_remove_dict.items():
             m_list.append([0, num, list(indices), None])
 
         self.logger.debug("Calling EwaldMinimizer...")
-        minimizer = EwaldMinimizer(
-            ewaldmatrix,
-            m_list,
-            num_to_return,
-            PartialRemoveSitesTransformation.ALGO_FAST,
-        )
-        self.logger.debug("Minimizing Ewald took {} seconds.".format(time.time() - starttime))
+        minimizer = EwaldMinimizer(ewaldmatrix, m_list, num_to_return, PartialRemoveSitesTransformation.ALGO_FAST)
+        self.logger.debug(f"Minimizing Ewald took {time.time() - starttime} seconds.")
 
         all_structures = []
 
@@ -502,7 +491,7 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
         for indices, frac in zip(self.indices, self.fractions):
             num_to_remove = len(indices) * frac
             if abs(num_to_remove - int(round(num_to_remove))) > 1e-3:
-                raise ValueError("Fraction to remove must be consistent with " "integer amounts in structure.")
+                raise ValueError("Fraction to remove must be consistent with integer amounts in structure.")
             num_to_remove = int(round(num_to_remove))
             num_remove_dict[tuple(indices)] = num_to_remove
             n = len(indices)
@@ -510,7 +499,7 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
                 round(math.factorial(n) / math.factorial(num_to_remove) / math.factorial(n - num_to_remove))
             )
 
-        self.logger.debug("Total combinations = {}".format(total_combis))
+        self.logger.debug(f"Total combinations = {total_combis}")
 
         try:
             num_to_return = int(return_ranked_list)
@@ -518,7 +507,7 @@ class PartialRemoveSitesTransformation(AbstractTransformation):
             num_to_return = 1
 
         num_to_return = max(1, num_to_return)
-        self.logger.debug("Will return {} best structures.".format(num_to_return))
+        self.logger.debug(f"Will return {num_to_return} best structures.")
 
         if self.algo == PartialRemoveSitesTransformation.ALGO_FAST:
             all_structures = self._fast_ordering(structure, num_remove_dict, num_to_return)
@@ -590,3 +579,93 @@ class AddSitePropertyTransformation(AbstractTransformation):
     def is_one_to_many(self):
         """Return: False"""
         return False
+
+
+class RadialSiteDistortionTransformation(AbstractTransformation):
+    """
+    Radially perturbs atoms around a site. Can be used to create spherical distortion due to a
+    point defect.
+    """
+
+    def __init__(self, site_index, displacement=0.1, nn_only=False):
+        """
+        Args:
+            site_index (int): index of the site in structure to place at the center of the distortion (will
+                not be distorted). This index must be provided before the structure is provided in
+                apply_transformation in order to keep in line with the base class.
+            displacement (float): distance to perturb the atoms around the objective site
+            nn_only (bool): Whether or not to perturb beyond the nearest neighbors. If True, then only the
+                nearest neighbors will be perturbed, leaving the other sites undisturbed. If False, then
+                the nearest neighbors will receive the full displacement, and then subsequent sites will receive
+                a displacement=0.1 / r, where r is the distance each site to the origin site. For small displacements,
+                atoms beyond the NN environment will receive very small displacements, and these are almost equal.
+                For large displacements, this difference is noticeable.
+        """
+        self.site_index = site_index
+        self.displacement = displacement
+        self.nn_only = nn_only
+
+    def apply_transformation(self, structure):
+        """
+        Apply the transformation.
+
+        Args:
+            structure: Structure or Molecule to apply the transformation to
+
+        Returns:
+            the transformed structure
+        """
+        structure = structure.copy()
+        site = structure[self.site_index]
+
+        def f(x, r, r0):
+            return x * r0 / r
+
+        r0 = max(site.distance(_["site"]) for _ in MinimumDistanceNN().get_nn_info(structure, self.site_index))
+        if hasattr(structure, "lattice"):
+            m = structure.lattice.matrix
+            m = (abs(m) > 1e-5) * m
+            a, b, c = m[0], m[1], m[2]
+            x = abs(np.dot(a, np.cross(b, c)) / np.linalg.norm(np.cross(b, c)))
+            y = abs(np.dot(b, np.cross(a, c)) / np.linalg.norm(np.cross(a, c)))
+            z = abs(np.dot(c, np.cross(a, b)) / np.linalg.norm(np.cross(a, b)))
+            rmax = np.floor(min([x, y, z]) / 2)
+        else:
+            rmax = np.max(structure.distance_matrix)
+
+        for vals in structure.get_neighbors(site, r=r0 if self.nn_only else rmax):
+            site2, distance, index = vals[:3]
+            v = site2.coords - site.coords
+            kwargs = {"indices": [index], "vector": v * f(self.displacement, distance, r0) / np.linalg.norm(v)}
+            if hasattr(structure, "lattice"):
+                kwargs["frac_coords"] = False
+            structure.translate_sites(**kwargs)
+        return structure
+
+    @property
+    def inverse(self):
+        """
+        Returns the inverse transformation if available.
+        Otherwise, should return None.
+        """
+        return False
+
+    @property
+    def is_one_to_many(self):
+        """
+        Determines if a Transformation is a one-to-many transformation. If a
+        Transformation is a one-to-many transformation, the
+        apply_transformation method should have a keyword arg
+        "return_ranked_list" which allows for the transformed structures to be
+        returned as a ranked list.
+        """
+        return False
+
+    @property
+    def use_multiprocessing(self):
+        """
+        Indicates whether the transformation can be applied by a
+        subprocessing pool. This should be overridden to return True for
+        transformations that the transmuter can parallelize.
+        """
+        return False
