@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.import string
 
@@ -409,7 +408,7 @@ def get_weights(xs, ys, mode=2):
             mind = min(abs(d), mind)
         weights = []
         for d in ds:
-            weights.append(abs((mind / d)))
+            weights.append(abs(mind / d))
     if mode == 2:
         maxxs = max(xs) ** 2
         weights = []
@@ -438,27 +437,27 @@ def multi_curve_fit(xs, ys, verbose):
 
     fit_results = {}
     best = ["", np.inf]
-    for function in functions:
+    for k, v in functions.items():
         try:
             weights = get_weights(xs, ys)
             popt, pcov = curve_fit(
-                function,
+                k,
                 xs,
                 ys,
-                functions[function](xs, ys),
+                v(xs, ys),
                 maxfev=8000,
                 sigma=weights,
             )
             pcov = []
-            m = measure(function, xs, ys, popt, weights)
-            fit_results.update({function: {"measure": m, "popt": popt, "pcov": pcov}})
-            for f in fit_results:
-                if fit_results[f]["measure"] <= best[1]:
-                    best = f, fit_results[f]["measure"]
+            m = measure(k, xs, ys, popt, weights)
+            fit_results.update({k: {"measure": m, "popt": popt, "pcov": pcov}})
+            for f, v in fit_results.items():
+                if v["measure"] <= best[1]:
+                    best = f, v["measure"]
             if verbose:
-                print(str(function), m)
+                print(str(k), m)
         except RuntimeError:
-            print("no fit found for ", function)
+            print("no fit found for ", k)
 
     return fit_results[best[0]]["popt"], fit_results[best[0]]["pcov"], best
 
@@ -478,9 +477,9 @@ def multi_reciprocal_extra(xs, ys, noise=False):
         m = measure(reciprocal, xs, ys, popt, weights)
         pcov = []
         fit_results.update({n: {"measure": m, "popt": popt, "pcov": pcov}})
-    for n in fit_results:
-        if fit_results[n]["measure"] <= best[1]:
-            best = reciprocal, fit_results[n]["measure"], n
+    for n, v in fit_results.items():
+        if v["measure"] <= best[1]:
+            best = reciprocal, v["measure"], n
     return fit_results[best[2]]["popt"], fit_results[best[2]]["pcov"], best
 
 
@@ -489,31 +488,26 @@ def print_plot_line(function, popt, xs, ys, name, tol=0.05, extra=""):
     print the gnuplot command line to plot the x, y data with the fitted function using the popt parameters
     """
     idp = id_generator()
-    f = open("convdat." + str(idp), mode="w")
-    for n in range(0, len(ys), 1):
-        f.write(str(xs[n]) + " " + str(ys[n]) + "\n")
-    f.close()
+    with open("convdat." + str(idp), mode="w") as f:
+        for n in range(0, len(ys), 1):
+            f.write(str(xs[n]) + " " + str(ys[n]) + "\n")
     tol = abs(tol)
-    line = "plot 'convdat.%s' pointsize 4 lt 0, " % idp
-    line += "%s lt 3, %s lt 4, %s lt 4, " % (popt[0], popt[0] - tol, popt[0] + tol)
+    line = f"plot 'convdat.{idp}' pointsize 4 lt 0, "
+    line += f"{popt[0]} lt 3, {popt[0] - tol} lt 4, {popt[0] + tol} lt 4, "
     if function is exponential:
-        line += "%s + %s * %s ** -x" % (
-            popt[0],
-            popt[1],
-            min(max(1.00001, popt[2]), 1.2),
-        )
+        line += f"{popt[0]} + {popt[1]} * {min(max(1.00001, popt[2]), 1.2)} ** -x"
     elif function is reciprocal:
-        line += "%s + %s / x**%s" % (popt[0], popt[1], min(max(0.5, popt[2]), 6))
+        line += f"{popt[0]} + {popt[1]} / x**{min(max(0.5, popt[2]), 6)}"
     elif function is single_reciprocal:
-        line += "%s + %s / (x - %s)" % (popt[0], popt[1], popt[2])
+        line += f"{popt[0]} + {popt[1]} / (x - {popt[2]})"
     elif function is simple_reciprocal:
-        line += "%s + %s / x" % (popt[0], popt[1])
+        line += f"{popt[0]} + {popt[1]} / x"
     elif function is simple_2reciprocal:
-        line += "%s + %s / x**2" % (popt[0], popt[1])
+        line += f"{popt[0]} + {popt[1]} / x**2"
     elif function is simple_4reciprocal:
-        line += "%s + %s / x**4" % (popt[0], popt[1])
+        line += f"{popt[0]} + {popt[1]} / x**4"
     elif function is simple_5reciprocal:
-        line += "%s + %s / x**0.5" % (popt[0], popt[1])
+        line += f"{popt[0]} + {popt[1]} / x**0.5"
     else:
         print(function, " no plot ")
 
