@@ -1,12 +1,13 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 """
 Utilities for generating nicer plots.
 """
 import math
+from typing import Literal
 
 import numpy as np
+from matplotlib import cm, colors
 
 from pymatgen.core.periodic_table import Element
 
@@ -40,7 +41,7 @@ def pretty_plot(width=8, height=None, plt=None, dpi=None, color_cycle=("qualitat
 
         import matplotlib.pyplot as plt
 
-        mod = importlib.import_module("palettable.colorbrewer.%s" % color_cycle[0])
+        mod = importlib.import_module(f"palettable.colorbrewer.{color_cycle[0]}")
         colors = getattr(mod, color_cycle[1]).mpl_colors
         from cycler import cycler
 
@@ -172,6 +173,14 @@ def pretty_polyfit_plot(x, y, deg=1, xlabel=None, ylabel=None, **kwargs):
     return plt
 
 
+def _decide_fontcolor(rgba: tuple) -> Literal["black", "white"]:
+    red, green, blue, _ = rgba
+    if (red * 0.299 + green * 0.587 + blue * 0.114) * 255 > 186:
+        return "black"
+
+    return "white"
+
+
 def periodic_table_heatmap(
     elemental_data,
     cbar_label="",
@@ -180,11 +189,15 @@ def periodic_table_heatmap(
     cmap="YlOrRd",
     cmap_range=None,
     blank_color="grey",
+    edge_color="white",
     value_format=None,
+    value_fontsize=10,
+    symbol_fontsize=14,
     max_row=9,
+    readable_fontcolor=False,
 ):
     """
-    A static method that generates a heat map overlayed on a periodic table.
+    A static method that generates a heat map overlaid on a periodic table.
 
     Args:
          elemental_data (dict): A dictionary with the element as a key and a
@@ -194,18 +207,26 @@ def periodic_table_heatmap(
          cbar_label (string): Label of the colorbar. Default is "".
          cbar_label_size (float): Font size for the colorbar label. Default is 14.
          cmap_range (tuple): Minimum and maximum value of the colormap scale.
-            If None, the colormap will autotmatically scale to the range of the
+            If None, the colormap will automatically scale to the range of the
             data.
          show_plot (bool): Whether to show the heatmap. Default is False.
          value_format (str): Formatting string to show values. If None, no value
             is shown. Example: "%.4f" shows float to four decimals.
+         value_fontsize (float): Font size for values. Default is 10.
+         symbol_fontsize (float): Font size for element symbols. Default is 14.
          cmap (string): Color scheme of the heatmap. Default is 'YlOrRd'.
             Refer to the matplotlib documentation for other options.
          blank_color (string): Color assigned for the missing elements in
             elemental_data. Default is "grey".
+         edge_color (string): Color assigned for the edge of elements in the
+            periodic table. Default is "white".
          max_row (integer): Maximum number of rows of the periodic table to be
             shown. Default is 9, which means the periodic table heat map covers
-            the first 9 rows of elements.
+            the standard 7 rows of the periodic table + 2 rows for the lanthanides
+            and actinides. Use a value of max_row = 7 to exclude the lanthanides and
+            actinides.
+         readable_fontcolor (bool): Whether to use readable fontcolor depending
+            on background color. Default is False.
     """
 
     # Convert primitive_elemental data in the form of numpy array for plotting.
@@ -225,10 +246,19 @@ def periodic_table_heatmap(
     blank_value = min_val - 0.01
 
     for el in Element:
-        if el.row > max_row:
-            continue
         value = elemental_data.get(el.symbol, blank_value)
-        value_table[el.row - 1, el.group - 1] = value
+        if 57 <= el.Z <= 71:
+            plot_row = 8
+            plot_group = (el.Z - 54) % 32
+        elif 89 <= el.Z <= 103:
+            plot_row = 9
+            plot_group = (el.Z - 54) % 32
+        else:
+            plot_row = el.row
+            plot_group = el.group
+        if plot_row > max_row:
+            continue
+        value_table[plot_row - 1, plot_group - 1] = value
 
     # Initialize the plt object
     import matplotlib.pyplot as plt
@@ -241,7 +271,7 @@ def periodic_table_heatmap(
     heatmap = ax.pcolor(
         data_mask,
         cmap=cmap,
-        edgecolors="w",
+        edgecolors=edge_color,
         linewidths=1,
         vmin=min_val - 0.001,
         vmax=max_val + 0.001,
@@ -259,18 +289,25 @@ def periodic_table_heatmap(
     ax.axis("off")
     ax.invert_yaxis()
 
+    # Set the scalermap for fontcolor
+    norm = colors.Normalize(vmin=min_val, vmax=max_val)
+    scalar_cmap = cm.ScalarMappable(norm=norm, cmap=cmap)
+
     # Label each block with corresponding element and value
     for i, row in enumerate(value_table):
         for j, el in enumerate(row):
             if not np.isnan(el):
                 symbol = Element.from_row_and_group(i + 1, j + 1).symbol
+                rgba = scalar_cmap.to_rgba(el)
+                fontcolor = _decide_fontcolor(rgba) if readable_fontcolor else "black"
                 plt.text(
                     j + 0.5,
                     i + 0.25,
                     symbol,
                     horizontalalignment="center",
                     verticalalignment="center",
-                    fontsize=14,
+                    fontsize=symbol_fontsize,
+                    color=fontcolor,
                 )
                 if el != blank_value and value_format is not None:
                     plt.text(
@@ -279,7 +316,8 @@ def periodic_table_heatmap(
                         value_format % el,
                         horizontalalignment="center",
                         verticalalignment="center",
-                        fontsize=10,
+                        fontsize=value_fontsize,
+                        color=fontcolor,
                     )
 
     plt.tight_layout()
@@ -328,13 +366,13 @@ def van_arkel_triangle(list_of_materials, annotate=True):
             A.E. van Arkel, Molecules and Crystals in Inorganic Chemistry,
                 Interscience, New York (1956)
         and
-            J.A.A Ketelaar, Chemical Constitution (2nd edn.), An Introduction
+            J.A.A Ketelaar, Chemical Constitution (2nd edition), An Introduction
                 to the Theory of the Chemical Bond, Elsevier, New York (1958)
 
     Args:
          list_of_materials (list): A list of computed entries of binary
             materials or a list of lists containing two elements (str).
-         annotate (bool): Whether or not to lable the points on the
+         annotate (bool): Whether or not to label the points on the
             triangle with reduced formula (if list of entries) or pair
             of elements (if list of list of str).
     """
@@ -592,7 +630,7 @@ def add_fig_kwargs(func):
             if len(fig.axes) > len(tags):
                 tags = (1 + len(ascii_letters) // len(fig.axes)) * ascii_letters
             for ax, tag in zip(fig.axes, tags):
-                ax.annotate("(%s)" % tag, xy=(0.05, 0.95), xycoords="axes fraction")
+                ax.annotate(f"({tag})", xy=(0.05, 0.95), xycoords="axes fraction")
 
         if tight_layout:
             try:
